from .security import Security


class Stock(Security):
    """
    Main class to access data about stocks, inherit from Security class
    Args:
        term (str): text to find a stock, can be a name, part of a name or the isin of the stocks
        pageSize (int): number of stocks to return
        itemRange (int) : index of stocks to return (must be inferior to PageSize)
        proxies = (dict) : set the proxy if needed , example : {"http": "http://host:port","https": "https://host:port"}

    Examples:
        >>> Stocks('0P0000712R', 9, 0)
        >>> Stocks('visa')

    Raises:
        TypeError: raised whenever the parameter type is not the type expected
        ValueError : raised whenever the parameter is not valid or no stock found
    """

    def __init__(self, term = None, exchange: str = "", pageSize : int =1, itemRange: int = 0, filters: dict = {}, proxies: dict = {}):
        
        super().__init__(term=term,asset_type='stock',exchange=exchange,pageSize=pageSize,itemRange=itemRange,filters=filters,proxies=proxies)



    def analysisData(self):
        """
        This function retrieves general data about the stock.

        Returns:
            dict with general data about stock

        Examples:
            >>> Stock("visa", exchange="nyse").analysisData()

            {'ticker': 'V', 'isQuan': False, 'name': 'Visa Inc Class A', 'secId': '0P0000CPCP', 'userType': 'Free', 'valuation': {'fairValue': '_PO_', 'fairValueDate': '2022-06-22T00:00:00.000', 'assessment': '_PO_', 'assessmentDate': '2023-02-20T00:00:00.000', 'uncertainty': '_PO_', 'moat': '_PO_', 'moatDate': '2023-02-20T00:00:00.000', 'moatTrendEvaluate': 'Stable', 'premiumDisc': '_PO_', 'stewardship': '_PO_', 'stewardshipDate': '2023-02-20T00:00:00.000', 'premDiscDelta': '_PO_', 'oneStar': '_PO_', 'oneStarDate': '2023-02-20T00:00:00.000', 'fiveStar': '_PO_', 'fiveStarDate': '2023-02-20T00:00:00.000', 'fairValCurrency': 'USD', 'bf2': '_PO_', 'bf3': '_PO_', 'bf4': '_PO_', 'bf5': '_PO_', 'lastClose': '_PO_', 'startRating': '_PO_', 'lastCloseCurrency': 'USD', 'lastCloseDate': '2023-02-17T06:00:00.000'}, 
            'companyProfile': 'Visa is the largest payment processor in the world. In fiscal 2021, it processed over $14 trillion in total volume. Visa operates in over 200 countries and processes transactions in over 160 currencies. Its systems are capable of processing over 65,000 transactions per second.', 'esgRisk': None}
        """
        return self.GetData("morningstarTake/v3",url_suffixe='analysisData')
    
    def analysisReport(self):
        """
        This function retrieves the analysis of the stock.

        Returns:
            dict with analyst overview

        Examples:
            >>> Stock("visa", exchange="nyse").analysisReport()

            {'userType': 'Free', 'total': 158, 'rpsCovered': True, 'isLocalized': True, 'analysisReport': {'headLine': 'Visa Is a Longtime, Established Market Leader That Still Enjoys Strong Growth Prospects', 'investmentThesis': 'Visa is a somewhat unique company in that it is a longtime, established market leader that still enjoys strong growth prospects. Despite the ongoing evolution of the payments industry, we think a wide moat surrounds the business and that Visa’s position in the global electronic payment infrastructure is essentially unassailable.\n\nThe shift toward e', 'investmentThesisDateUTC': '2022-10-31T15:48:00Z', 'economicMoat': None, 'economicMoatDateUTC': '2022-10-31T15:48:00Z', 'valuation': None, 'valuationDateUTC': '2022-10-31T15:48:00Z', 'risk': None, 'riskDateUTC': '2022-10-31T15:48:00Z', 'management': None, 'managementDateUTC': '2022-10-31T15:48:00Z', 'bullsSay': None, 'bearsSay': None, 'author': {'authorId': 708, 'authorName': 'Brett Horn', 'authorImage': 'https://im.mstar.com/Content/CMSImages/78x78/708-bhorn-78x78.jpg', 'authorImageHeadshot': 'https://im.mstar.com/im/BetaSiteHeadshots/Brett-Horn_708.jpg', 'email': 'brett.horn@morningstar.com', 'phoneNumber': '+1 312 384 5440', 'holdings': 'GIS VIAC WELL WFC', 'jobTitle': None, 'isPrimaryAuthor': True, 'profiles': [{'byLine': 'Brett Horn', 'isPrimaryProfile': True, 'jobTitle': 'Senior Equity Analyst', 'language': 'en-US'}]}}, 'analystNote': {'note': "<xhtml><p>Visa’s fiscal first-quarter results largely mirrored what we saw from its peer Mastercard. Both companies are still enjoying a bit of a boost from the recovery in travel spending, but the impact is fading. Although there have been concerns about a potential macroeconomic downturn 
            and the impact on consumer spending, we think volume has been holding up reasonably well. We will maintain our $229 per share fair value estimate for the wide-moat firm, and we view the shares as being fairly valued at this point.</p><p>Net revenue increased 12% year over year, or 15% excluding currency impacts, during the quarter. Payment volume increased 7% when excluding currency impacts, while transactions grew 10%. Like Mastercard, Visa is seeing volume growth slow, but, in our view, the growth rate remains solid.</p><p>Coming out of the pandemic, cross-border volume has been the biggest driver for the business, due to the outsize fees Visa collects on these transactions, and the exposure to travel spending. Constant-currency cross-border volume excluding intra-Europe transactions—which are priced similarly to domestic transactions—grew 31% year over year in the firm's fiscal first quarter.</p><p>This represents a significant comedown from recent quarters, but that was expected as we moved deeper into the recovery in travel. The cross-border rebound has been a material tailwind for Visa, and that tailwind would eventually diminish. Further, a negative turn in the economy could put the recovery in travel at risk. On the positive side, the reopening of China's borders could be a significant positive on this front.</p><p>Excluding one-time charges, operating margins (based on net revenue) declined modestly to 68.4% from 69.8% last year. Management pointed to increased personnel expenses as the culprit, with the company making some investments for growth.</p></xhtml>", 'title': 'Visa Holds Up Relatively Well in Fiscal Q1 2023', 'date': '2023-01-27T02:17:00Z', 'author': {'authorId': 708, 'authorName': 'Brett Horn', 'authorImage': 'https://im.mstar.com/Content/CMSImages/78x78/708-bhorn-78x78.jpg', 'authorImageHeadshot': 'https://im.mstar.com/im/BetaSiteHeadshots/Brett-Horn_708.jpg', 'email': 'brett.horn@morningstar.com', 'phoneNumber': '+1 312 384 5440', 'holdings': 'GIS VIAC WELL WFC', 'jobTitle': None, 'isPrimaryAuthor': True, 'profiles': [{'byLine': 'Brett Horn', 'isPrimaryProfile': True, 'jobTitle': 'Senior Equity Analyst', 'language': 'en-US'}]}, 'author2': None, 'institutionalNoteContent': None}, 'lastUpdateBy': 708}
        """
        return self.GetData("morningstarTake/v4",url_suffixe='analysisReport')
    
    def balanceSheet(self, period='annual', reportType='original'):
        """
        This function retrieves the balance sheet.

        Args:
            period (str) : possible values are annual, quarterly
            reportType (str) : possible values are original, restated

        Returns:
            dict with balance sheet

        Examples:
            >>> Stock("visa", exchange="nyse").balanceSheet('quarterly', 'original')
            >>> Stock("visa", exchange="nyse").balanceSheet('annual', 'restated')

        {'_meta': {'companyId': '0C00000VF2', 'statementType': 'balance-sheet', 'periodReport': 'Success', 'latestReport': 'no data available'}, 'columnDefs': ['Q1 2018', 'Q2 2018', 'Q3 2018', 'Q4 2018', 'Q1 2019', 'Q2 2019', 'Q3 2019', 'Q4 2019', 'Q1 2020', 'Q2 2020', 'Q3 2020', 'Q4 2020', 'Q1 
        2021', 'Q2 2021', 'Q3 2021', 'Q4 2021', 'Q1 2022', 'Q2 2022', 'Q3 2022', 'Q4 2022', 'Q1 2023'], 'filingIdList': [None, None, None, None, None, '219784291', '231424262', '251173203', '262306702', '277085675', '290998262', '309018162', '319266934', '333897070', '347849551', '364101718', '374708643', '388626196', '401773298', '417618183', '429622549'], 'rows': [{'parentId': '', 'label': 'BalanceSheet', 'dataPointId': 'IFBS000000', 
        'order': 0, 'subLevel': [{'parentId': 'IFBS0000000', 'label': 'Total Assets', 'dataPointId': 'IFBS002270', 'order': 1, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 80919.0, 80426.0, 80195.0, 82671.0, 82896.0, 81929.0, 81808.0, 85410.0, 85501.0, 85387.0], 'subLevel': [{'parentId': 'IFBS0022701', 'label': 'Total Current Assets', 'dataPointId': 'IFBS000470', 'order': 2, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 27645.0, 26280.0, 26901.0, 28042.0, 27607.0, 25906.0, 24091.0, 28971.0, 30205.0, 27859.0], 'subLevel': [{'parentId': 'IFBS0004702', 'label': 'Cash, Cash Equivalents and Short Term Investments', 'dataPointId': 'IFBS000350', 
        'order': 3, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 20041.0, 18041.0, 18552.0, 19240.0, 18512.0, 15921.0, 13529.0, 17356.0, 18522.0, 16119.0], 'subLevel': [{'parentId': 'IFBS0003503', 'label': 'Cash and Cash Equivalents', 'dataPointId': 'IFBS000320', 'order': 4, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 16289.0, 15032.0, 16494.0, 18034.0, 16487.0, 14720.0, 12299.0, 14047.0, 15689.0, 13334.0]}, {'parentId': 'IFBS0003503', 'label': 'Short Term Investments', 'dataPointId': 'IFBS001690', 'order': 7, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 3752.0, 3009.0, 2058.0, 1206.0, 2025.0, 1201.0, 1230.0, 3309.0, 2833.0, 2785.0], 'subLevel': [{'parentId': 'IFBS0016907', 'label': 'Other Short Term Investments', 'dataPointId': 'IFBS002712', 'order': 11, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, None, None, 1201.0, 1230.0, 3309.0, None, 2785.0]}, {'parentId': 'IFBS0016907', 'label': 'Trading/ Listed Investments, Current', 'dataPointId': 'IFBS002677', 'order': 8, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None]}, {'parentId': 'IFBS0016907', 'label': 'Available-for-Sale Securities, Current', 'dataPointId': 'IFBS002880', 'order': 9, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None]}]}]}, {'parentId': 'IFBS0004702', 'label': 'Cash Restricted or Pledged, Current', 'dataPointId': 'IFBS002607', 'order': 12, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 2751.0, 2887.0, 2951.0, 3115.0, 3154.0, 3428.0, 3191.0, 3744.0, 3791.0, 4296.0]}, {'parentId': 'IFBS0004702', 'label': 'Trade and Other Receivables, Current', 'dataPointId': 'IFBS200660', 'order': 22, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 2882.0, 3342.0, 3225.0, 3515.0, 3726.0, 3956.0, 3767.0, 3881.0, 3952.0, 4240.0], 'subLevel': [{'parentId': 'IFBS20066022', 'label': 'Trade/Accounts Receivable, Current', 'dataPointId': 'IFBS000020', 'order': 23, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 1618.0, 1746.0, 1791.0, 1852.0, 1968.0, 2168.0, 2135.0, 2021.0, 2020.0, 2113.0]}, {'parentId': 'IFBS20066022', 'label': 'Other Receivables, Current', 'dataPointId': 'IFBS001680', 'order': 36, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 1264.0, 1596.0, 1434.0, 1663.0, 1758.0, 1788.0, 1632.0, 1860.0, 1932.0, 2127.0]}]}, {'parentId': 'IFBS0004702', 'label': 'Other Current Assets', 'dataPointId': 'IFBS001550', 'order': 43, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 1214.0, 1281.0, 1324.0, 1321.0, 1359.0, 1334.0, 1309.0, 1323.0, 1272.0, 3204.0]}, {'parentId': 'IFBS0004702', 'label': 'Prepayments and Deposits, Current', 'dataPointId': 'IFBS100688', 'order': 38, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 757.0, 729.0, 849.0, 851.0, 856.0, 1267.0, 2295.0, 2667.0, 2668.0, None]}]}, {'parentId': 'IFBS0022701', 'label': 'Total Non-Current Assets', 
        'dataPointId': 'IFBS002330', 'order': 44, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 53274.0, 54146.0, 53294.0, 54629.0, 55289.0, 56023.0, 57717.0, 56439.0, 55296.0, 57528.0], 'subLevel': [{'parentId': 'IFBS00233044', 'label': 'Net Property, Plant and Equipment', 'dataPointId': 'IFBS001450', 'order': 45, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 2737.0, 2713.0, 2684.0, 2707.0, 2715.0, 2908.0, 3120.0, 3146.0, 3223.0, 3236.0], 'subLevel': [{'parentId': 'IFBS00145045', 'label': 'Gross Property, Plant and Equipment', 'dataPointId': 'IFBS001050', 'order': 46, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 7161.0, None, None, None, 7674.0, None, None, None, 8881.0, None], 'subLevel': [{'parentId': 'IFBS00105046', 'label': 'Properties', 'dataPointId': 'IFBS001880', 'order': 47, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 1078.0, None, 
        None, None, 1080.0, None, None, None, 1075.0, None], 'subLevel': [{'parentId': 'IFBS00188047', 'label': 'Land and Improvements', 'dataPointId': 
        'IFBS001200', 'order': 48, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 71.0, None, 
        None, None, 72.0, None, None, None, 72.0, None]}, {'parentId': 'IFBS00188047', 'label': 'Buildings and Improvements', 'dataPointId': 'IFBS000280', 'order': 49, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 1007.0, None, None, None, 1008.0, None, None, None, 1003.0, None]}]}, {'parentId': 'IFBS00105046', 'label': 'Machinery, Furniture and Equipment', 'dataPointId': 'IFBS001340', 'order': 52, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 5920.0, None, None, None, 6368.0, None, None, None, 7521.0, None], 'subLevel': [{'parentId': 'IFBS00134052', 'label': 'Furniture, Fixtures and Office Equipment', 'dataPointId': 'IFBS002651', 'order': 54, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 5920.0, None, None, None, 6368.0, None, None, None, 7521.0, None]}]}, {'parentId': 'IFBS00105046', 'label': 'Construction in Progress and Advance Payments', 'dataPointId': 'IFBS000430', 'order': 62, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 
        163.0, None, None, None, 226.0, None, None, None, 285.0, None]}]}, {'parentId': 'IFBS00145045', 'label': 'Accumulated Depreciation and Impairment', 'dataPointId': 'IFBS002567', 'order': 65, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -4424.0, None, None, None, -4959.0, None, None, None, -5658.0, None], 'subLevel': [{'parentId': 'IFBS00256765', 'label': 'Accumulated Depreciation', 'dataPointId': 'IFBS000090', 'order': 66, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -4424.0, None, None, None, -4959.0, None, None, None, -5658.0, None]}]}]}, {'parentId': 'IFBS00233044', 'label': 'Net Intangible Assets', 'dataPointId': 'IFBS001020', 'order': 92, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 43718.0, 44714.0, 43803.0, 44044.0, 43622.0, 43827.0, 45149.0, 44070.0, 42852.0, 44331.0], 'subLevel': [{'parentId': 'IFBS00102092', 'label': 'Gross Goodwill and Other Intangible Assets', 'dataPointId': 'IFBS002655', 'order': 93, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 44331.0, 44714.0, 43803.0, 44044.0, 44317.0, 43827.0, 45149.0, 44070.0, 43635.0, 44331.0], 'subLevel': [{'parentId': 'IFBS00265593', 'label': 
        'Goodwill', 'dataPointId': 'IFBS001010', 'order': 94, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 15910.0, 16121.0, 15980.0, 16021.0, 15958.0, 16555.0, 18143.0, 17977.0, 17787.0, 18024.0]}, {'parentId': 'IFBS00265593', 'label': 'Intangibles other than Goodwill', 'dataPointId': 'IFBS003302', 'order': 95, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 28421.0, 28593.0, 27823.0, 28023.0, 28359.0, 27272.0, 27006.0, 26093.0, 25848.0, 26307.0], 'subLevel': [{'parentId': 'IFBS00330295', 'label': 'Trademarks and Patents', 'dataPointId': 'IFBS002771', 'order': 96, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 4283.0, None, None, None, 4283.0, None, None, None, 4279.0, None]}, {'parentId': 'IFBS00330295', 
        'label': 'Customer Relationships', 'dataPointId': 'IFBS003303', 'order': 101, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 24026.0, None, None, None, 23965.0, None, None, None, 21458.0, None]}, {'parentId': 'IFBS00330295', 'label': 'Other Intangible Assets', 'dataPointId': 'IFBS001610', 'order': 102, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 112.0, None, None, None, 111.0, None, None, None, 111.0, None]}]}]}, {'parentId': 'IFBS00102092', 'label': 'Accumulated Amortization and Impairment', 'dataPointId': 'IFBS002549', 'order': 103, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -613.0, None, None, None, -695.0, None, None, None, -783.0, None], 'subLevel': [{'parentId': 'IFBS002549103', 'label': 'Accumulated Amortization of Intangible Assets', 'dataPointId': 'IFBS002551', 'order': 104, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -613.0, None, None, None, -695.0, None, None, None, -783.0, None], 'subLevel': [{'parentId': 'IFBS002551104', 'label': 'Accumulated Amortization of Intangibles other than Goodwill', 'dataPointId': 'IFBS003304', 'order': 106, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -613.0, None, None, None, -695.0, None, None, None, -783.0, None], 'subLevel': [{'parentId': 'IFBS003304106', 'label': 'Accumulated Amortization of Trademarks and Patents', 'dataPointId': 'IFBS002556', 'order': 107, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -134.0, None, None, None, -148.0, None, None, None, -159.0, None]}, {'parentId': 'IFBS003304106', 'label': 'Accumulated Amortization of Customer Relationships', 'dataPointId': 'IFBS003305', 'order': 112, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -376.0, None, None, None, -440.0, None, None, None, -513.0, None]}, {'parentId': 'IFBS003304106', 'label': 'Accumulated Amortization of Other Intangible Assets', 'dataPointId': 'IFBS002553', 'order': 113, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -103.0, None, None, None, -107.0, None, None, None, -111.0, None]}]}]}]}]}, {'parentId': 'IFBS00233044', 'label': 'Total Long Term Investments', 'dataPointId': 'IFBS002310', 'order': 121, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 231.0, 161.0, 110.0, 1111.0, 1705.0, 2087.0, 2296.0, 2240.0, 2136.0, 2735.0], 'subLevel': [{'parentId': 'IFBS002310121', 'label': 'Investment in Financial Assets, Non-Current', 'dataPointId': 'IFBS100685', 'order': 127, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 231.0, 161.0, 110.0, 1111.0, 1705.0, 2087.0, 2296.0, 2240.0, 2136.0, 2735.0], 'subLevel': [{'parentId': 'IFBS100685127', 'label': 'Financial Investment Securities, Available-for-Sale & Held-to-Maturity, Non-Current', 'dataPointId': 'IFBS002642', 'order': 132, 'datum': ['_PO_', '_PO_', '_PO_', 
        '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None], 'subLevel': [{'parentId': 'IFBS002642132', 'label': 'Available-for-Sale Securities, Non-Current', 'dataPointId': 'IFBS000220', 'order': 133, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None]}]}]}]}, {'parentId': 'IFBS00233044', 'label': 'Other Non-Current Assets', 'dataPointId': 'IFBS001650', 'order': 158, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 6588.0, 6558.0, 6697.0, 6767.0, 7247.0, 7201.0, 7152.0, 6983.0, 7085.0, 7226.0]}]}]}, {'parentId': 'IFBS0000000', 'label': 'Total Liabilities', 'dataPointId': 'IFBS002646', 'order': 159, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 44709.0, 42747.0, 42499.0, 44881.0, 45307.0, 45735.0, 45819.0, 49927.0, 49920.0, 48463.0], 'subLevel': [{'parentId': 'IFBS002646159', 'label': 'Total Current Liabilities', 'dataPointId': 'IFBS000500', 'order': 160, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 14510.0, 12390.0, 12711.0, 14023.0, 15739.0, 18568.0, 18702.0, 20309.0, 20853.0, 
        19353.0], 'subLevel': [{'parentId': 'IFBS000500160', 'label': 'Payables and Accrued Expenses, Current', 'dataPointId': 'IFBS002716', 'order': 161, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 4664.0, 5318.0, 5100.0, 5644.0, 6026.0, 7018.0, 6532.0, 8085.0, 8803.0, 9473.0], 'subLevel': [{'parentId': 'IFBS002716161', 'label': 'Trade and Other Payables, Current', 'dataPointId': 'IFBS001710', 
        'order': 162, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 1910.0, 2179.0, 2126.0, 2548.0, 2709.0, 3026.0, 2591.0, 3296.0, 3621.0, 3831.0], 'subLevel': [{'parentId': 'IFBS001710162', 'label': 'Trade/Accounts Payable, Current', 'dataPointId': 'IFBS000010', 'order': 163, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 174.0, 131.0, 126.0, 172.0, 266.0, 252.0, 182.0, 228.0, 340.0, 258.0]}, {'parentId': 'IFBS001710162', 'label': 'Other Payable, Current', 'dataPointId': 'IFBS002540', 'order': 171, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 1736.0, 2048.0, 2000.0, 2376.0, 2443.0, 2774.0, 2409.0, 3068.0, 3281.0, 3573.0]}]}, {'parentId': 'IFBS002716161', 'label': 'Accrued Expenses, Current', 'dataPointId': 'IFBS002541', 'order': 172, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 2754.0, 3139.0, 2974.0, 3096.0, 3317.0, 3992.0, 3941.0, 4789.0, 5182.0, 5642.0]}]}, {'parentId': 'IFBS000500160', 'label': 'Financial Liabilities, Current', 'dataPointId': 'IFBS100734', 'order': 173, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 2999.0, 0.0, 0.0, 0.0, 999.0, 3247.0, 3548.0, 3249.0, 2250.0, 0.0], 'subLevel': [{'parentId': 'IFBS100734173', 'label': 'Current Debt and Capital Lease Obligation', 'dataPointId': 'IFBS002965', 'order': 174, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 2999.0, 0.0, 0.0, 0.0, 999.0, 3247.0, 3548.0, 3249.0, 2250.0, 0.0], 'subLevel': [{'parentId': 'IFBS002965174', 'label': 'Current Portion of Long Term Debt and Capital Lease', 'dataPointId': 'IFBS003330', 'order': 183, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, 0.0, 0.0, 999.0, 3247.0, 3548.0, 3249.0, 2250.0, 0.0], 'subLevel': [{'parentId': 'IFBS003330183', 'label': 'Current Portion of Long Term Debt', 'dataPointId': 'IFBS002617', 'order': 184, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, 0.0, 0.0, 999.0, 3247.0, 3548.0, 3249.0, 2250.0, 0.0], 'subLevel': [{'parentId': 'IFBS002617184', 'label': 'Other Current 
        Portion of LT Debt', 'dataPointId': 'IFBS002703', 'order': 189, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, 0.0, None, 999.0, 3247.0, 3548.0, 3249.0, None, 0.0]}]}]}]}]}, {'parentId': 'IFBS000500160', 'label': 'Provisions, Current', 'dataPointId': 'IFBS001890', 'order': 193, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 821.0, 600.0, 794.0, 1044.0, 1211.0, 725.0, 877.0, 1106.0, 1359.0, 736.0], 'subLevel': [{'parentId': 'IFBS001890193', 'label': 'Provision for Employee Entitlements, Current', 'dataPointId': 'IFBS003035', 'order': 194, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 821.0, 600.0, 794.0, 1044.0, 1211.0, 725.0, 877.0, 1106.0, 1359.0, 736.0]}]}, {'parentId': 'IFBS000500160', 'label': 'Other Current Liabilities', 'dataPointId': 'IFBS001570', 'order': 208, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 6026.0, 6472.0, 6817.0, 7335.0, 7503.0, 7578.0, 7745.0, 7869.0, 8441.0, 9144.0]}, {'parentId': 'IFBS000500160', 'label': 'Deferred Liabilities, Current', 'dataPointId': 'IFBS000610', 'order': 203, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None], 'subLevel': [{'parentId': 'IFBS000610203', 'label': 'Other Deferred Liabilities, Current', 'dataPointId': 'IFBS002854', 'order': 205, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None]}]}]}, {'parentId': 'IFBS002646159', 'label': 'Total 
        Non-Current Liabilities', 'dataPointId': 'IFBS002647', 'order': 209, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 30199.0, 30357.0, 29788.0, 30858.0, 29568.0, 27167.0, 27117.0, 29618.0, 29067.0, 29110.0], 'subLevel': [{'parentId': 'IFBS002647209', 'label': 'Financial Liabilities, Non-Current', 'dataPointId': 'IFBS100735', 'order': 210, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 21071.0, 21055.0, 20974.0, 20996.0, 19978.0, 17673.0, 17479.0, 20546.0, 20200.0, 20487.0], 'subLevel': [{'parentId': 'IFBS100735210', 'label': 'Long Term Debt and Capital Lease Obligation', 'dataPointId': 'IFBS002961', 'order': 211, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 21071.0, 21055.0, 20974.0, 20835.0, 19978.0, 17673.0, 17479.0, 20546.0, 20200.0, 20487.0], 'subLevel': [{'parentId': 'IFBS002961211', 'label': 'Long Term Debt', 'dataPointId': 'IFBS001290', 'order': 212, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 21071.0, 21055.0, 20974.0, 20835.0, 19978.0, 17673.0, 17479.0, 20546.0, 20200.0, 20487.0], 'subLevel': [{'parentId': 'IFBS001290212', 'label': 'Notes Payables, Non-Current', 'dataPointId': 'IFBS002695', 'order': 213, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 21071.0, 21055.0, 20974.0, 20835.0, 19978.0, 17673.0, 17479.0, 20546.0, 20200.0, 20487.0]}]}]}, {'parentId': 'IFBS100735210', 'label': 'Derivative and Hedging Liabilities, Non-Current', 'dataPointId': 'IFBS100699', 'order': 221, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, 161.0, None, None, None, None, None, None]}]}, {'parentId': 'IFBS002647209', 'label': 'Tax Liabilities, Non-Current', 'dataPointId': 'IFBS003143', 'order': 230, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 5237.0, 5343.0, 5212.0, 6240.0, 6128.0, 6078.0, 6081.0, 5685.0, 5332.0, 5443.0], 'subLevel': [{'parentId': 'IFBS003143230', 'label': 'Deferred Tax Liabilities, Non-Current', 'dataPointId': 'IFBS000710', 'order': 231, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 5237.0, 5343.0, 5212.0, 6240.0, 6128.0, 6078.0, 6081.0, 5685.0, 5332.0, 5443.0]}]}, {'parentId': 'IFBS002647209', 'label': 'Other Non-Current Liabilities', 'dataPointId': 'IFBS002480', 'order': 247, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 3891.0, 3959.0, 3602.0, 3622.0, 3462.0, 3416.0, 3557.0, 3387.0, 3535.0, 3180.0]}, {'parentId': 'IFBS002647209', 'label': 'Deferred Liabilities, Non-Current', 'dataPointId': 'IFBS002805', 'order': 233, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None], 'subLevel': [{'parentId': 'IFBS002805233', 'label': 'Other Deferred Liabilities, Non-Current', 'dataPointId': 'IFBS002855', 'order': 235, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', 
        '_PO_', None, None, None, None, None, None, None, None, None, None, None, None, None, None, None, None]}]}]}]}, {'parentId': 'IFBS0000000', 'label': 'Total Equity', 'dataPointId': 'IFBS002220', 'order': 249, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, 
        None, 36210.0, 37679.0, 37696.0, 37790.0, 37589.0, 36194.0, 35989.0, 35483.0, 35581.0, 36924.0], 'subLevel': [{'parentId': 'IFBS002220249', 'label': 'Equity Attributable to Parent Stockholders', 'dataPointId': 'IFBS200140', 'order': 250, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 
        None, None, None, None, None, None, 36210.0, 37679.0, 37696.0, 37790.0, 37589.0, 36194.0, 35989.0, 35483.0, 35581.0, 36924.0], 'subLevel': [{'parentId': 'IFBS200140250', 'label': 'Paid in Capital', 'dataPointId': 'IFBS003308', 'order': 251, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 21807.0, 21746.0, 21852.0, 21919.0, 21935.0, 21771.0, 21863.0, 21813.0, 21869.0, 21808.0], 'subLevel': [{'parentId': 'IFBS003308251', 'label': 'Capital Stock', 'dataPointId': 'IFBS000290', 'order': 252, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 21807.0, 21746.0, 21852.0, 21919.0, 21935.0, 21771.0, 21863.0, 21813.0, 21869.0, 21808.0], 'subLevel': [{'parentId': 'IFBS000290252', 'label': 'Common Stock', 'dataPointId': 'IFBS000400', 'order': 253, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 0.0, None, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, None, None]}, {'parentId': 'IFBS000290252', 'label': 'Preferred Stock', 'dataPointId': 'IFBS001800', 'order': 256, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 5086.0, 3683.0, 3347.0, 3132.0, 3080.0, 2995.0, 2987.0, 2851.0, None, None]}, {'parentId': 'IFBS000290252', 'label': 'Additional Paid in Capital/Share Premium', 'dataPointId': 'IFBS000140', 'order': 257, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 16721.0, 18063.0, 18505.0, 18787.0, 18855.0, 18776.0, 18876.0, 18962.0, None, None]}]}]}, {'parentId': 'IFBS200140250', 'label': 'Retained Earnings/Accumulated Deficit', 'dataPointId': 'IFBS002100', 'order': 266, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 14088.0, 14813.0, 15513.0, 15294.0, 15351.0, 14606.0, 14651.0, 14960.0, 16116.0, 16403.0]}, {'parentId': 'IFBS200140250', 'label': 'Reserves/Accumulated Comprehensive Income/Losses', 'dataPointId': 'IFBS002737', 'order': 267, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 354.0, 1154.0, 372.0, 601.0, 436.0, -72.0, -405.0, -1267.0, -2369.0, -1259.0], 'subLevel': [{'parentId': 'IFBS002737267', 'label': 'Cumulative Foreign Exchange Translation Reserves/Accum.Comp.Inc', 'dataPointId': 'IFBS002559', 'order': 272, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 838.0, 1884.0, 873.0, 1160.0, 743.0, 155.0, -180.0, -1280.0, -2512.0, -785.0]}, {'parentId': 'IFBS002737267', 'label': ' Available for Sale Financial Investments Reserves/Accum.Comp.Inc', 'dataPointId': 'IFBS002558', 'order': 273, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 3.0, 2.0, 1.0, -1.0, -1.0, -9.0, -41.0, -74.0, -106.0, -94.0]}, {'parentId': 'IFBS002737267', 
        'label': 'Gain/Loss from Cash Flow Hedges Reserves/Accum.Comp.Inc', 'dataPointId': 'IFBS002560', 'order': 274, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -291.0, -538.0, -310.0, -371.0, -257.0, -171.0, -136.0, 135.0, 418.0, -213.0]}, {'parentId': 'IFBS002737267', 'label': 'Pension and Other PostRetirement Benefit Plans Reserves/Accum.Comp.Inc', 'dataPointId': 'IFBS002563', 'order': 275, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -196.0, -194.0, -192.0, -187.0, -49.0, -47.0, -48.0, -48.0, -169.0, -167.0]}]}, {'parentId': 'IFBS200140250', 'label': 'Other Equity Interest', 'dataPointId': 'IFBS200130', 'order': 277, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -39.0, -34.0, -41.0, -24.0, -133.0, -111.0, -120.0, -23.0, -35.0, -28.0]}]}]}, {'parentId': 'IFBS0000000', 'label': 'Debt Maturity Schedule Total', 'dataPointId': 'DMBS000001', 'order': 337, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 24070.0, None, None, None, 20977.0, None, None, None, 22450.0, None], 'subLevel': [{'parentId': 'DMBS000001337', 'label': 'Debt due in Year 1', 'dataPointId': 'DMBS000002', 'order': 338, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 3000.0, None, None, None, 1000.0, None, None, None, 2250.0, None]}, {'parentId': 'DMBS000001337', 'label': 'Debt due in Year 4', 'dataPointId': 'DMBS000005', 'order': 341, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, None, None, None, None, None, 5325.0, None]}, {'parentId': 'DMBS000001337', 'label': 'Debt due in Year 5', 'dataPointId': 'DMBS000006', 'order': 342, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, None, None, None, None, 4000.0, None, None, None, 2750.0, None]}, {'parentId': 'DMBS000001337', 'label': 'Debt due Beyond', 'dataPointId': 'DMBS000007', 'order': 343, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 17750.0, None, None, None, 13750.0, None, None, None, 12620.0, None]}, {'parentId': 'DMBS000001337', 'label': 'Debt - Interests Charges and Other Adjustments', 'dataPointId': 'DMBS000008', 'order': 344, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 70.0, None, None, None, -23.0, None, None, None, -495.0, None]}, {'parentId': 'DMBS000001337', 'label': 'Debt due in Year 2', 'dataPointId': 'DMBS000003', 'order': 339, 'collapsed': True, 
        'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 1000.0, None, None, None, 2250.0, None, None, None, None, 
        None]}, {'parentId': 'DMBS000001337', 'label': 'Debt due in Year 3', 'dataPointId': 'DMBS000004', 'order': 340, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 2250.0, None, None, None, None, None, None, None, None, None]}]}, {'parentId': 'IFBS0000000', 'label': 'Operating Lease Obligation Maturity Schedule Total', 'dataPointId': 'DMBS000017', 'order': 353, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 570.0, None, None, None, 574.0, None, None, None, 520.0, None], 'subLevel': [{'parentId': 'DMBS000017353', 'label': 'Operating Lease due in Year 1', 'dataPointId': 'DMBS000018', 'order': 354, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 108.0, None, None, None, 120.0, None, None, None, 102.0, None]}, {'parentId': 'DMBS000017353', 'label': 'Operating Lease due in Year 2', 'dataPointId': 'DMBS000019', 'order': 355, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 103.0, None, None, None, 108.0, None, None, None, 107.0, None]}, {'parentId': 'DMBS000017353', 'label': 'Operating Lease due in Year 3', 'dataPointId': 'DMBS000020', 'order': 356, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 95.0, None, None, None, 100.0, None, None, None, 91.0, None]}, {'parentId': 'DMBS000017353', 'label': 'Operating Lease due in Year 4', 'dataPointId': 'DMBS000021', 'order': 357, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 82.0, None, None, None, 85.0, None, None, None, 78.0, None]}, {'parentId': 'DMBS000017353', 'label': 'Operating Lease due in Year 5', 'dataPointId': 'DMBS000022', 'order': 358, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 70.0, None, None, None, 75.0, None, None, None, 58.0, None]}, {'parentId': 'DMBS000017353', 'label': 'Operating Lease due Beyond', 'dataPointId': 'DMBS000023', 'order': 359, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 163.0, None, None, None, 132.0, None, None, None, 121.0, None]}, {'parentId': 'DMBS000017353', 'label': 'Operating Lease - Interests Charges and Other Adjustments', 'dataPointId': 'DMBS000024', 'order': 360, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -51.0, None, None, None, -46.0, None, None, None, -37.0, None]}]}, {'parentId': 'IFBS0000000', 'label': 'Total Lease Liability', 'dataPointId': 'DMBS000033', 'order': 369, 'datum': ['_PO_', '_PO_', 
        '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 570.0, None, None, None, 574.0, None, None, None, 520.0, None], 'subLevel': [{'parentId': 'DMBS000033369', 'label': 'Total Lease Liability - Due in year 1', 'dataPointId': 'DMBS000034', 'order': 370, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 108.0, None, None, None, 120.0, None, None, None, 102.0, None]}, {'parentId': 'DMBS000033369', 'label': 'Total Lease Liability - Due in year 2', 'dataPointId': 'DMBS000035', 'order': 371, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', 
        '_PO_', None, None, None, None, None, None, 103.0, None, None, None, 108.0, None, None, None, 107.0, None]}, {'parentId': 'DMBS000033369', 'label': 'Total Lease Liability - Due in year 3', 'dataPointId': 'DMBS000036', 'order': 372, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, 
        None, None, None, None, None, 95.0, None, None, None, 100.0, None, None, None, 91.0, None]}, {'parentId': 'DMBS000033369', 'label': 'Total Lease Liability - Due in year 4', 'dataPointId': 'DMBS000037', 'order': 373, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 82.0, None, None, None, 85.0, None, None, None, 78.0, None]}, {'parentId': 'DMBS000033369', 'label': 'Total Lease Liability - Due 
        in year 5', 'dataPointId': 'DMBS000038', 'order': 374, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 70.0, None, None, None, 75.0, None, None, None, 58.0, None]}, {'parentId': 'DMBS000033369', 'label': 'Total Lease Liability - Beyond', 'dataPointId': 'DMBS000039', 'order': 375, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 163.0, None, None, None, 132.0, None, None, None, 121.0, None]}, {'parentId': 'DMBS000033369', 'label': 'Total Lease Liability - Interest Charges and Other Adjustments', 'dataPointId': 'DMBS000040', 'order': 376, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, -51.0, None, None, None, -46.0, None, None, None, -37.0, None]}]}, {'parentId': 'IFBS0000000', 'label': 'Total Contractual Obligations', 'dataPointId': 'DMBS000041', 'order': 377, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 29636.0, None, None, None, 25710.0, None, None, None, 22970.0, None], 'subLevel': [{'parentId': 'DMBS000041377', 'label': 'Total Contractual Obligations due in year 1', 'dataPointId': 'DMBS000042', 'order': 378, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 5438.0, None, None, None, 2937.0, None, None, None, 2352.0, None]}, {'parentId': 'DMBS000041377', 'label': 'Total Contractual Obligations due in year 2', 'dataPointId': 'DMBS000043', 'order': 379, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 1103.0, None, None, None, 2358.0, None, None, None, 107.0, None]}, {'parentId': 'DMBS000041377', 'label': 'Total Contractual Obligations due in year 3', 'dataPointId': 'DMBS000044', 'order': 380, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 3253.0, None, None, None, 1034.0, None, None, None, 91.0, None]}, {'parentId': 'DMBS000041377', 'label': 'Total Contractual Obligations due in year 4', 'dataPointId': 'DMBS000045', 'order': 381, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 82.0, None, None, None, 85.0, None, None, None, 5403.0, None]}, {'parentId': 'DMBS000041377', 'label': 'Total Contractual Obligations due in year 5', 'dataPointId': 'DMBS000046', 'order': 382, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 852.0, None, None, None, 4914.0, None, None, None, 2808.0, None]}, {'parentId': 'DMBS000041377', 'label': 'Total Contractual Obligations due Beyond', 'dataPointId': 'DMBS000047', 'order': 383, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 18889.0, None, None, None, 14451.0, None, None, None, 12741.0, None]}, {'parentId': 'DMBS000041377', 'label': 'Total Contractual Obligations - Interests Charges and Other Adjustments', 'dataPointId': 'DMBS000048', 'order': 384, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 19.0, None, None, None, -69.0, None, None, None, -532.0, None]}]}, {'parentId': 'IFBS0000000', 'label': 'Other Contractual Obligations Maturity Schedule Total', 'dataPointId': 'DMBS000025', 'order': 361, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 4996.0, None, None, None, 4159.0, None, None, None, None, None], 'subLevel': [{'parentId': 'DMBS000025361', 'label': 'Other Contractual Obligations due in Year 1', 'dataPointId': 'DMBS000026', 'order': 362, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 2330.0, None, None, None, 1817.0, None, None, None, None, None]}, {'parentId': 'DMBS000025361', 'label': 'Other Contractual Obligations due in Year 3', 'dataPointId': 'DMBS000028', 'order': 364, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 908.0, None, None, None, 934.0, None, 
        None, None, None, None]}, {'parentId': 'DMBS000025361', 'label': 'Other Contractual Obligations due in Year 5', 'dataPointId': 'DMBS000030', 'order': 366, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 782.0, None, None, None, 839.0, None, None, None, None, None]}, {'parentId': 'DMBS000025361', 'label': 'Other Contractual Obligations due Beyond', 'dataPointId': 'DMBS000031', 'order': 367, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None, 976.0, None, None, None, 569.0, None, None, None, None, None]}]}]}], 'footer': {'currency': 'USD', 'currencySymbol': '$', 'orderOfMagnitude': 'Million', 'fiscalYearEndDate': '09-30'}, 'userType': 'Free'}     
        """

        return self.financialStatement("balancesheet",period=period, reportType=reportType)
    
    def boardOfDirectors(self):
        """
        This function retrieves information about the board of directors.

        Returns:
            dict with board of directors information

        Examples:
            >>> Stock("Alphabet Inc Class A").boardOfDirectors()

        {'rows': [{'type': 'person', 'personId': 'PS00000QWT', 'name': 'John L. Hennessy', 'title': 'Non- Executive Chairman of the Board', 'memberSince': '2004', 'age': '69', 'committee': 'Nominating and Corporate Governance', 'totalCompensation': ['430567.00', '486428.00', '604711.00', '603314.00', '620310.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': ['75000.00', '83717.00', '100000.00', '100000.00', '100000.00']}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': ['355567.00', '402711.00', '504711.00', '503314.00', '520310.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'person', 'personId': 'PS00007HMI', 'name': 'Frances H. Arnold', 'title': 'Independent Director', 'memberSince': '2019', 'age': '65', 'committee': 'Nominating and Corporate Governance', 'totalCompensation': [None, None, None, '1226305.00', '439217.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': [None, None, None, '38248.00', '75000.00']}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': [None, None, None, '1188057.00', '364217.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'person', 'personId': 'PS00001LRQ', 'name': 'Sergey Brin', 'title': 'Director', 'memberSince': '1998', 'age': '48', 'committee': 
        'Executive', 'totalCompensation': [None, None, None, '1.00', '1.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': [None, None, None, None, None]}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, 
        None, None, '1.00', '1.00']}]}, {'type': 'person', 'personId': 'PS00001BFB', 'name': 'Ramon Martin Chavez', 'title': 'Director', 'memberSince': 
        '2022', 'age': None, 'committee': None, 'totalCompensation': [None, None, None, None, None], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': [None, None, None, 
        None, None]}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'person', 'personId': 'PS00000ZUY', 'name': 'L. John Doerr', 'title': 'Independent Director', 'memberSince': '1999', 'age': '70', 'committee': 'Leadership Development, Inclusion and Compensation', 'totalCompensation': ['430567.00', '425786.00', '428298.00', '427320.00', '439217.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': ['75000.00', '75000.00', '75000.00', '75000.00', '75000.00']}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': ['355567.00', '350786.00', '353298.00', '352320.00', '364217.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'person', 'personId': 'PS00004ORU', 'name': 'Roger W. Ferguson', 'title': 'Independent Director', 'memberSince': '2016', 'age': '70', 'committee': 'Audit and Compliance', 'totalCompensation': ['410708.00', '425786.00', '428298.00', '427320.00', '439217.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': ['71552.00', '75000.00', '75000.00', '75000.00', '75000.00']}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': ['339156.00', '350786.00', '353298.00', '352320.00', '364217.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'person', 'personId': 'PS000004TI', 'name': 'Ann Mather', 'title': 'Independent Director', 'memberSince': '2005', 'age': '61', 'committee': 'Audit and Compliance', 'totalCompensation': ['455567.00', '450786.00', '453298.00', '452320.00', '464217.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': ['100000.00', '100000.00', '100000.00', '100000.00', '100000.00']}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': ['355567.00', '350786.00', '353298.00', '352320.00', '364217.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'person', 'personId': 'PS000024LV', 'name': 'Larry 
        Page', 'title': 'Director', 'memberSince': '1998', 'age': '49', 'committee': 'Executive', 'totalCompensation': [None, None, None, '1.00', '1.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 
        'Restricted Stock Award', 'datum': [None, None, None, None, None]}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, None, None, '1.00', '1.00']}]}, {'type': 'person', 'personId': 'PS00007ZVH', 'name': 'Sundar Pichai', 'title': 'Director; Chief Executive Officer of the Company and Google Inc', 'memberSince': '2017', 'age': '49', 'committee': 'Executive', 'totalCompensation': [None, None, None, None, None], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': [None, None, None, None, None]}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 
        'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'person', 'personId': 'PS00000VNS', 'name': 'K. Ram Shriram', 'title': 'Independent Director', 
        'memberSince': '1998', 'age': '65', 'committee': 'Leadership Development, Inclusion and Compensation', 'totalCompensation': ['430567.00', '425786.00', '428298.00', '427320.00', '439217.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': ['75000.00', '75000.00', '75000.00', '75000.00', '75000.00']}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': ['355567.00', '350786.00', '353298.00', '352320.00', '364217.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'person', 'personId': 'PS000026YH', 'name': 'Robin L. Washington', 'title': 'Independent Director', 'memberSince': '2019', 'age': '59', 'committee': 'Leadership Development, Inclusion and Compensation', 'totalCompensation': [None, None, '1047309.00', '427320.00', '439217.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': [None, None, '11071.00', '75000.00', '75000.00']}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': [None, None, '1036238.00', '352320.00', '364217.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'OTHER_COMPENSATION', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'total', 'personId': 'totalBoardOfDirectors', 'name': 'Compensation 
        for all Board of Directors', 'totalCompensation': ['2157976.00', '2214572.00', '3390212.00', '3991221.00', '3280614.00'], 'compensation': [{'nameId': 'totalSalary', 'name': 'Salary', 'datum': ['396552.00', '408717.00', '436071.00', '538248.00', '575000.00']}, {'nameId': 'totalStockAward', 'name': 'Restricted Stock Award', 'datum': ['1761424.00', '1805855.00', '2954141.00', '3452971.00', '2705612.00']}, {'nameId': 'totalOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'totalNon-Equity', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'totalOther', 'name': 'Other Compensation', 'datum': [None, None, None, '2.00', '2.00']}]}], 'datesDef': ['2017', '2018', '2019', '2020', '2021'], 'currency': 'USD', 'template': ''}
                
        """
        return self.GetData("insiders/boardOfDirectors")

    def cashFlow(self, period='annual', reportType='original'):
        """
        This function retrieves the cash flow.

        Args:
            period (str) : possible values are annual, quarterly
            reportType (str) : possible values are original, restated

        Returns:
            dict with cash flow

        Examples:
            >>> Stock("visa", exchange="nyse").cashFlow('annual', 'restated')
            >>> Stock("visa", exchange="nyse").cashFlow('quarterly', 'restated')

        {'_meta': {'companyId': '0C00000VF2', 'statementType': 'cash-flow', 'periodReport': 'Success', 'latestReport': 'Success'}, 'columnDefs': ['2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', 'TTM'], 'filingIdList': [None, None, None, None, None, '309018162', '364101718', '417618183', '417618183', '417618183', None], 'rows': [{'parentId': '', 'label': 'CashFlow', 'dataPointId': 'IFCF000000', 'order': 697, 'subLevel': [{'parentId': 'IFCF000000697', 'label': 'Cash Flow from Operating Activities, Indirect', 'dataPointId': 'IFCF000150', 'order': 727, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 12941.0, 12784.0, 10440.0, 15227.0, 18849.0, 18788.0], 'subLevel': [{'parentId': 'IFCF000150727', 'label': 'Net Cash Flow from Continuing Operating Activities, Indirect', 'dataPointId': 'IFCF001591', 'order': 728, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 12941.0, 12784.0, 10440.0, 15227.0, 18849.0, 18788.0], 'subLevel': [{'parentId': 'IFCF001591728', 'label': 'Cash Generated from Operating Activities', 'dataPointId': 'IFCF200620', 'order': 729, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 12941.0, 12784.0, 10440.0, 15227.0, 18849.0, 18788.0], 'subLevel': [{'parentId': 'IFCF200620729', 'label': 'Income/Loss before Non-Cash Adjustment', 'dataPointId': 'IFCF001613', 'order': 730, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 10301.0, 12080.0, 10866.0, 12311.0, 14957.0, 15177.0]}, {'parentId': 'IFCF200620729', 'label': 'Total Adjustments for Non-Cash Items', 'dataPointId': 'IFCF200190', 'order': 733, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 5079.0, 7007.0, 7972.0, 9618.0, 11549.0, 12267.0], 'subLevel': [{'parentId': 'IFCF200190733', 'label': 'Depreciation, Amortization and Depletion, Non-Cash Adjustment', 'dataPointId': 'IFCF001559', 'order': 734, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 613.0, 656.0, 767.0, 804.0, 861.0, 890.0], 'subLevel': [{'parentId': 'IFCF001559734', 'label': 'Depreciation and Amortization, Non-Cash Adjustment', 'dataPointId': 'IFCF000620', 'order': 735, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 613.0, 656.0, 767.0, 804.0, 861.0, 890.0]}]}, {'parentId': 'IFCF200190733', 'label': 'Stock-Based Compensation, Non-Cash Adjustment', 'dataPointId': 'IFCF001500', 'order': 742, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 327.0, 407.0, 416.0, 542.0, 602.0, 651.0]}, {'parentId': 'IFCF200190733', 'label': 'Deferred Taxes, Non-Cash Adjustment', 'dataPointId': 'IFCF000570', 'order': 743, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -1277.0, 214.0, 307.0, 873.0, -336.0, -453.0]}, {'parentId': 'IFCF200190733', 'label': 'Net Investment Income/Loss, Non-Cash Adjustment', 'dataPointId': 'IFCF002200', 'order': 748, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, -131.0, -101.0, -712.0, 264.0, 601.0], 'subLevel': [{'parentId': 'IFCF002200748', 'label': 'Share of Profit/Loss from Associates, Joint Ventures and other Equity Investments, Non-Cash Adjustment', 'dataPointId': 'IFCF100472', 'order': 755, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, -131.0, -101.0, -712.0, 264.0, 601.0]}, {'parentId': 'IFCF002200748', 'label': 'Gain/Loss on Derivatives, Non-Cash Adjustment', 'dataPointId': 'IFCF001644', 'order': 752, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None]}]}, {'parentId': 'IFCF200190733', 'label': 'Irregular Income/Loss, Non-Cash Adjustment', 'dataPointId': 'IFCF002202', 'order': 758, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -11.0, -172.0, -37.0, -147.0, -43.0, -44.0], 'subLevel': [{'parentId': 'IFCF002202758', 'label': 'Other Irregular Income/Loss, Non-Cash Adjustment', 'dataPointId': 'IFCF002203', 'order': 769, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -11.0, -172.0, -37.0, -147.0, -43.0, -44.0]}]}, {'parentId': 'IFCF200190733', 'label': 'Other Non-Cash Items', 'dataPointId': 'IFCF000920', 'order': 771, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 5427.0, 6033.0, 6620.0, 8258.0, 10201.0, 10622.0]}, {'parentId': 'IFCF200190733', 'label': 'Excess Tax Benefit from Stock-Based Compensation, Non-Cash Adjustment', 'dataPointId': 'IFCF000660', 'order': 741, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None]}, {'parentId': 'IFCF200190733', 'label': 'Other Provisions, Non-Cash Adjustment', 'dataPointId': 'IFCF200621', 'order': 747, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None]}]}, {'parentId': 'IFCF200620729', 'label': 'Changes in Operating Capital', 'dataPointId': 'IFCF000530', 'order': 772, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -2439.0, -6303.0, -8398.0, -6702.0, -7657.0, -8656.0], 'subLevel': [{'parentId': 'IFCF000530772', 'label': 'Change in Trade and Other Receivables', 'dataPointId': 'IFCF000450', 'order': 774, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -293.0, -1866.0, 1815.0, -811.0, -494.0, -319.0], 'subLevel': [{'parentId': 'IFCF000450774', 'label': 'Change in Trade/Accounts Receivable', 'dataPointId': 'IFCF001547', 'order': 775, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -70.0, -333.0, -43.0, -343.0, -97.0, 56.0]}, {'parentId': 'IFCF000450774', 'label': 'Change in Other Receivables', 'dataPointId': 'IFCF001536', 'order': 783, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -223.0, -1533.0, 1858.0, -468.0, -397.0, -375.0]}]}, {'parentId': 'IFCF000530772', 'label': 'Change in Other Current Assets', 'dataPointId': 'IFCF000370', 'order': 786, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 59.0, -310.0, -402.0, -147.0, -666.0, -343.0]}, {'parentId': 'IFCF000530772', 'label': 'Change in Payables and Accrued Expenses', 'dataPointId': 'IFCF001537', 'order': 787, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 2477.0, 2303.0, -1730.0, 1766.0, 2854.0, 1761.0], 'subLevel': [{'parentId': 'IFCF001537787', 'label': 'Change in Trade and Other Payables', 'dataPointId': 'IFCF000400', 'order': 788, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 265.0, 1907.0, -2363.0, 767.0, 1323.0, 903.0], 'subLevel': [{'parentId': 'IFCF000400788', 'label': 'Change in Trade/Accounts Payable', 'dataPointId': 'IFCF001545', 'order': 789, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 3.0, 
        -24.0, 21.0, 88.0, 67.0, 12.0]}, {'parentId': 'IFCF000400788', 'label': 'Change in Other Payables', 'dataPointId': 'IFCF001534', 'order': 794, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 262.0, 1931.0, -2384.0, 679.0, 1256.0, 891.0]}]}, {'parentId': 'IFCF001537787', 'label': 'Change in Accrued Expenses', 'dataPointId': 'IFCF000260', 'order': 795, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 2212.0, 396.0, 633.0, 999.0, 1531.0, 858.0]}]}, {'parentId': 'IFCF000530772', 'label': 'Change in Other Operating Capital', 'dataPointId': 'IFCF001532', 'order': 799, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -4682.0, -6430.0, -8081.0, -7510.0, -9351.0, -9755.0]}]}]}]}]}, {'parentId': 'IFCF000000697', 'label': 'Cash Flow from Investing Activities', 'dataPointId': 'IFCF000140', 'order': 815, 
        'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -3084.0, -591.0, 1427.0, -152.0, -4288.0, -4251.0], 'subLevel': [{'parentId': 'IFCF000140815', 'label': 'Cash Flow from Continuing Investing Activities', 'dataPointId': 'IFCF001510', 'order': 816, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -3084.0, -591.0, 1427.0, -152.0, -4288.0, -4251.0], 'subLevel': [{'parentId': 'IFCF001510816', 'label': 'Purchase/Sale and Disposal of 
        Property, Plant and Equipment, Net', 'dataPointId': 'IFCF001330', 'order': 822, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -718.0, -756.0, -736.0, -705.0, -970.0, -1046.0], 'subLevel': [{'parentId': 'IFCF001330822', 'label': 'Purchase of Property, Plant and Equipment', 'dataPointId': 'IFCF001260', 'order': 823, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -718.0, -756.0, -736.0, -705.0, -970.0, -1046.0]}, {'parentId': 'IFCF001330822', 'label': 'Sale and Disposal of Property, Plant and Equipment', 'dataPointId': 'IFCF001430', 'order': 825, 
        'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None]}]}, {'parentId': 'IFCF001510816', 'label': 'Purchase/Sale of Business, Net', 'dataPointId': 'IFCF001300', 'order': 830, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -196.0, -699.0, -77.0, -75.0, -1948.0, -1116.0], 'subLevel': [{'parentId': 'IFCF001300830', 'label': 'Purchase/Acquisition of Business', 'dataPointId': 'IFCF001290', 'order': 831, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -196.0, -699.0, -77.0, -75.0, -1948.0, -1116.0]}]}, 
        {'parentId': 'IFCF001510816', 'label': 'Purchase/Sale of Investments, Net', 'dataPointId': 'IFCF001320', 'order': 839, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -2186.0, 842.0, 2168.0, 519.0, -1498.0, -2187.0], 'subLevel': [{'parentId': 'IFCF001320839', 'label': 'Purchase of Investments', 'dataPointId': 'IFCF001240', 'order': 840, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -5822.0, -3154.0, -2342.0, -5182.0, -6083.0, -7110.0]}, {'parentId': 'IFCF001320839', 'label': 'Sale of Investments', 'dataPointId': 'IFCF001410', 'order': 841, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 3636.0, 3996.0, 4510.0, 5701.0, 4585.0, 4923.0]}]}, {'parentId': 'IFCF001510816', 'label': 'Other Investing Cash Flow', 'dataPointId': 'IFCF000910', 'order': 848, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', 
        '_PO_', 16.0, 22.0, 72.0, 109.0, 128.0, 98.0]}, {'parentId': 'IFCF001510816', 'label': 'Capital Expenditure, Reported', 'dataPointId': 'IFCF000061', 'order': 817, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None]}]}]}, {'parentId': 'IFCF000000697', 'label': 'Cash Flow from Financing Activities', 'dataPointId': 'IFCF000130', 'order': 850, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -10790.0, -12061.0, -3968.0, -14410.0, -12696.0, -14076.0], 'subLevel': [{'parentId': 'IFCF000130850', 'label': 'Cash Flow from Continuing Financing Activities', 'dataPointId': 'IFCF001509', 'order': 851, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -10790.0, -12061.0, 
        -3968.0, -14410.0, -12696.0, -14076.0], 'subLevel': [{'parentId': 'IFCF001509851', 'label': 'Issuance of/Payments for Common Stock, Net', 'dataPointId': 'IFCF000760', 'order': 852, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -7192.0, -8607.0, -8114.0, -8676.0, -11589.0, -10600.0], 
        'subLevel': [{'parentId': 'IFCF000760852', 'label': 'Payments for Common Stock', 'dataPointId': 'IFCF000950', 'order': 854, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -7192.0, -8607.0, -8114.0, -8676.0, -11589.0, -10600.0]}]}, {'parentId': 'IFCF001509851', 'label': 'Issuance of/Repayments for Debt, Net', 'dataPointId': 'IFCF000770', 'order': 858, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -1750.0, 0.0, 7212.0, -3000.0, 2218.0, -32.0], 'subLevel': [{'parentId': 'IFCF000770858', 'label': 'Issuance of/Repayments for Long Term Debt, Net', 'dataPointId': 'IFCF000780', 'order': 862, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -1750.0, 0.0, 7212.0, -3000.0, 2218.0, -32.0], 'subLevel': [{'parentId': 'IFCF000780862', 'label': 'Proceeds from Issuance of Long Term Debt', 'dataPointId': 'IFCF001090', 'order': 863, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 0.0, 0.0, 7212.0, 0.0, 3218.0, None]}, {'parentId': 'IFCF000780862', 'label': 'Repayments for Long Term Debt', 'dataPointId': 'IFCF000980', 'order': 864, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -1750.0, 0.0, 0.0, -3000.0, -1000.0, -3250.0]}]}]}, {'parentId': 'IFCF001509851', 'label': 'Cash Dividends and Interest Paid', 'dataPointId': 'IFCF002150', 'order': 877, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -1918.0, -2269.0, -2664.0, -2798.0, -3203.0, -3339.0], 'subLevel': [{'parentId': 'IFCF002150877', 'label': 'Cash Dividends Paid', 'dataPointId': 'IFCF000080', 'order': 878, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -1918.0, -2269.0, -2664.0, -2798.0, -3203.0, -3339.0], 'subLevel': [{'parentId': 'IFCF000080878', 'label': 'Preferred Stock Dividends Paid', 'dataPointId': 'IFCF001020', 'order': 880, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None]}]}]}, {'parentId': 'IFCF001509851', 'label': 'Proceeds from Issuance/Exercising of Stock Options/Warrants', 'dataPointId': 'IFCF001643', 'order': 884, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 70.0, 51.0, 30.0, 64.0, 76.0, 74.0]}, {'parentId': 'IFCF001509851', 'label': 'Other Financing Cash Flow', 'dataPointId': 'IFCF000900', 'order': 885, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 0.0, -1236.0, -432.0, 0.0, -198.0, None]}, {'parentId': 'IFCF001509851', 'label': 'Issue and Financing Costs', 'dataPointId': 'IFCF100477', 'order': 874, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None], 'subLevel': [{'parentId': 'IFCF100477874', 'label': 'Debt Issuance Costs', 'dataPointId': 'IFCF001555', 'order': 876, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None]}]}, {'parentId': 'IFCF001509851', 'label': 'Excess Tax Benefit from Share-Based Compensation, Financing Activities', 'dataPointId': 'IFCF001794', 'order': 883, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None]}, {'parentId': 'IFCF001509851', 'label': 'Issuance of/Repayments for Lease Financing', 'dataPointId': 'IFCF100469', 'order': 865, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None], 'subLevel': [{'parentId': 'IFCF100469865', 'label': 'Repayments for Lease Financing', 'dataPointId': 'IFCF100471', 'order': 867, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None]}]}]}]}, {'parentId': 'IFCF000000697', 'label': 'Cash and Cash Equivalents, End of Period', 'dataPointId': 'IFCF000170', 'order': 889, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 10977.0, 10832.0, 19171.0, 19799.0, 20377.0, 18383.0], 'subLevel': [{'parentId': 'IFCF000170889', 'label': 'Change in Cash', 'dataPointId': 'IFCF000280', 'order': 890, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -933.0, 132.0, 7899.0, 665.0, 1865.0, 461.0]}, {'parentId': 'IFCF000170889', 'label': 'Effect of Exchange Rate Changes', 'dataPointId': 'IFCF000650', 'order': 891, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -101.0, -277.0, 440.0, -37.0, -1287.0, -401.0]}, {'parentId': 'IFCF000170889', 'label': 'Cash and Cash Equivalents, Beginning of Period', 'dataPointId': 'IFCF000160', 'order': 896, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 12011.0, 10977.0, 10832.0, 19171.0, 19799.0, 18323.0]}]}, {'parentId': 'IFCF000000697', 'label': 'Cash Flow Supplemental Section', 'dataPointId': 'IFCF001730', 'order': 897, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None], 'subLevel': [{'parentId': 'IFCF001730897', 'label': 'Change in Cash as Reported, Supplemental', 'dataPointId': 'IFCF002210', 'order': 898, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -1034.0, -145.0, 8339.0, 628.0, 578.0, 60.0]}, {'parentId': 'IFCF001730897', 'label': 'Income Tax Paid, Supplemental', 'dataPointId': 'IFCF001580', 'order': 899, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -2285.0, -2648.0, -2671.0, -3012.0, -3741.0, -4194.0]}, {'parentId': 'IFCF001730897', 'label': 'Interest Paid, Supplemental', 'dataPointId': 'IFCF001731', 'order': 901, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -545.0, -537.0, -537.0, -643.0, -607.0, -607.0]}]}]}], 'footer': {'currency': 'USD', 'currencySymbol': '$', 'orderOfMagnitude': 'Million', 'fiscalYearEndDate': '09-30'}, 'userType': 'Free'}  
        """

        return self.financialStatement("cashflow",period=period, reportType=reportType)
    
    def dividends(self):
        """
        This function retrieves the dividends of the stock.

        Returns:
            dict with dividends

        Examples:
            >>> Stock("visa", exchange="nyse").dividends()

        {'rows': [{'label': 'Dividend Per Share', 'salDataId': 'dividends.per.share.label', 'datum': ['0.34750000000000003', '0.42000000000000004', '0.5', '0.5850000000000001', '0.69', '0.88', '1.05', '1.22', '1.335', '1.575', '1.8', '1.8', '1.14225']}, 
        {'label': 'Trailing Dividend Yield %', 'salDataId': 'trailing.dividends.yield.label', 'datum': ['0.62', '0.64', '0.64', '0.75', '0.61', '0.67', '0.56', '0.56', '0.62', '0.76', '0.74', '0.74', '0.61'], 'percentage': True}, {'label': 'Buyback Yield %', 'salDataId': 'buyback.yield.label', 'datum': ['3.83', '2.56', '1.57', '3.91', '2.65', '2.45', '2.10', '1.71', '1.88', '2.63', '2.26', '2.26', '2.19'], 'percentage': True}, {'label': 'Total Yield %', 'salDataId': 'total.yield.label', 'datum': ['4.45', '3.20', '2.21', '4.66', '3.26', '3.12', '2.66', '2.27', '2.50', '3.39', '3.00', '3.00', '2.80'], 'percentage': True}, {'label': 'Payout Ratio %', 'salDataId': 'payout.ratio.label', 'datum': ['17.39', '18.56', '18.60', '22.58', '23.57', '18.67', '18.80', '24.54', '22.74', '21.43', '22.00', '22.00', '21.24'], 'percentage': True}], 'columnDefs_labels': ['tabular.data.label.column.year', '2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', 'dividends.headers.current', 'dividends.headers.oneyearttm', 'dividends.headers.fiveyear'], 'dividendData': {'label': ['exdividend.date.label', 'declaration.date.label', 'record.date.label', 'payable.date.label', 'dividend.type.label', 'dividend.amount.label'], 'upcomingData': [['2023-02-09T00:00:00.000', '2023-01-24T00:00:00.000', '2023-02-10T00:00:00.000', '2023-03-01T00:00:00.000', 'Cash Dividend', '0.45']], 'dividendHistory': [{'year': '2023', 'datum': [['2023-02-09T00:00:00.000', '2023-01-24T00:00:00.000', '2023-02-10T00:00:00.000', '2023-03-01T00:00:00.000', 'Cash Dividend', '0.45']]}, {'year': '2022', 'datum': [['2022-11-09T00:00:00.000', '2022-10-21T00:00:00.000', '2022-11-11T00:00:00.000', '2022-12-01T00:00:00.000', 'Cash Dividend', '0.45'], ['2022-08-11T00:00:00.000', '2022-07-22T00:00:00.000', '2022-08-12T00:00:00.000', '2022-09-01T00:00:00.000', 'Cash Dividend', '0.375'], ['2022-05-12T00:00:00.000', '2022-04-26T00:00:00.000', '2022-05-13T00:00:00.000', '2022-06-01T00:00:00.000', 'Cash Dividend', '0.375'], ['2022-02-10T00:00:00.000', '2022-01-25T00:00:00.000', '2022-02-11T00:00:00.000', '2022-03-01T00:00:00.000', 'Cash Dividend', '0.375']]}, {'year': '2021', 'datum': [['2021-11-10T00:00:00.000', '2021-10-22T00:00:00.000', '2021-11-12T00:00:00.000', '2021-12-07T00:00:00.000', 'Cash Dividend', '0.375'], ['2021-08-12T00:00:00.000', '2021-07-23T00:00:00.000', '2021-08-13T00:00:00.000', '2021-09-01T00:00:00.000', 'Cash Dividend', '0.32'], ['2021-05-13T00:00:00.000', '2021-04-27T00:00:00.000', '2021-05-14T00:00:00.000', '2021-06-01T00:00:00.000', 'Cash Dividend', '0.32'], ['2021-02-11T00:00:00.000', '2021-01-28T00:00:00.000', '2021-02-12T00:00:00.000', '2021-03-01T00:00:00.000', 'Cash Dividend', '0.32']]}, {'year': '2020', 'datum': [['2020-11-12T00:00:00.000', '2020-10-30T00:00:00.000', '2020-11-13T00:00:00.000', '2020-12-01T00:00:00.000', 'Cash Dividend', '0.32'], ['2020-08-13T00:00:00.000', '2020-07-20T00:00:00.000', '2020-08-14T00:00:00.000', '2020-09-01T00:00:00.000', 'Cash Dividend', '0.3'], ['2020-05-13T00:00:00.000', '2020-04-22T00:00:00.000', '2020-05-14T00:00:00.000', '2020-06-02T00:00:00.000', 'Cash Dividend', '0.3'], ['2020-02-13T00:00:00.000', '2019-12-30T00:00:00.000', '2020-02-14T00:00:00.000', '2020-03-03T00:00:00.000', 'Cash Dividend', '0.3']]}, {'year': '2019', 'datum': [['2019-11-14T00:00:00.000', '2019-10-22T00:00:00.000', '2019-11-15T00:00:00.000', '2019-12-03T00:00:00.000', 'Cash Dividend', '0.3'], ['2019-08-15T00:00:00.000', '2019-07-17T00:00:00.000', '2019-08-16T00:00:00.000', '2019-09-03T00:00:00.000', 'Cash Dividend', '0.25'], ['2019-05-16T00:00:00.000', '2019-04-16T00:00:00.000', '2019-05-17T00:00:00.000', '2019-06-04T00:00:00.000', 'Cash Dividend', '0.25'], ['2019-02-14T00:00:00.000', '2019-01-29T00:00:00.000', '2019-02-15T00:00:00.000', '2019-03-05T00:00:00.000', 'Cash Dividend', '0.25']]}]}, 'quoteData': [{'name': 'dividendYield', 'label': 'Dividend Yield', 'salDataId': 'dividend.yield.label', 'date': '2023-02-20T00:00:00.000', 'value': '0.0074'}, {'name': 'divReinvestmentPlan', 'label': 'Div Reinvestment Plan', 'salDataId': 'div.reinvestment.plan.label', 'date': '2023-02-09T00:00:00.000', 'value': 'No'}], 'footer': {'asOfLabel': 'As of', 'distributionCurrency': 'USD', 'asOfDate': '2023-02-20T00:00:00.000'}} 
                
        """
        return self.GetData("dividends/v4")
    
    def esgRisk(self):
        """
        This function retrieves the esg risk of the stock.

        Returns:
            dict with esg risk

        Examples:
            >>> Stock("visa", exchange="nyse").esgRisk()

            {'susEsgRiskScore': 15.5125, 'susEsgRiskGlobes': 4, 'susEsgRiskCategory': 'Low', 'comHighestControversyLevel': 3, 
            'comControversyLevelDescriptor': 'Significant', 'comHighestControversyTopics': 'Customer', 'notableIssue1Name': 'Overall', 'notableIssue2Name': 'Overall', 'notableIssue3Name': 'Overall', 'notableIssue1': 'Data Privacy & Security', 'notableIssue2': 'Business Ethics', 'notableIssue3': 'Human Capital', 'asOfDate': '2023-02-01T06:00:00.000', 'subIndustry': 'Data Processing', 'controversyAsOfDate': '2023-02-08T06:00:00.000'}       
        """
        return self.GetData("esgRisk")
    
    def financialHealth(self):
        """
        This function retrieves the financial health of the stock.

        Returns:
            dict with financial health

        Examples:
            >>> Stock("visa", exchange="nyse").financialHealth()

        {'currency': 'USD', 'asOfDate': '2022-12-31T06:00:00.000', 'dataList': [{'fiscalPeriodYearMonth': '2013-09', 'morningstarEndingDate': '2013-09-30T05:00:00.000', 'currentRatio': 1.8044, 'quickRatio': 1.3742, 'financialLeverage': 1.3381, 'debtEquityRatio': None, 'bookValuePerShare': 10.6586}, {'fiscalPeriodYearMonth': '2014-09', 'morningstarEndingDate': '2014-09-30T05:00:00.000', 'currentRatio': 1.5921, 'quickRatio': 0.9406, 'financialLeverage': 1.407, 'debtEquityRatio': None, 'bookValuePerShare': 11.0854}, 
        {'fiscalPeriodYearMonth': '2015-09', 'morningstarEndingDate': '2015-09-30T05:00:00.000', 'currentRatio': 2.0268, 'quickRatio': 1.3671, 'financialLeverage': 1.3483, 'debtEquityRatio': None, 'bookValuePerShare': 11.8125}, {'fiscalPeriodYearMonth': '2016-09', 'morningstarEndingDate': '2016-09-30T05:00:00.000', 'currentRatio': 1.7789, 'quickRatio': 1.4514, 'financialLeverage': 2.3547, 'debtEquityRatio': 0.584, 'bookValuePerShare': 11.4629}, {'fiscalPeriodYearMonth': '2017-09', 'morningstarEndingDate': '2017-09-30T05:00:00.000', 'currentRatio': 1.9034, 'quickRatio': 1.6002, 'financialLeverage': 2.496, 'debtEquityRatio': 0.6102, 'bookValuePerShare': 11.506}, {'fiscalPeriodYearMonth': '2018-09', 'morningstarEndingDate': '2018-09-30T05:00:00.000', 'currentRatio': 1.6113, 'quickRatio': 1.2825, 'financialLeverage': 2.4259, 'debtEquityRatio': 0.5828, 'bookValuePerShare': 12.6041}, {'fiscalPeriodYearMonth': '2019-09', 'morningstarEndingDate': '2019-09-30T05:00:00.000', 'currentRatio': 1.5632, 'quickRatio': 1.2422, 'financialLeverage': 2.4835, 'debtEquityRatio': 0.5725, 'bookValuePerShare': 13.4975}, {'fiscalPeriodYearMonth': '2020-09', 'morningstarEndingDate': '2020-09-30T05:00:00.000', 'currentRatio': 1.9052, 'quickRatio': 1.5798, 'financialLeverage': 2.5999, 'debtEquityRatio': 0.677, 'bookValuePerShare': 14.2546}, {'fiscalPeriodYearMonth': '2021-09', 'morningstarEndingDate': '2021-09-30T05:00:00.000', 'currentRatio': 1.7541, 'quickRatio': 1.4129, 'financialLeverage': 2.4022, 'debtEquityRatio': 0.5789, 'bookValuePerShare': 16.1724}, {'fiscalPeriodYearMonth': '2022-09', 'morningstarEndingDate': '2022-09-30T05:00:00.000', 'currentRatio': 1.4485, 'quickRatio': 1.0777, 'financialLeverage': 2.403, 'debtEquityRatio': 0.5677, 'bookValuePerShare': 15.4465}, {'fiscalPeriodYearMonth': 'Latest Qtr', 'morningstarEndingDate': '2022-12-31T06:00:00.000', 'currentRatio': 1.4395, 'quickRatio': 1.052, 'financialLeverage': 2.3125, 'debtEquityRatio': 0.5548, 'bookValuePerShare': 16.9189}]}    
        
        """
        return self.GetData("keyStats/financialHealth", url_suffixe='')
    
    def financialStatement(self,statement ='summary', period='annual', reportType='original'):
        """
        This function retrieves the financial statement.

        Args:
            statement (str) : possible values are balancesheet, cashflow, incomestatement, summary
            period (str) : possible values are annual, quarterly
            reportType (str) : possible values are original, restated

        Returns:
            dict with financial statement 

        Examples:
            >>> Stock("visa", exchange="nyse").financialStatement('summary', 'quarterly', 'original')
            >>> Stock("visa", exchange="nyse").financialStatement('cashflow', 'annual', 'restated')
            >>> Stock("visa", exchange="nyse").financialStatement('balancesheet', 'annual', 'restated')
            >>> Stock("visa", exchange="nyse").financialStatement('incomestatement', 'annual', 'restated')

        {'incomeStatement': {'_meta': {'companyId': '0C00000VF2', 'statementType': 'income-statement', 'periodReport': 'Success', 'latestReport': 'Success'}, 'columnDefs': ['Q3 2022', 'Q4 2022', 'Q1 2023', 'TTM'], 'filingIdList': ['401773298', None, '429622549', None], 'columnDefs_labels': ['20220630', '20220930', '20221231', '20221231'], 'rows': [{'label': 'Total Revenue', 'dataPointId': 'IFIS001170', 'datum': [7.28, 7.79, 7.94, 30.19]}, {'label': 'Total Operating Profit/Loss', 'dataPointId': 'IFIS001230', 'datum': [4.87, 5.09, 5.43, 20.19]}, {'label': 'Net Income Available to Common Stockholders', 'dataPointId': 'IFIS001100', 'datum': [3.41, 3.94, 4.18, 15.18]}, {'label': 'EBITDA', 'dataPointId': 'SAL0000003', 'datum': [4.17, 5.25, 5.34, 19.65]}, {'label': 'Diluted EPS', 'dataPointId': 'IFIS000280', 'datum': [1.6, 1.86, 1.99, 7.16]}, {'label': 'Normalized EPS', 'dataPointId': 'IFIS101213', 'datum': [1.99, 1.88, 2.12, 7.72]}], 'footer': {'currency': 'USD', 'currencySymbol': '$', 'orderOfMagnitude': 'Billion', 'fiscalYearEndDate': '09-30'}}, 'balanceSheet': {'_meta': {'companyId': '0C00000VF2', 'statementType': 'balance-sheet', 'periodReport': 'Success', 'latestReport': 'OK'}, 'columnDefs': ['Q2 2022', 'Q3 2022', 'Q4 2022', 'Q1 2023'], 'filingIdList': ['388626196', '401773298', '417618183', '429622549'], 'columnDefs_labels': ['20220331', '20220630', '20220930', '20221231'], 'rows': [{'label': 'Total Assets', 'dataPointId': 'IFBS002270', 'datum': [81.81, 85.41, 85.5, 85.39]}, {'label': 'Total Liabilities', 'dataPointId': 'IFBS002646', 'datum': [45.82, 49.93, 49.92, 48.46]}, {'label': 'Total Debt', 'dataPointId': 'SAL0000001', 'datum': [21.03, 23.8, 22.45, 20.49]}, {'label': 'Total Equity', 'dataPointId': 'IFBS002220', 'datum': [35.99, 35.48, 35.58, 36.92]}, {'label': 'Cash, Cash Equivalents and Short Term Investments', 'dataPointId': 'IFBS000350', 'datum': [13.53, 17.36, 18.52, 16.12]}, {'label': 'Working Capital', 'dataPointId': 'SAL0000004', 'datum': [5.39, 8.66, 9.35, 8.51]}], 'footer': 
        {'currency': 'USD', 'currencySymbol': '$', 'orderOfMagnitude': 'Billion', 'fiscalYearEndDate': '09-30'}}, 'cashFlow': {'_meta': {'companyId': '0C00000VF2', 'statementType': 'cash-flow', 'periodReport': 'Success', 'latestReport': 'Success'}, 'columnDefs': ['Q3 2022', 'Q4 2022', 'Q1 2023', 'TTM'], 'filingIdList': [None, None, '429622549', None], 'columnDefs_labels': ['20220630', '20220930', '20221231', '20221231'], 'rows': [{'label': 'Cash Flow from Operating Activities, Indirect', 'dataPointId': 'IFCF000150', 'datum': [5.25, 5.88, 4.17, 18.79]}, {'label': 'Cash Flow from Investing Activities', 'dataPointId': 'IFCF000140', 'datum': [-2.06, 0.11, -0.51, -4.25]}, {'label': 'Cash Flow from Financing Activities', 'dataPointId': 'IFCF000130', 'datum': [-0.29, -4.04, -6.35, -14.08]}, {'label': 'Capital Expenditure', 'dataPointId': 'SAL0000005', 'datum': [-0.24, -0.3, -0.25, -1.05]}, {'label': 'Free Cash Flow', 'dataPointId': 'SAL0000002', 'datum': [5.02, 5.58, 3.92, 17.74]}], 'footer': {'currency': 'USD', 'currencySymbol': '$', 'orderOfMagnitude': 'Billion', 'fiscalYearEndDate': '09-30'}}}
        """

        if not isinstance(statement, str):
            raise TypeError('statement parameter should be a string')
        
        if not isinstance(period, str):
            raise TypeError('period parameter should be a string')
        
        if not isinstance(reportType, str):
            raise TypeError('reportType parameter should be a string')
        
        statement_choice = {"balancesheet" : "balanceSheet", "cashflow" : "cashFlow", "incomestatement" : "incomeStatement", "summary" : "summary"}
        
        period_choice = {"annual" : "A", "quarterly" : "Q"}

        reportType_choice = {"original" : "A", "restated" : "R"}

        if statement not in statement_choice:
            raise ValueError(f"statement parameter must take one of the following value : { ', '.join(statement_choice)}")

        if period not in period_choice:
            raise ValueError(f"period parameter must take one of the following value : { ', '.join(period_choice)}")

        if reportType not in reportType_choice:
            raise ValueError(f"reportType parameter must take one of the following value : { ', '.join(reportType_choice.keys())}")
        
        params = {"reportType" : reportType_choice[reportType]}
        if statement == "summary":
            return self.GetData("newfinancials",params=params, url_suffixe=f"{period}/summary")
        
        params["dataType"] = period_choice[period]

        return self.GetData("newfinancials",params=params, url_suffixe=f"{statement_choice[statement]}/detail")
    
    def financialSummary(self, period='annual', reportType='original'):
        """
        This function retrieves the financial statement summary.

        Args:
            period (str) : possible values are annual, quarterly
            reportType (str) : possible values are original, restated

        Returns:
            dict with financial statement summary

        Examples:
            >>> Stock("visa", exchange="nyse").financialSummary('quarterly', 'original')
            >>> Stock("visa", exchange="nyse").financialSummary('annual', 'restated')

        {'incomeStatement': {'_meta': {'companyId': '0C00000VF2', 'statementType': 'income-statement', 'periodReport': 'Success', 'latestReport': 'Success'}, 'columnDefs': ['Q3 2022', 'Q4 2022', 'Q1 2023', 'TTM'], 'filingIdList': ['401773298', None, '429622549', None], 'columnDefs_labels': ['20220630', '20220930', '20221231', '20221231'], 'rows': [{'label': 'Total Revenue', 'dataPointId': 'IFIS001170', 'datum': [7.28, 7.79, 7.94, 30.19]}, {'label': 'Total Operating Profit/Loss', 'dataPointId': 'IFIS001230', 'datum': [4.87, 5.09, 5.43, 20.19]}, {'label': 'Net Income Available to Common Stockholders', 'dataPointId': 'IFIS001100', 'datum': [3.41, 3.94, 4.18, 15.18]}, {'label': 'EBITDA', 'dataPointId': 'SAL0000003', 'datum': [4.17, 5.25, 5.34, 19.65]}, {'label': 'Diluted EPS', 'dataPointId': 'IFIS000280', 'datum': [1.6, 1.86, 1.99, 7.16]}, {'label': 'Normalized EPS', 'dataPointId': 'IFIS101213', 'datum': [1.99, 1.88, 2.12, 7.72]}], 'footer': {'currency': 'USD', 'currencySymbol': '$', 'orderOfMagnitude': 'Billion', 'fiscalYearEndDate': '09-30'}}, 'balanceSheet': {'_meta': {'companyId': '0C00000VF2', 'statementType': 'balance-sheet', 'periodReport': 'Success', 'latestReport': 'OK'}, 'columnDefs': ['Q2 2022', 'Q3 2022', 'Q4 2022', 'Q1 2023'], 'filingIdList': ['388626196', '401773298', '417618183', '429622549'], 'columnDefs_labels': ['20220331', '20220630', '20220930', '20221231'], 'rows': [{'label': 'Total Assets', 'dataPointId': 'IFBS002270', 'datum': [81.81, 85.41, 85.5, 85.39]}, {'label': 'Total Liabilities', 'dataPointId': 'IFBS002646', 'datum': [45.82, 49.93, 49.92, 48.46]}, {'label': 'Total Debt', 'dataPointId': 'SAL0000001', 'datum': [21.03, 23.8, 22.45, 20.49]}, {'label': 'Total Equity', 'dataPointId': 'IFBS002220', 'datum': [35.99, 35.48, 35.58, 36.92]}, {'label': 'Cash, Cash Equivalents and Short Term Investments', 'dataPointId': 'IFBS000350', 'datum': [13.53, 17.36, 18.52, 16.12]}, {'label': 'Working Capital', 'dataPointId': 'SAL0000004', 'datum': [5.39, 8.66, 9.35, 8.51]}], 'footer': 
        {'currency': 'USD', 'currencySymbol': '$', 'orderOfMagnitude': 'Billion', 'fiscalYearEndDate': '09-30'}}, 'cashFlow': {'_meta': {'companyId': '0C00000VF2', 'statementType': 'cash-flow', 'periodReport': 'Success', 'latestReport': 'Success'}, 'columnDefs': ['Q3 2022', 'Q4 2022', 'Q1 2023', 'TTM'], 'filingIdList': [None, None, '429622549', None], 'columnDefs_labels': ['20220630', '20220930', '20221231', '20221231'], 'rows': [{'label': 'Cash Flow from Operating Activities, Indirect', 'dataPointId': 'IFCF000150', 'datum': [5.25, 5.88, 4.17, 18.79]}, {'label': 'Cash Flow from Investing Activities', 'dataPointId': 'IFCF000140', 'datum': [-2.06, 0.11, -0.51, -4.25]}, {'label': 'Cash Flow from Financing Activities', 'dataPointId': 'IFCF000130', 'datum': [-0.29, -4.04, -6.35, -14.08]}, {'label': 'Capital Expenditure', 'dataPointId': 'SAL0000005', 'datum': [-0.24, -0.3, -0.25, -1.05]}, {'label': 'Free Cash Flow', 'dataPointId': 'SAL0000002', 'datum': [5.02, 5.58, 3.92, 17.74]}], 'footer': {'currency': 'USD', 'currencySymbol': '$', 'orderOfMagnitude': 'Billion', 'fiscalYearEndDate': '09-30'}}}
        """

        return self.financialStatement("summary",period=period, reportType=reportType)
    
    def freeCashFlow(self):
        """
        This function retrieves the free cash flow.

        Returns:
            dict with free cash flow

        Examples:
            >>> Stock("visa", exchange="nyse").freeCashFlow()

        {'currency': 'USD', 'asOfDate': '2022-12-31T06:00:00.000', 'dataList': [{'fiscalPeriodDate': '2013-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2013-09', 'operatingCFGrowthPer': -39.67, 'freeCashFlowGrowthPer': -44.940000000000005, 'capExAsPerOfSales': 3.998981151299032, 'freeCFPerSales': 21.659025301409407, 'freeCashFlowPerNetIncome': 0.5122489959839357, 'freeCashFlowPerShare': 0.7199}, {'fiscalPeriodDate': '2014-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2014-09', 'operatingCFGrowthPer': 138.42000000000002, 'freeCashFlowGrowthPer': 160.76, 'capExAsPerOfSales': 4.353645095260589, 'freeCFPerSales': 52.36970555817982, 'freeCashFlowPerNetIncome': 1.223243839646929, 'freeCashFlowPerShare': 2.7499}, {'fiscalPeriodDate': '2015-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2015-09', 'operatingCFGrowthPer': -8.62, 'freeCashFlowGrowthPer': -7.249999999999999, 'capExAsPerOfSales': 2.9827089337175794, 'freeCFPerSales': 44.45244956772334, 'freeCashFlowPerNetIncome': 0.9750316055625791, 'freeCashFlowPerShare': 2.4851}, {'fiscalPeriodDate': '2016-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2016-09', 'operatingCFGrowthPer': -15.340000000000002, 'freeCashFlowGrowthPer': -18.14, 'capExAsPerOfSales': 3.467709852804668, 'freeCFPerSales': 33.490253282058084, 'freeCashFlowPerNetIncome': 0.843097980303789, 'freeCashFlowPerShare': 1.7954}, {'fiscalPeriodDate': '2017-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2017-09', 'operatingCFGrowthPer': 65.2, 'freeCashFlowGrowthPer': 68.30000000000001, 'capExAsPerOfSales': 3.851182045974507, 'freeCFPerSales': 46.30678723172459, 'freeCashFlowPerNetIncome': 1.2689953724436482, 'freeCashFlowPerShare': 3.4155}, {'fiscalPeriodDate': '2018-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2018-09', 'operatingCFGrowthPer': 38.06, 'freeCashFlowGrowthPer': 41.099999999999994, 'capExAsPerOfSales': 3.483914794507254, 'freeCFPerSales': 58.20272696394779, 'freeCashFlowPerNetIncome': 1.1644500533928746, 'freeCashFlowPerShare': 4.8056}, {'fiscalPeriodDate': '2019-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2019-09', 'operatingCFGrowthPer': 0.5599999999999999, 'freeCashFlowGrowthPer': 0.27999999999999997, 'capExAsPerOfSales': 3.290246768507638, 'freeCFPerSales': 52.34800017408713, 'freeCashFlowPerNetIncome': 0.9956953642384105, 'freeCashFlowPerShare': 4.9406}, {'fiscalPeriodDate': '2020-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2020-09', 'operatingCFGrowthPer': -18.34, 'freeCashFlowGrowthPer': -19.32, 'capExAsPerOfSales': 3.3690378101254232, 'freeCFPerSales': 44.420031126979765, 'freeCashFlowPerNetIncome': 0.8930609239830665, 
        'freeCashFlowPerShare': 5.1792}, {'fiscalPeriodDate': '2021-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2021-09', 'operatingCFGrowthPer': 45.85, 'freeCashFlowGrowthPer': 49.65, 'capExAsPerOfSales': 2.924704418170504, 'freeCFPerSales': 60.24476249740718, 'freeCashFlowPerNetIncome': 1.179595483713752, 'freeCashFlowPerShare': 5.7753}, {'fiscalPeriodDate': '2022-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2022-09', 'operatingCFGrowthPer': 23.79, 'freeCashFlowGrowthPer': 23.119999999999997, 'capExAsPerOfSales': 3.309450699419993, 'freeCFPerSales': 60.999658819515524, 'freeCashFlowPerNetIncome': 1.195360032091997, 'freeCashFlowPerShare': 7.4659}, {'fiscalPeriodDate': '2022-12-31T06:00:00.000', 'fiscalPeriodYearMonth': 'TTM', 'operatingCFGrowthPer': None, 'freeCashFlowGrowthPer': None, 'capExAsPerOfSales': 3.465067744393282, 'freeCFPerSales': 58.77364428396329, 'freeCashFlowPerNetIncome': 1.1690057323581735, 'freeCashFlowPerShare': 8.3703}]}   
                
        """
        return self.GetData("keyStats/cashFlow", url_suffixe='')
    
    def historical(self, start_date,end_date,frequency="daily"):
        """
        This function retrieves the historical price, volume and divide of the stock.

        Returns:
            list of dict with price, volume and dividend

        Examples:
            >>> Stock("visa", exchange="nyse").history(datetime.datetime.today()- datetime.timedelta(30),datetime.datetime.today())

            [{'open': 90.35, 'high': 90.65, 'low': 87.271, 'close': 88.08, 'volume': 34854776, 'previousClose': 89.12, 'date': '2023-01-04'}, {'open': 87.47, 'high': 87.57, 'low': 85.9, 'close': 86.2, 'volume': 27194375, 'previousClose': 88.08, 'date': '2023-01-05'}, {'open': 86.79, 'high': 87.69, 'low': 84.86, 'close': 87.34, 'volume': 41381495, 'previousClose': 86.2, 'date': '2023-01-06'}, {'open': 88.36, 'high': 90.05, 'low': 87.86, 'close': 88.02, 'volume': 29003901, 'previousClose': 87.34, 'date': '2023-01-09'}, {'open': 85.98, 'high': 88.67, 'low': 85.83, 'close': 88.42, 'volume': 30467755, 'previousClose': 88.02, 'date': '2023-01-10'}, {'open': 89.18, 'high': 91.6, 'low': 89.01, 'close': 91.52, 'volume': 26861969, 'previousClose': 88.42, 'date': '2023-01-11'}, {'open': 91.48, 'high': 91.87, 'low': 89.75, 'close': 91.13, 'volume': 30258135, 'previousClose': 
            91.52, 'date': '2023-01-12'}, {'open': 90.85, 'high': 92.19, 'low': 90.1299, 'close': 92.12, 'volume': 26329212, 'previousClose': 91.13, 'date': '2023-01-13'}, {'open': 92.06, 'high': 92.25, 'low': 90.05, 'close': 91.29, 'volume': 32602423, 'previousClose': 92.12, 'date': '2023-01-17'}, 
            {'open': 92.14, 'high': 92.7999, 'low': 90.64, 'close': 91.12, 'volume': 29116691, 'previousClose': 91.29, 'date': '2023-01-18'}, {'open': 90.72, 'high': 93.605, 'low': 90.63, 'close': 93.05, 'volume': 37000395, 'previousClose': 91.12, 'date': '2023-01-19'}, {'open': 95.1, 'high': 98.3, 
            'low': 95.02, 'close': 98.02, 'volume': 63191078, 'previousClose': 93.05, 'date': '2023-01-20'}, {'open': 97.95, 'high': 100.04, 'low': 97.5, 'close': 99.79, 'volume': 40005055, 'previousClose': 98.02, 'date': '2023-01-23'}, {'open': 98.1, 'high': 99.61, 'low': 97.2, 'close': 97.7, 'volume': 33078512, 'previousClose': 99.79, 'date': '2023-01-24'}, {'open': 95.57, 'high': 96.162, 'low': 93.76, 'close': 95.22, 'volume': 42329952, 
            'previousClose': 97.7, 'date': '2023-01-25'}, {'open': 96.5, 'high': 97.57, 'low': 95.38, 'close': 97.52, 'volume': 30113974, 'previousClose': 95.22, 'date': '2023-01-26'}, {'open': 97.31, 'high': 100.32, 'low': 97.31, 'close': 99.37, 'volume': 33879753, 'previousClose': 97.52, 'date': '2023-01-27'}, {'open': 97.48, 'high': 98.29, 'low': 96.395, 'close': 96.94, 'volume': 27226198, 'previousClose': 99.37, 'date': '2023-01-30'}, {'open': 96.87, 'high': 98.88, 'low': 96.82, 'close': 98.84, 'volume': 29870669, 'previousClose': 96.94, 'date': '2023-01-31'}, {'open': 98.71, 'high': 101.19, 'low': 97.58, 'close': 100.43, 'volume': 35531104, 'previousClose': 98.84, 'date': '2023-02-01'}, {'open': 105.8, 'high': 107.85, 'low': 105.61, 'close': 107.74, 'volume': 69883762, 'previousClose': 100.43, 'date': '2023-02-02'}, {'open': 102.93, 'high': 107.81, 'low': 102.58, 'close': 104.78, 'volume': 65309261, 'previousClose': 107.74, 'date': '2023-02-03'}, {'open': 102.4, 'high': 104.36, 'low': 101.88, 'close': 102.9, 'volume': 31999562, 'previousClose': 104.78, 'date': '2023-02-06'}, {'open': 103.22, 'high': 108.18, 'low': 103.12, 'close': 107.64, 'volume': 49010230, 'previousClose': 102.9, 'date': '2023-02-07'}, {'open': 102.05, 'high': 103.14, 'low': 98.035, 'close': 99.37, 'volume': 94743515, 'previousClose': 107.64, 'date': '2023-02-08'}, {'open': 100.0, 'high': 100.03, 'low': 93.63, 'close': 95.01, 'volume': 119455020, 'previousClose': 99.37, 'date': '2023-02-09'}, {'open': 95.45, 'high': 96.75, 'low': 94.25, 'close': 94.57, 'volume': 54980696, 'previousClose': 95.01, 
            'date': '2023-02-10'}, {'open': 94.74, 'high': 95.2, 'low': 93.84, 'close': 94.61, 'volume': 50076120, 'previousClose': 94.57, 'date': '2023-02-13'}, {'open': 94.43, 'high': 94.845, 'low': 92.26, 'close': 94.68, 'volume': 54726120, 'previousClose': 94.61, 'date': '2023-02-14'}, {'open': 
            94.49, 'high': 97.12, 'low': 94.15, 'close': 96.94, 'volume': 50298481, 'previousClose': 94.68, 'date': '2023-02-15'}, {'open': 95.37, 'high': 97.68, 'low': 94.74, 'close': 95.51, 'volume': 42462618, 'previousClose': 96.94, 'date': '2023-02-16'}, {'open': 94.85, 'high': 95.5599, 'low': 93.21, 'close': 94.35, 'volume': 34284118, 'previousClose': 95.51, 'date': '2023-02-17'}]       
        """
        return self.TimeSeries(["open","high","low","close",
                                "volume","previousClose","dividend"],
                                start_date=start_date,end_date=end_date,frequency=frequency)
    def incomeStatement(self, period='annual', reportType='original'):
        """
        This function retrieves the income statement.

        Args:
            period (str) : possible values are annual, quarterly
            reportType (str) : possible values are original, restated

        Returns:
            dict with income statement

        Examples:
            >>> Stock("visa", exchange="nyse").incomeStatement('quarterly', 'original')
            >>> Stock("visa", exchange="nyse").incomeStatement('annual', 'restated')

            {'_meta': {'companyId': '0C00000VF2', 'statementType': 'income-statement', 'periodReport': 'Success', 'latestReport': 'Success'}, 'columnDefs': 
            ['2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', 'TTM'], 'filingIdList': [None, None, None, None, None, '199364939', '251173203', '309018162', '364101718', '417618183', None], 'rows': [{'parentId': '', 'label': 'IncomeStatement', 'dataPointId': 'IFIS000000', 'order': 385, 'subLevel': [{'parentId': 'IFIS000000385', 'label': 'Gross Profit', 'dataPointId': 'IFIS000590', 'order': 386, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 16753.0, 18812.0, 17334.0, 19135.0, 23577.0, 24254.0], 'subLevel': [{'parentId': 'IFIS000590386', 'label': 'Total Revenue', 'dataPointId': 'IFIS001170', 'order': 387, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 20609.0, 22977.0, 
            21846.0, 24105.0, 29310.0, 30187.0], 'subLevel': [{'parentId': 'IFIS001170387', 'label': 'Business Revenue', 'dataPointId': 'IFIS001839', 'order': 388, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 25156.0, 27837.0, 27078.0, 30797.0, 37614.0, 38768.0]}, {'parentId': 'IFIS001170387', 'label': 'Sales Return and Allowances/Discounts', 'dataPointId': 'IFIS002095', 'order': 390, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -5491.0, -6173.0, -6664.0, -8367.0, -10295.0, -10710.0]}, {'parentId': 'IFIS001170387', 'label': 'Other Revenue', 'dataPointId': 'IFIS002228', 'order': 391, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 944.0, 1313.0, 1432.0, 1675.0, 1991.0, 2129.0]}]}, {'parentId': 'IFIS000590386', 'label': 'Cost of Revenue', 'dataPointId': 'IFIS000200', 'order': 392, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -3856.0, -4165.0, -4512.0, -4970.0, -5733.0, -5933.0], 'subLevel': [{'parentId': 'IFIS000200392', 'label': 'Cost of Goods and Services', 'dataPointId': 'IFIS003118', 'order': 393, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -686.0, -721.0, -727.0, -730.0, -743.0, -731.0]}, {'parentId': 'IFIS000200392', 'label': 'Staff Cost, Cost of Revenue', 'dataPointId': 'IFIS002720', 'order': 402, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -3170.0, -3444.0, -3785.0, -4240.0, -4990.0, -5202.0]}]}]}, {'parentId': 'IFIS000000385', 'label': 'Operating Income/Expenses', 'dataPointId': 'IFIS001220', 'order': 405, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -3192.0, -3411.0, -3242.0, -3328.0, -3896.0, -4066.0], 'subLevel': [{'parentId': 'IFIS001220405', 'label': 'Selling, General and 
            Administrative Expenses', 'dataPointId': 'IFIS001700', 'order': 406, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -2579.0, -2755.0, -2475.0, -2524.0, -3035.0, -3176.0], 'subLevel': [{'parentId': 'IFIS001700406', 'label': 'General and Administrative Expenses', 'dataPointId': 'IFIS000040', 'order': 414, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -1145.0, -1196.0, -1096.0, -985.0, -1194.0, -1274.0]}, 
            {'parentId': 'IFIS001700406', 'label': 'Legal, Accounting, Auditing and Consulting Fee', 'dataPointId': 'IFIS002081', 'order': 416, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -446.0, -454.0, -408.0, -403.0, -505.0, -514.0]}, {'parentId': 'IFIS001700406', 'label': 'Selling and Marketing Expenses', 'dataPointId': 'IFIS001690', 'order': 428, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -988.0, -1105.0, -971.0, -1136.0, -1336.0, -1388.0]}]}, {'parentId': 'IFIS001220405', 'label': 'Depreciation, Amortization and Depletion', 'dataPointId': 'IFIS001855', 'order': 432, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -613.0, -656.0, -767.0, -804.0, -861.0, -890.0], 'subLevel': [{'parentId': 'IFIS001855432', 'label': 'Depreciation and Amortization', 'dataPointId': 'IFIS000250', 'order': 433, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -613.0, -656.0, -767.0, -804.0, -861.0, -890.0]}]}]}, {'parentId': 'IFIS000000385', 'label': 'Total Operating Profit/Loss', 'dataPointId': 'IFIS001230', 'order': 446, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 13561.0, 15401.0, 14092.0, 15807.0, 19681.0, 20188.0]}, {'parentId': 'IFIS000000385', 'label': 'Non-Operating Income/Expense, Total', 'dataPointId': 'IFIS001917', 'order': 447, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -755.0, -517.0, -302.0, 256.0, -1545.0, -1972.0], 'subLevel': [{'parentId': 'IFIS001917447', 'label': 'Total Net Finance Income/Expense', 'dataPointId': 'IFIS000930', 'order': 448, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -612.0, -533.0, -516.0, -513.0, -538.0, -541.0], 'subLevel': [{'parentId': 'IFIS000930448', 'label': 'Net Interest Income/Expense', 'dataPointId': 'IFIS002367', 'order': 449, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -612.0, -533.0, -516.0, -513.0, -538.0, -541.0], 'subLevel': [{'parentId': 'IFIS002367449', 'label': 'Interest Expense Net of Capitalized Interest', 'dataPointId': 'IFIS000800', 'order': 450, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -612.0, None, -516.0, -513.0, -538.0, -541.0]}]}]}, {'parentId': 'IFIS001917447', 'label': 'Net Investment Income', 'dataPointId': 'IFIS001915', 'order': 461, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, 416.0, 225.0, 772.0, -139.0, -370.0]}, {'parentId': 
            'IFIS001917447', 'label': 'Irregular Income/Expense', 'dataPointId': 'IFIS101189', 'order': 480, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -607.0, -400.0, -11.0, -3.0, -868.0, -1061.0], 'subLevel': [{'parentId': 'IFIS101189480', 'label': 'Litigation Income/Expense', 'dataPointId': 'IFIS002698', 'order': 502, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -607.0, -400.0, -11.0, -3.0, -868.0, -1061.0]}, {'parentId': 'IFIS101189480', 'label': 'Other Irregular Income/Expense', 'dataPointId': 'IFIS003107', 'order': 507, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 0.0, None, None, None, None, None]}]}, {'parentId': 'IFIS001917447', 'label': 'Other Income/Expense, Non-Operating', 'dataPointId': 'IFIS200292', 'order': 514, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 464.0, None, None, None, None, None]}]}, {'parentId': 'IFIS000000385', 'label': 'Pretax Income', 'dataPointId': 'IFIS001490', 'order': 518, 'collapsed': True, 
            'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 12806.0, 14884.0, 13790.0, 16063.0, 18136.0, 18216.0]}, {'parentId': 'IFIS000000385', 'label': 'Provision for Income Tax', 'dataPointId': 'IFIS001560', 'order': 519, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -2505.0, -2804.0, -2924.0, -3752.0, -3179.0, -3039.0]}, {'parentId': 'IFIS000000385', 'label': 'Net Income before Extraordinary Items and Discontinued Operations', 'dataPointId': 'IFIS001110', 'order': 529, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 10301.0, 12080.0, 10866.0, 12311.0, 14957.0, 15177.0]}, {'parentId': 'IFIS000000385', 'label': 'Net Income after Extraordinary Items and Discontinued Operations', 'dataPointId': 
            'IFIS001977', 'order': 535, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 10301.0, 12080.0, 10866.0, 12311.0, 14957.0, 15177.0]}, {'parentId': 'IFIS000000385', 'label': 'Net Income after Non-Controlling/Minority Interests', 'dataPointId': 'IFIS001090', 'order': 537, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 10301.0, 12080.0, 10866.0, 12311.0, 14957.0, 15177.0]}, {'parentId': 'IFIS000000385', 'label': 'Net Income Available to Common Stockholders', 'dataPointId': 'IFIS001100', 'order': 543, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 9942.0, 12080.0, 10866.0, 12311.0, 14957.0, 15177.0]}, {'parentId': 'IFIS000000385', 'label': 'Diluted Net 
            Income Available to Common Stockholders', 'dataPointId': 'IFIS002230', 'order': 545, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 10301.0, 12080.0, 10866.0, 12311.0, 14957.0, 15177.0]}, {'parentId': 'IFIS000000385', 'label': 'Income Statement Supplemental Section', 'dataPointId': 'IFIS002156', 'order': 560, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None], 'subLevel': 
            [{'parentId': 'IFIS002156560', 'label': 'Reported Normalized and Operating Income/Expense Supplemental Section', 'dataPointId': 'IFIS003147', 'order': 561, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None], 'subLevel': [{'parentId': 'IFIS003147561', 'label': 'Total Revenue as Reported, Supplemental', 'dataPointId': 'IFIS003203', 'order': 562, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 20609.0, 22977.0, 21846.0, None, 29310.0, 30187.0]}, {'parentId': 'IFIS003147561', 'label': 'Reported Operating Expense', 'dataPointId': 'IFIS003137', 'order': 563, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 7655.0, -7976.0, -7765.0, -8301.0, -10497.0, -11060.0]}, {'parentId': 'IFIS003147561', 'label': 'Reported Total Operating Profit/Loss', 'dataPointId': 'IFIS003138', 'order': 564, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 12954.0, 15001.0, 14081.0, 15804.0, 18813.0, 19127.0]}, {'parentId': 'IFIS003147561', 'label': 'Reported Normalized Income', 'dataPointId': 'IFIS002229', 'order': 565, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 10729.0, 12367.0, 11193.0, 12933.0, 16034.0, 16714.0]}, {'parentId': 'IFIS003147561', 'label': 'Reported Effective Tax Rate', 'dataPointId': 'IFIS002303', 'order': 566, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 0.196, 0.188, 0.212, 0.234, 0.175, None]}, {'parentId': 'IFIS003147561', 'label': 'Reported Normalized Operating Profit', 'dataPointId': 'IFIS003202', 'order': 572, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 13749.0, 15374.0, 14144.0, None, None, None]}]}]}, {'parentId': 'IFIS000000385', 'label': 'Other Adjustments to Net Income Available to Common Stockholders', 'dataPointId': 'IFIS003108', 'order': 542, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', -359.0, None, None, None, None, None]}, {'parentId': 'IFIS000000385', 'label': 'Dilution to Earnings', 'dataPointId': 'IFIS101211', 'order': 544, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 359.0, None, None, None, None, None]}, {'parentId': 'IFIS000000385', 'label': 'Non-Controlling/Minority Interests', 'dataPointId': 'IFIS001040', 'order': 536, 'collapsed': True, 
            'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', None, None, None, None, None, None]}]}, {'parentId': '', 'label': 'EpsAndWaso', 'dataPointId': 'IFES000000', 'order': 935, 'subLevel': [{'parentId': 'IFES000000935', 'label': 'Basic EPS', 'dataPointId': 'IFIS000100', 'order': 936, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 4.43, 5.32, 4.9, 5.63, 7.01, 7.15], 'subLevel': [{'parentId': 'IFIS000100936', 'label': 'Basic EPS from Continuing Operations', 'dataPointId': 'IFIS000120', 'order': 937, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 4.43, 5.32, 4.9, 5.63, 7.01, 7.15]}]}, {'parentId': 'IFES000000935', 'label': 'Diluted EPS', 'dataPointId': 'IFIS000280', 'order': 939, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 4.42, 5.32, 4.89, 5.63, 7.0, 7.15], 'subLevel': [{'parentId': 'IFIS000280939', 'label': 'Diluted EPS from Continuing Operations', 'dataPointId': 'IFIS000300', 'order': 940, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 4.42, 5.32, 4.89, 5.63, 7.0, 7.15]}]}, {'parentId': 'IFES000000935', 'label': 'Basic Weighted Average Shares Outstanding', 'dataPointId': 'IFIS000150', 'order': 942, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 2239.0, 2187.59, 2138.59, 2130.3, 2088.0, 2078.0]}, {'parentId': 'IFES000000935', 'label': 'Diluted Weighted Average Shares Outstanding', 'dataPointId': 'IFIS000330', 'order': 943, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 2329.0, 2272.0, 2223.0, 2188.0, 2136.0, 2121.75]}, {'parentId': 'IFES000000935', 'label': 'Reported 
            Normalized Diluted EPS', 'dataPointId': 'IFIS101199', 'order': 945, 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]}, {'parentId': 'IFES000000935', 'label': 'Total Dividend Per Share', 'dataPointId': 'ORISZ00026', 'order': 950, 'datum': 
            ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 'subLevel': [{'parentId': 'ORISZ00026950', 'label': 'Regular Dividend Per Share Calc', 'dataPointId': 'ORISZ00021', 'order': 951, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]}]}]}, 
            {'parentId': '', 'label': 'WasoAndEpsData', 'dataPointId': 'SAL0001000', 'collapsed': False, 'subLevel': [{'parentId': 'SAL0001000', 'label': 'Basic EPS', 'dataPointId': 'SAL0001001', 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 4.43, 5.32, 4.9, 5.63, 7.01, 7.16]}, {'parentId': 'SAL0001000', 'label': 'Diluted EPS', 'dataPointId': 'SAL0001002', 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 4.42, 5.32, 4.89, 5.63, 7.0, 7.16]}, {'parentId': 'SAL0001000', 'label': 'Basic WASO', 'dataPointId': 'SAL0001003', 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 2239.0, 2187.59, 2138.59, 2130.3, 2088.0, 2078.0]}, {'parentId': 'SAL0001000', 'label': 'Diluted WASO', 'dataPointId': 'SAL0001004', 'collapsed': True, 'datum': ['_PO_', '_PO_', '_PO_', '_PO_', '_PO_', 2329.0, 2272.0, 2223.0, 2188.0, 2136.0, 2121.75]}]}], 'footer': {'currency': 'USD', 'currencySymbol': '$', 'orderOfMagnitude': 'Million', 'fiscalYearEndDate': '09-30'}, 'userType': 'Free'}
        """
        
        return self.financialStatement("incomestatement",period=period, reportType=reportType)
    
    def institutionBuyers(self, top=20):
        """
        This function retrieves the institutions which buy the stock.

        Args:
            top (int) : number of institutions to return
        Returns:
            dict with the buyers

        Examples:
            >>> Stock("visa", exchange="nyse").institutionBuyers(top=50)

        {'isRestricted': False, 'userType': 'Free', 'country': 'USA', 'rows': [{'secId': None, 'name': 'Viking Global Investors LP', 'securityType': None, 'totalSharesHeld': 0.2543828813360818, 'totalAssets': 4.38544, 'currentShares': 5346584, 'changeAmount': 2790724, 'changePercentage': 109.18923571713631, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Mitsubishi UFJ Securities International PLC', 'securityType': None, 'totalSharesHeld': 1.0422728633881277, 'totalAssets': 71.30217, 'currentShares': 21906346, 'changeAmount': 2452169, 'changePercentage': 12.604845735699845, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 
        'T. Rowe Price Investment Management,Inc.', 'securityType': None, 'totalSharesHeld': 0.11156420807285163, 'totalAssets': 0.33278, 'currentShares': 2344841, 'changeAmount': 2344841, 'changePercentage': None, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'T. Rowe Price Associates, Inc.', 'securityType': None, 'totalSharesHeld': 2.4312719733470267, 'totalAssets': 1.70793, 'currentShares': 51100136, 'changeAmount': 2216275, 'changePercentage': 4.533756038623872, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Vanguard Group Inc', 'securityType': None, 'totalSharesHeld': 7.011028911161065, 'totalAssets': 0.83359, 'currentShares': 147356830, 'changeAmount': 2119920, 'changePercentage': 1.459628960709781, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': None, 'securityType': None, 'totalSharesHeld': 10.850520837305153, 'totalAssets': None, 'currentShares': 228054737, 'changeAmount': 11923929, 'changePercentage': None, 'date': None, 'trend': '_PO_', 'starRating': None}], 'columnDefs': [{'columnId': 'name', 'dataType': 'string'}, {'columnId': 'totalSharesHeld', 'dataType': 'number'}, {'columnId': 'totalAssets', 'dataType': 'number'}, {'columnId': 'trend', 'dataType': 'number'}, {'columnId': 'currentShares', 'dataType': 'number'}, {'columnId': 'changeAmount', 'dataType': 'number'}, {'columnId': 'changePercentage', 'dataType': 'number'}, {'columnId': 'date', 'dataType': 'string'}]}
                
        """
        
        if not isinstance(top, int):
            raise TypeError('top parameter should be an integer')
        
        return self.GetData("ownership/v1", url_suffixe= f"Buyers/institution/{top}/data")
    
    def institutionConcentratedOwners(self, top=20):
        """
        This function retrieves the institutions which are concentrated on the stock.

        Args:
            top (int) : number of institutions to return
        Returns:
            dict with the concentarted owners

        Examples:
            >>> Stock("visa", exchange="nyse").institutionConcentratedOwners(top=50)

        {'isRestricted': False, 'userType': 'Free', 'country': 'USA', 'rows': [{'secId': None, 'name': 'Mitsubishi UFJ Securities International PLC', 'securityType': None, 'totalSharesHeld': 1.0422728633881277, 'totalAssets': 71.30217, 'currentShares': 21906346, 'changeAmount': 2452169, 'changePercentage': 12.604845735699845, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Hengistbury Investment Partners LLP', 'securityType': None, 'totalSharesHeld': 0.02437826589016468, 'totalAssets': 37.9333, 'currentShares': 512379, 'changeAmount': 1500, 'changePercentage': 0.2936115988326003, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Center Lake Capital Management, LLC', 'securityType': None, 'totalSharesHeld': 0.004519965220209346, 'totalAssets': 29.25954, 'currentShares': 95000, 'changeAmount': 65404, 'changePercentage': 220.98932288147046, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'NAYA CAPITAL MANAGEMENT UK LTD', 'securityType': None, 'totalSharesHeld': 0.06070213375720495, 'totalAssets': 24.97845, 'currentShares': 1275829, 'changeAmount': 146471, 'changePercentage': 12.969403855996061, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Atalan Capital Partners, LP', 'securityType': None, 'totalSharesHeld': 0.04115547279453773, 'totalAssets': 17.03327, 'currentShares': 865000, 'changeAmount': 10000, 'changePercentage': 1.1695906432748537, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Two Creeks Capital Management, LP', 'securityType': None, 'totalSharesHeld': 0.05209659576373901, 'totalAssets': 16.22311, 'currentShares': 1094959, 'changeAmount': 4569, 'changePercentage': 0.419024385770229, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Triple Frond Partners LLC', 'securityType': None, 'totalSharesHeld': 0.0222085398487204, 'totalAssets': 14.60452, 'currentShares': 466776, 'changeAmount': -16931, 'changePercentage': -3.500259454587178, 
        'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'KADENSA CAPITAL Ltd', 'securityType': None, 'totalSharesHeld': 0.01078392333670052, 'totalAssets': 14.12172, 'currentShares': 226655, 'changeAmount': 96526, 'changePercentage': 74.1771626616665, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'BlueSpruce Investments, LP', 'securityType': None, 'totalSharesHeld': 0.16686055858384874, 'totalAssets': 14.00657, 'currentShares': 3507052, 'changeAmount': 691380, 'changePercentage': 24.55470665617302, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'TRB Advisors LP', 'securityType': None, 'totalSharesHeld': 0.0038062865012289233, 'totalAssets': 13.96825, 'currentShares': 80000, 'changeAmount': 0, 'changePercentage': 
        0.0, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': None, 'securityType': None, 'totalSharesHeld': 1.428784605084482, 'totalAssets': None, 'currentShares': 30029996, 'changeAmount': 3451088, 'changePercentage': None, 'date': None, 'trend': '_PO_', 'starRating': None}], 'columnDefs': [{'columnId': 'name', 'dataType': 'string'}, {'columnId': 'totalSharesHeld', 'dataType': 'number'}, {'columnId': 'totalAssets', 'dataType': 'number'}, {'columnId': 'trend', 'dataType': 'number'}, {'columnId': 'currentShares', 'dataType': 'number'}, {'columnId': 'changeAmount', 'dataType': 'number'}, {'columnId': 'changePercentage', 'dataType': 'number'}, {'columnId': 'date', 'dataType': 'string'}]}    
                
        """
        
        if not isinstance(top, int):
            raise TypeError('top parameter should be an integer')
        
        return self.GetData("ownership/v1", url_suffixe= f"ConcentratedOwners/institution/{top}/data")
    
    def institutionOwnership(self, top=20):
        """
        This function retrieves the main institutions which own the stock.

        Args:
            top (int) : number of institutions to return
        Returns:
            dict with the main owners

        Examples:
            >>> Stock("visa", exchange="nyse").institutionOwnership(top=50)

        {'isRestricted': False, 'userType': 'Free', 'country': 'USA', 'rows': [{'secId': None, 'name': 'Vanguard Group Inc', 'securityType': None, 'totalSharesHeld': 7.011028911161065, 'totalAssets': 0.83359, 'currentShares': 147356830, 'changeAmount': 2119920, 'changePercentage': 1.459628960709781, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'BlackRock Inc', 'securityType': None, 'totalSharesHeld': 5.852905961099703, 'totalAssets': 0.74326, 'currentShares': 123015563, 'changeAmount': -1152338, 'changePercentage': -0.9280482239930914, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'State Street Corporation', 'securityType': None, 'totalSharesHeld': 3.345961491293231, 'totalAssets': 0.85849, 'currentShares': 70324953, 'changeAmount': -179343, 'changePercentage': -0.254371733603297, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'T. Rowe Price Associates, Inc.', 'securityType': None, 'totalSharesHeld': 2.4312719733470267, 'totalAssets': 1.70793, 'currentShares': 51100136, 'changeAmount': 2216275, 'changePercentage': 4.533756038623872, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'FMR 
        Inc', 'securityType': None, 'totalSharesHeld': 2.28358524996305, 'totalAssets': 0.88917, 'currentShares': 47996077, 'changeAmount': -595703, 'changePercentage': -1.225933686726438, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Morgan Stanley - Brokerage Accounts', 'securityType': None, 'totalSharesHeld': 2.1533364572414784, 'totalAssets': 1.09582, 'currentShares': 45258526, 'changeAmount': 657493, 'changePercentage': 1.4741654077832682, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': 
        None, 'name': 'Geode Capital Management, LLC', 'securityType': None, 'totalSharesHeld': 1.502546202984566, 'totalAssets': 0.85023, 'currentShares': 31580307, 'changeAmount': 499601, 'changePercentage': 1.6074313112449892, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': 
        None}, {'secId': None, 'name': 'Bank of America Corp', 'securityType': None, 'totalSharesHeld': 1.2917595280833536, 'totalAssets': 0.57641, 'currentShares': 27150022, 'changeAmount': 742174, 'changePercentage': 2.8104296874171646, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'AllianceBernstein L.P.', 'securityType': None, 'totalSharesHeld': 1.2458379660677208, 'totalAssets': 2.44834, 'currentShares': 26184849, 'changeAmount': -956951, 'changePercentage': -3.5257462659071983, 'date': '2022-12-31T00:00:00.000', 'trend': 
        '_PO_', 'starRating': None}, {'secId': None, 'name': 'Wellington Management Company LLP', 'securityType': None, 'totalSharesHeld': 1.1225778484124742, 'totalAssets': 0.89278, 'currentShares': 23594185, 'changeAmount': 229058, 'changePercentage': 0.9803413437470295, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Massachusetts Financial Services Company', 'securityType': None, 'totalSharesHeld': 1.1053325158470313, 'totalAssets': 1.68488, 'currentShares': 23231725, 'changeAmount': -58198, 'changePercentage': -0.24988489657093327, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Mitsubishi UFJ Securities International PLC', 'securityType': None, 'totalSharesHeld': 1.0422728633881277, 'totalAssets': 71.30217, 'currentShares': 21906346, 'changeAmount': 2452169, 'changePercentage': 12.604845735699845, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'TCI Fund Management Limited', 'securityType': None, 'totalSharesHeld': 0.9485656581214665, 'totalAssets': 13.83942, 'currentShares': 19936821, 'changeAmount': 0, 'changePercentage': 0.0, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Northern Trust Corp', 'securityType': None, 'totalSharesHeld': 0.880019148513879, 'totalAssets': 0.80642, 'currentShares': 18496120, 'changeAmount': -34029, 'changePercentage': -0.18364126483818344, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Capital Research & Mgmt Co - Division 3', 'securityType': None, 'totalSharesHeld': 0.7902599187634549, 'totalAssets': 0.92634, 'currentShares': 16609573, 'changeAmount': 41364, 'changePercentage': 0.24965884966806007, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Goldman Sachs Group Inc', 'securityType': None, 'totalSharesHeld': 0.703909504046369, 'totalAssets': 0.68798, 'currentShares': 14794672, 'changeAmount': -5193505, 'changePercentage': -25.98288478233908, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'NORGES BANK', 'securityType': None, 'totalSharesHeld': 0.7024189622524877, 'totalAssets': 0.73934, 'currentShares': 14763344, 'changeAmount': 2117058, 'changePercentage': 16.74055133657423, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Nuveen Asset Management, LLC', 'securityType': None, 'totalSharesHeld': 0.6635343199845831, 'totalAssets': 1.00379, 'currentShares': 13946072, 'changeAmount': -306372, 'changePercentage': -2.1496102703508253, 'date': '2022-09-30T00:00:00.000', 
        'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Bank of New York Mellon Corp', 'securityType': None, 'totalSharesHeld': 0.6413530902415091, 'totalAssets': 0.63956, 'currentShares': 13479870, 'changeAmount': 98388, 'changePercentage': 0.7352548843244717, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Jennison Associates LLC', 'securityType': None, 'totalSharesHeld': 0.6298207082429231, 'totalAssets': 2.78714, 'currentShares': 13237484, 'changeAmount': -32643, 'changePercentage': -0.24598860282196244, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': None, 'securityType': None, 'totalSharesHeld': 36.3482982790555, 'totalAssets': None, 'currentShares': 763963475, 'changeAmount': 2664418, 'changePercentage': None, 'date': None, 'trend': '_PO_', 
        'starRating': None}], 'columnDefs': [{'columnId': 'name', 'dataType': 'string'}, {'columnId': 'totalSharesHeld', 'dataType': 'number'}, {'columnId': 'totalAssets', 'dataType': 'number'}, {'columnId': 'trend', 'dataType': 'number'}, {'columnId': 'currentShares', 'dataType': 'number'}, {'columnId': 'changeAmount', 'dataType': 'number'}, {'columnId': 'changePercentage', 'dataType': 'number'}, {'columnId': 'date', 'dataType': 'string'}]}   
                
        """
        
        if not isinstance(top, int):
            raise TypeError('top parameter should be an integer')
        
        return self.GetData("ownership/v1", url_suffixe= f"OwnershipData/institution/{top}/data")
    
    def institutionSellers(self, top=20):
        """
        This function retrieves the institutions which sell on the stock.

        Args:
            top (int) : number of institutions to return
        Returns:
            dict with sellers

        Examples:
            >>> Stock("visa", exchange="nyse").institutionSellers(top=50)

        {'isRestricted': False, 'userType': 'Free', 'country': 'USA', 'rows': [{'secId': None, 'name': 'Goldman Sachs Group Inc', 'securityType': None, 
        'totalSharesHeld': 0.703909504046369, 'totalAssets': 0.68798, 'currentShares': 14794672, 'changeAmount': -5193505, 'changePercentage': -25.98288478233908, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Ensign Peak Advisors Inc', 'securityType': None, 'totalSharesHeld': 0.0, 'totalAssets': None, 'currentShares': 0, 'changeAmount': -2395335, 'changePercentage': -100.0, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Polen Capital', 'securityType': None, 'totalSharesHeld': 
        0.48364427176405267, 'totalAssets': 5.05013, 'currentShares': 10165168, 'changeAmount': -2313368, 'changePercentage': -18.53877730528645, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Allianz Asset Management AG', 'securityType': None, 
        'totalSharesHeld': 0.1066410619549996, 'totalAssets': 0.80972, 'currentShares': 2241367, 'changeAmount': -1980352, 'changePercentage': -46.9086644563506, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': 'Magellan Asset Management Limited', 
        'securityType': None, 'totalSharesHeld': 0.25965620581204685, 'totalAssets': 6.77626, 'currentShares': 5457418, 'changeAmount': -1663319, 'changePercentage': -23.358804011438703, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': None}, {'secId': None, 'name': None, 'securityType': None, 'totalSharesHeld': 1.5538510435774682, 'totalAssets': None, 'currentShares': 32658625, 'changeAmount': -13545879, 'changePercentage': None, 'date': None, 'trend': '_PO_', 'starRating': None}], 'columnDefs': [{'columnId': 'name', 'dataType': 'string'}, {'columnId': 'totalSharesHeld', 'dataType': 'number'}, {'columnId': 'totalAssets', 'dataType': 'number'}, {'columnId': 'trend', 'dataType': 'number'}, {'columnId': 
        'currentShares', 'dataType': 'number'}, {'columnId': 'changeAmount', 'dataType': 'number'}, {'columnId': 'changePercentage', 'dataType': 'number'}, {'columnId': 'date', 'dataType': 'string'}]}
                
        """
        if not isinstance(top, int):
            raise TypeError('top parameter should be an integer')
        
        return self.GetData("ownership/v1", url_suffixe= f"Sellers/institution/{top}/data")
    
    def keyExecutives(self):
        """
        This function retrieves information oabout key excutives of the company.

        Returns:
            dict with key executives information

        Examples:
            >>> Stock("visa", exchange="nyse").keyExecutives()

        {'rows': [{'type': 'person', 'personId': 'PS00008TV4', 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'holding': '0.0000', 'memberSince': '2013', 'age': '47', 'totalCompensation': ['13286026.00', '12088417.00', '14952045.00', '17815022.00', '17104605.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': ['900035.00', '900047.00', '1100050.00', '1100019.00', '1104239.00']}, {'nameId': 
        'bonus', 'name': 'Bonus', 'datum': [None, None, None, None, None]}, {'nameId': 'otherAnnualIncome', 'name': 'Annual Other Income', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': ['6777537.00', '7133141.00', '9439901.00', '9845103.00', '9114177.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': ['2250003.00', '1825008.00', '2624994.00', '3212500.00', '3337503.00']}, {'nameId': 'LTIP-Payout', 'name': 'LTIP Payout', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': ['3333600.00', '2196000.00', '1760000.00', '3630000.00', '3520000.00']}, {'nameId': 'otherCompensation', 'name': 'Other Compensation', 'datum': ['24851.0', '34221.0', '27100.00', '27400.00', '28686.00']}]}, {'type': 'person', 'personId': 'PS00001RQY', 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'holding': '93060.0000', 'memberSince': '2015', 'age': '63', 'totalCompensation': ['12569940.00', '11586718.00', '13048644.00', '15573437.00', '14536210.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': ['1000038.00', '1000040.00', '1100050.00', '1100019.00', '1104239.00']}, {'nameId': 'bonus', 'name': 'Bonus', 'datum': [None, None, None, None, None]}, {'nameId': 'otherAnnualIncome', 'name': 'Annual Other Income', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': ['5848193.00', '6493501.00', '8039757.00', '8187025.00', '7263594.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': ['1999994.00', '1625005.00', '2124987.00', '2624993.00', '2624981.00']}, {'nameId': 'LTIP-Payout', 'name': 'LTIP Payout', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': ['3704000.00', '2440000.00', '1760000.00', '3630000.00', '3520000.00']}, {'nameId': 'otherCompensation', 'name': 'Other Compensation', 'datum': ['17715.0', '28172.0', '23850.00', '31400.00', '23396.00']}]}, {'type': 'person', 'personId': 
        'PS00007VT2', 'name': 'Rajat Taneja', 'title': 'President, Technology', 'holding': '232112.0000', 'memberSince': '2013', 'age': '58', 'totalCompensation': ['13154517.00', '11810570.00', '14320558.00', '17130343.00', '16354828.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 
        'datum': ['900035.00', '900047.00', '1100050.00', '1100019.00', '1104239.00']}, {'nameId': 'bonus', 'name': 'Bonus', 'datum': [None, None, None, None, None]}, {'nameId': 'otherAnnualIncome', 'name': 'Annual Other Income', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': ['6803356.00', '6977836.00', '8939651.00', '9329605.00', '8585208.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': ['2125007.00', '1712503.00', '2500007.00', '3049986.00', '3124998.00']}, {'nameId': 'LTIP-Payout', 'name': 'LTIP Payout', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': ['3306600.00', '2196000.00', '1760000.00', '3630000.00', '3520000.00']}, {'nameId': 'otherCompensation', 'name': 'Other Compensation', 'datum': ['19519.0', '24184.0', '20850.00', '20733.00', '20383.00']}]}, {'type': 'person', 'personId': 'PS00008OTS', 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'holding': '53114.0000', 'memberSince': '2014', 'age': '56', 'totalCompensation': ['6422617.00', '6343064.00', '6723943.00', '8180067.00', '9341805.00'], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': ['675026.00', '675027.00', '725033.00', '779535.00', '853282.00']}, {'nameId': 'bonus', 'name': 'Bonus', 'datum': [None, None, None, None, None]}, {'nameId': 'otherAnnualIncome', 'name': 'Annual Other Income', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': ['3075525.00', '3581544.00', '4055546.00', '4119771.00', '4316984.00']}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': ['999997.00', '950001.00', '1000014.00', '1262494.00', '1762489.00']}, {'nameId': 'LTIP-Payout', 'name': 'LTIP Payout', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': ['1636767.00', '1111725.00', '913500.00', '1980000.00', '2380000.00']}, {'nameId': 'otherCompensation', 'name': 'Other 
        Compensation', 'datum': ['35302.0', '24767.0', '29850.00', '38267.00', '29050.00']}]}, {'type': 'person', 'personId': 'PS0000A8VE', 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'holding': '32467.0000', 'memberSince': '2019', 'age': '57', 'totalCompensation': [None, None, '23059878.00', None, None], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': [None, None, '750035.00', None, None]}, {'nameId': 'bonus', 'name': 'Bonus', 'datum': [None, None, None, None, None]}, {'nameId': 'otherAnnualIncome', 'name': 'Annual Other Income', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': [None, None, '9128814.00', None, None]}, 
        {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, '3381227.00', None, None]}, {'nameId': 'LTIP-Payout', 'name': 'LTIP Payout', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, '900000.00', None, None]}, {'nameId': 'otherCompensation', 'name': 'Other Compensation', 'datum': [None, None, '8899802.00', None, None]}]}, {'type': 'person', 'personId': 'PS0000DTTA', 'name': 'Peter Andreski', 'title': 'Senior Vice President, Global Corporate Controller and Chief Accounting Officer', 'holding': '5711.0000', 'memberSince': '2019', 'age': '49', 'totalCompensation': [None, None, None, None, None], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': [None, None, None, None, None]}, {'nameId': 'bonus', 'name': 'Bonus', 'datum': [None, None, None, None, None]}, {'nameId': 'otherAnnualIncome', 'name': 'Annual Other Income', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': [None, None, None, None, None]}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'LTIP-Payout', 'name': 'LTIP Payout', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'otherCompensation', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'person', 'personId': 'PS0000CUNJ', 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'holding': '4583.0000', 'memberSince': '2008', 'age': '54', 'totalCompensation': [None, None, None, None, None], 'compensation': [{'nameId': 'salary', 'name': 'Salary', 'datum': [None, None, None, None, None]}, {'nameId': 'bonus', 'name': 'Bonus', 'datum': [None, None, None, None, None]}, {'nameId': 'otherAnnualIncome', 'name': 'Annual Other Income', 'datum': [None, None, None, None, None]}, {'nameId': 'restrictedStockAward', 'name': 'Restricted Stock Award', 'datum': [None, None, None, None, None]}, {'nameId': 'securitiesOptions', 'name': 'Securities Options', 'datum': [None, None, None, None, None]}, {'nameId': 'LTIP-Payout', 'name': 'LTIP Payout', 'datum': [None, None, None, None, None]}, {'nameId': 'non-EquityCompensation', 'name': 'Non-Equity Compensation', 'datum': [None, None, None, None, None]}, {'nameId': 'otherCompensation', 'name': 'Other Compensation', 'datum': [None, None, None, None, None]}]}, {'type': 'total', 'personId': 'totalKeyCompensation', 'name': 'Compensation for all Key Executives', 'totalCompensation': ['45433100.00', '41828769.00', '72105068.00', '58698869.00', '57337448.00'], 'compensation': [{'nameId': 'totalSalary', 'name': 'Salary', 'datum': ['3475134.00', '3475161.00', '4775218.00', '4079592.00', '4165999.00']}, {'nameId': 'totalBonus', 'name': 'Bonus', 'datum': [None, None, None, None, None]}, {'nameId': 'totalOtherAnnualIncome', 'name': 'Annual Other Income', 'datum': [None, None, None, None, None]}, {'nameId': 'totalStockAward', 'name': 'Restricted Stock Award', 'datum': ['22504611.00', '24186022.00', '39603669.00', '31481504.00', '29279963.00']}, {'nameId': 'totalOptions', 'name': 'Securities Options', 'datum': ['7375001.00', '6112517.00', '11631229.00', '10149973.00', '10849971.00']}, {'nameId': 'totalLTIP-Payout', 'name': 'LTIP Payout', 'datum': [None, None, None, None, None]}, {'nameId': 'totalNon-Equity', 'name': 'Non-Equity Compensation', 'datum': ['11980967.00', '7943725.00', '7093500.00', '12870000.00', '12940000.00']}, {'nameId': 'totalOther', 'name': 'Other Compensation', 'datum': ['97387.0', '111344.0', '9001452.00', '117800.00', '101515.00']}]}], 'datesDef': ['2018', '2019', '2020', '2021', '2022'], 'currency': 'USD', 'template': ''}
        
        """
        return self.GetData("insiders/keyExecutives")
    def keyRatio(self):
        """
        This function retrieves the key ratio of the stock.

        Returns:
            dict with key ratio

        Examples:
            >>> Stock("visa", exchange="nyse").keyRatio()

            {'valuationRatio': {'asOfDate': '2023-02-17', 'priceToBook': 12.722646, 'priceToCashFlow': 25.252525, 'priceToSales': 15.72327, 'priceToEPS': 31.25}, 'growthRatio': {'reportDate': '2022-09-30', 'revenue': 0.084528, 'operatingIncome': 0.085174, 'netIncome': 0.073806, 'eps': 0.095794}, 'financialHealth': {'reportDate': '2022-12-31', 'quickRatio': 1.051982, 'currentRatio': 1.439518, 'debtToEquity': 0.554842}, 'efficiencyRatio': {'reportDate': '2022-12-31', 'returnOnAssets': 0.181417, 'returnOnEquity': 0.432868, 'returnOnInvestedCapital': 0.288646}, 'profitabilityRatio': {'reportDate': '2022-12-31', 'interestCoverage': 34.67098, 'netMargin': 0.502766}, 'keyStatsQuoteJson': {'revenue3YearGrowth': {'stockValue': '8.4500', 'indAvg': '6.5700'}, 'netIncome3YearGrowth': {'stockValue': '7.3800', 'indAvg': '5.7800'}, 'operatingMarginTTM': {'stockValue': '66.8800', 'indAvg': '37.2400'}, 'netMarginTTM': {'stockValue': '50.2800', 'indAvg': '23.8600'}, 'roaTTM': {'stockValue': '18.1400', 'indAvg': '3.6500'}, 
            'roeTTM': {'stockValue': '43.2900', 'indAvg': '23.6700'}, 'debitToEquity': {'stockValue': '0.5548', 'indAvg': '1.5918'}, 'freeCashFlow': {'cashFlowTTM': '17742000000', 'date': '2022-12-31T06:00:00.000'}}}
        
        """
        return self.GetData("keyratios")
    
    def mutualFundBuyers(self, top=20):
        """
        This function retrieves the mutual funds which buy the stock.

        Args:
            top (int) : number of mutual funds to return
        Returns:
            dict with the buyers

        Examples:
            >>> Stock("visa", exchange="nyse").mutualFundBuyers(top=50)

        {'isRestricted': False, 'userType': 'Free', 'country': 'USA', 'rows': [{'secId': 'F000013YOE', 'name': 'Blackrock Equity Index Non-Lendable', 'securityType': 'SA', 'totalSharesHeld': 0.2719601220844319, 'totalAssets': 1.07372, 'currentShares': 5716020, 'changeAmount': 5716020, 'changePercentage': None, 'date': '2022-06-30T00:00:00.000', 'trend': '_PO_', 'starRating': ''}, 
        {'secId': 'SAUSA000V5', 'name': 'Lazard U.S. Equity Select', 'securityType': 'SA', 'totalSharesHeld': 0.13099668044048196, 'totalAssets': 3.12826, 'currentShares': 2753270, 'changeAmount': 2726291, 'changePercentage': 10105.233700285407, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'F00000YCN5', 'name': 'Newton Dynamic US Equity Composite', 'securityType': 'SA', 'totalSharesHeld': 0.07201384629588213, 'totalAssets': 1.05608, 'currentShares': 1513577, 'changeAmount': 1433750, 'changePercentage': 1796.0715046287596, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'F00000PX37', 'name': 'Invesco OFI Large Cap Core', 'securityType': 'SA', 'totalSharesHeld': 0.0340467093890988, 'totalAssets': 1.74812, 'currentShares': 715589, 'changeAmount': 715589, 'changePercentage': None, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '3'}, {'secId': 'F000016CKT', 'name': 'Vanguard Instl Ttl Stck Mkt Idx Tr', 'securityType': 'CZ', 'totalSharesHeld': 2.345276685160506, 'totalAssets': 0.89495, 'currentShares': 49292699, 'changeAmount': 693067, 'changePercentage': 1.426074584268457, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '3'}, {'secId': None, 'name': None, 'securityType': None, 'totalSharesHeld': 2.854294043370401, 'totalAssets': None, 'currentShares': 59991155, 'changeAmount': 11284717, 'changePercentage': None, 'date': None, 'trend': '_PO_', 'starRating': None}], 'columnDefs': [{'columnId': 'name', 'dataType': 'string'}, {'columnId': 'starRating', 'dataType': 'string'}, {'columnId': 'totalSharesHeld', 'dataType': 'number'}, {'columnId': 'totalAssets', 'dataType': 'number'}, {'columnId': 'trend', 'dataType': 'number'}, {'columnId': 'currentShares', 'dataType': 'number'}, {'columnId': 'changeAmount', 'dataType': 'number'}, {'columnId': 'changePercentage', 'dataType': 'number'}, {'columnId': 'date', 'dataType': 'string'}]}    
        
        """
        
        if not isinstance(top, int):
            raise TypeError('top parameter should be an integer')
        
        return self.GetData("ownership/v1", url_suffixe= f"Buyers/mutualfund/{top}/data")
    
    def mutualFundConcentratedOwners(self, top=20):
        """
        This function retrieves the mutual funds which are concentrated on the stock.

        Args:
            top (int) : number of mutual funds to return
        Returns:
            dict with the concentarted owners

        Examples:
            >>> Stock("visa", exchange="nyse").mutualFundConcentratedOwners(top=50)

        {'isRestricted': False, 'userType': 'Free', 'country': 'USA', 'rows': [{'secId': 'F00000WQSR', 'name': 'Candriam GF US Eq Opps S Dist', 'securityType': 'FO', 'totalSharesHeld': 0.006963458418254519, 'totalAssets': 36.34273, 'currentShares': 146357, 'changeAmount': 10396, 'changePercentage': 7.646310338994271, 'date': '2022-10-31T00:00:00.000', 'trend': '_PO_', 'starRating': ''}, {'secId': 'FOUSA00I9H', 'name': 'Fidelity® Select 
        IT Services', 'securityType': 'FO', 'totalSharesHeld': 0.1074489938434667, 'totalAssets': 23.08749, 'currentShares': 2258348, 'changeAmount': 20600, 'changePercentage': 0.9205683571161721, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '3'}, {'secId': 'FOUSA00CJB', 'name': 'Fidelity® Select Fintech Port', 'securityType': 'FO', 'totalSharesHeld': 0.0035496952124648286, 'totalAssets': 15.10474, 'currentShares': 
        74607, 'changeAmount': -534, 'changePercentage': -0.7106639517706711, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '2'}, {'secId': 'F000016PYB', 'name': 'GENIUS21', 'securityType': 'FE', 'totalSharesHeld': 0.00024983513022441346, 'totalAssets': 13.8214, 'currentShares': 5251, 'changeAmount': 0, 'changePercentage': 0.0, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': ''}, {'secId': 'F00001CJPB', 'name': 'FT Skybridge Digital Innovation 2 CA', 'securityType': 'FI', 'totalSharesHeld': 0.0003101647712688919, 'totalAssets': 13.80358, 'currentShares': 6519, 'changeAmount': -167, 'changePercentage': -2.4977565061322164, 'date': '2022-11-23T00:00:00.000', 'trend': '_PO_', 'starRating': ''}, {'secId': 'F00001CJP8', 'name': 'FT FTP Skybridge Digital Innovation 2 CA', 'securityType': 'FI', 'totalSharesHeld': 0.00031011719268762654, 'totalAssets': 13.80224, 'currentShares': 6518, 'changeAmount': -167, 'changePercentage': -2.4981301421091997, 'date': '2022-11-23T00:00:00.000', 'trend': '_PO_', 'starRating': ''}, {'secId': 'F000017PWK', 'name': 'FT FTP Skybridge Digital Innovation CA', 'securityType': 'FI', 'totalSharesHeld': 0.0002861851663111497, 'totalAssets': 12.20531, 'currentShares': 6015, 'changeAmount': 412, 'changePercentage': 7.353203640906657, 'date': '2022-08-26T00:00:00.000', 'trend': '_PO_', 'starRating': ''}, {'secId': 'F000011NA5', 'name': 'Opportunity GlbEq R$ BDR Nv I FIC FIA IE', 'securityType': 'FO', 'totalSharesHeld': 0.0007424637606459668, 'totalAssets': 11.60995, 'currentShares': 15605, 'changeAmount': 0, 'changePercentage': 0.0, 'date': '2022-10-31T00:00:00.000', 'trend': '_PO_', 'starRating': ''}, {'secId': 'F0000174GZ', 'name': 'FT Skybridge Digital Innovation 1 CA', 'securityType': 'FI', 'totalSharesHeld': 0.00026972297719333457, 'totalAssets': 11.50355, 'currentShares': 5669, 'changeAmount': 395, 'changePercentage': 7.489571482745545, 'date': '2022-08-26T00:00:00.000', 'trend': '_PO_', 'starRating': ''}, {'secId': 'F00000N6I3', 'name': 'Inverfinit de Valores SICAV', 'securityType': 'FO', 'totalSharesHeld': 0.0005160848709853767, 'totalAssets': 10.62448, 'currentShares': 
        10847, 'changeAmount': -86, 'changePercentage': -0.7866093478459709, 'date': '2022-06-30T00:00:00.000', 'trend': '_PO_', 'starRating': ''}, {'secId': None, 'name': None, 'securityType': None, 'totalSharesHeld': 0.12064672134350281, 'totalAssets': None, 'currentShares': 2535736, 'changeAmount': 30849, 'changePercentage': None, 'date': None, 'trend': '_PO_', 'starRating': None}], 'columnDefs': [{'columnId': 'name', 'dataType': 'string'}, {'columnId': 'starRating', 'dataType': 'string'}, {'columnId': 'totalSharesHeld', 'dataType': 'number'}, {'columnId': 'totalAssets', 'dataType': 'number'}, {'columnId': 'trend', 'dataType': 'number'}, {'columnId': 'currentShares', 'dataType': 'number'}, {'columnId': 'changeAmount', 'dataType': 'number'}, {'columnId': 'changePercentage', 'dataType': 'number'}, {'columnId': 'date', 'dataType': 'string'}]}     
                
        """
        
        if not isinstance(top, int):
            raise TypeError('top parameter should be an integer')
        
        return self.GetData("ownership/v1", url_suffixe= f"ConcentratedOwners/mutualfund/{top}/data")
    
    def mutualFundOwnership(self, top=20):
        """
        This function retrieves the main mutual funds which own the stock.

        Args:
            top (int) : number of mutual funds to return
        Returns:
            dict with the main owners

        Examples:
            >>> Stock("visa", exchange="nyse").mutualFundOwnership(top=50)

        {'isRestricted': False, 'userType': 'Free', 'country': 'USA', 'rows': [{'secId': 'FOUSA00FQU', 'name': 'Vanguard Total Stock Mkt Idx Inv', 'securityType': 'FO', 'totalSharesHeld': 2.363312107646441, 'totalAssets': 0.91882, 'currentShares': 49671765, 'changeAmount': 379066, 'changePercentage': 0.7690104370223265, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '3'}, {'secId': 'F000016CKT', 'name': 'Vanguard Instl Ttl Stck Mkt Idx Tr', 'securityType': 'CZ', 'totalSharesHeld': 2.345276685160506, 'totalAssets': 0.89495, 'currentShares': 49292699, 'changeAmount': 693067, 'changePercentage': 1.426074584268457, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '3'}, {'secId': 'FOUSA00FS1', 'name': 'Vanguard 500 Index Investor', 'securityType': 'FO', 'totalSharesHeld': 1.7905494420430277, 'totalAssets': 1.09387, 'currentShares': 37633519, 'changeAmount': 89682, 'changePercentage': 0.23887276092744597, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'FEUSA00001', 'name': 'SPDR® S&P 500 ETF Trust', 'securityType': 'FE', 'totalSharesHeld': 0.8639271683368897, 'totalAssets': 1.10277, 'currentShares': 18157901, 'changeAmount': -58380, 'changePercentage': -0.32048253976758484, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'F00000LZSI', 'name': 'Fidelity® 500 Index', 'securityType': 'FO', 'totalSharesHeld': 0.8523729953480842, 'totalAssets': 1.0551, 'currentShares': 17915057, 'changeAmount': 185481, 'changePercentage': 1.046167150302974, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'FEUSA0000E', 'name': 'iShares Core S&P 500 ETF', 'securityType': 'FE', 'totalSharesHeld': 0.6990890325068876, 'totalAssets': 1.10031, 'currentShares': 14693356, 'changeAmount': 3904, 'changePercentage': 0.026576893406234623, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'FOUSA00FQV', 'name': 'Vanguard Growth Index Investor', 'securityType': 'FO', 'totalSharesHeld': 0.6359506850745711, 'totalAssets': 2.09427, 'currentShares': 13366323, 'changeAmount': 63038, 'changePercentage': 0.4738528867118159, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'FOUSA00FR3', 'name': 'Vanguard Institutional Index I', 'securityType': 'FO', 'totalSharesHeld': 0.5412274867835694, 'totalAssets': 1.09962, 'currentShares': 11375444, 'changeAmount': -46535, 'changePercentage': -0.4074162629785959, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'FEUSA04AF8', 'name': 'Vanguard Dividend Appreciation ETF', 'securityType': 'FE', 'totalSharesHeld': 0.47840201610165384, 'totalAssets': 2.95065, 'currentShares': 10054987, 'changeAmount': -158197, 'changePercentage': -1.548948888025517, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'SAUSA00085', 'name': 'T. Rowe Price US Lrg-Cap Core Gr Eq Comp', 'securityType': 'SA', 'totalSharesHeld': 0.4425197250473374, 'totalAssets': 3.85727, 'currentShares': 9300818, 'changeAmount': -377651, 'changePercentage': -3.901970445945531, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '2'}, {'secId': 'FOUSA00EI7', 'name': 'T. Rowe Price Blue Chip Growth', 'securityType': 'FO', 'totalSharesHeld': 0.4425197250473374, 'totalAssets': 3.85727, 'currentShares': 9300818, 'changeAmount': -377651, 'changePercentage': -3.901970445945531, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '2'}, {'secId': 'FOUSA06Y9Y', 'name': 'State St S&P 500® Indx SL Cl III', 'securityType': 'CZ', 'totalSharesHeld': 0.35933890025698734, 'totalAssets': 1.08311, 'currentShares': 7552535, 'changeAmount': 75300, 'changePercentage': 1.007056752930729, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'FEUSA0000C', 'name': 'Technology Select Sector SPDR® ETF', 'securityType': 'FE', 'totalSharesHeld': 0.3564264249834095, 'totalAssets': 4.23317, 'currentShares': 7491321, 'changeAmount': -23826, 'changePercentage': -0.3170397066085334, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'FEUSA04ACD', 'name': 'Vanguard Information Technology ETF', 'securityType': 'FE', 'totalSharesHeld': 0.3463252067092856, 'totalAssets': 3.42033, 'currentShares': 7279015, 'changeAmount': -77407, 'changePercentage': -1.052237079384516, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '5'}, {'secId': 'FOUSA00FUG', 'name': 'Vanguard Dividend Growth Inv', 'securityType': 'FO', 'totalSharesHeld': 0.3277516228906576, 'totalAssets': 2.97158, 'currentShares': 6888638, 'changeAmount': 0, 'changePercentage': 0.0, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': 
        '5'}, {'secId': 'FOUSA00CEO', 'name': 'Fidelity® Contrafund®', 'securityType': 'FO', 'totalSharesHeld': 0.3016857923015918, 'totalAssets': 1.44712, 'currentShares': 6340790, 'changeAmount': 0, 'changePercentage': 0.0, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'FOUSA00EJ5', 'name': 'T. Rowe Price Growth Stock', 'securityType': 'FO', 'totalSharesHeld': 0.3000167832493842, 'totalAssets': 3.12292, 'currentShares': 6305711, 'changeAmount': 528126, 'changePercentage': 9.140947298914686, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '2'}, {'secId': 'F000010NV2', 'name': 'T. Rowe Price US Growth Stock Composite', 'securityType': 'SA', 'totalSharesHeld': 0.3000167832493842, 'totalAssets': 3.12292, 'currentShares': 6305711, 'changeAmount': 528126, 'changePercentage': 9.140947298914686, 'date': '2022-12-31T00:00:00.000', 'trend': '_PO_', 'starRating': '2'}, {'secId': 'FOUSA05YQF', 'name': 'Morgan Stanley Glbl Franchise', 'securityType': 'SA', 'totalSharesHeld': 0.29298757123182095, 'totalAssets': 5.90207, 'currentShares': 6157972, 'changeAmount': -24592, 'changePercentage': -0.3977637756762405, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': '5'}, {'secId': 'F0GBR04TB5', 'name': 'MS INVF Global Brands A USD', 
        'securityType': 'FO', 'totalSharesHeld': 0.29067753595422513, 'totalAssets': 6.57572, 'currentShares': 6109420, 'changeAmount': -39879, 'changePercentage': -0.6485129443209706, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '5'}, {'secId': None, 'name': None, 'securityType': None, 'totalSharesHeld': 14.330373689923052, 'totalAssets': None, 'currentShares': 301193800, 'changeAmount': 1361672, 'changePercentage': None, 'date': None, 'trend': '_PO_', 'starRating': None}], 'columnDefs': [{'columnId': 'name', 'dataType': 'string'}, {'columnId': 'starRating', 'dataType': 'string'}, {'columnId': 'totalSharesHeld', 'dataType': 'number'}, {'columnId': 'totalAssets', 'dataType': 'number'}, {'columnId': 'trend', 'dataType': 'number'}, {'columnId': 'currentShares', 'dataType': 'number'}, {'columnId': 'changeAmount', 'dataType': 'number'}, {'columnId': 'changePercentage', 'dataType': 'number'}, {'columnId': 'date', 'dataType': 'string'}]}     
                
        """
        
        if not isinstance(top, int):
            raise TypeError('top parameter should be an integer')
        
        return self.GetData("ownership/v1", url_suffixe= f"OwnershipData/mutualfund/{top}/data")
    
    def mutualFundSellers(self, top=20):
        """
        This function retrieves the mutual funds which sell on the stock.

        Args:
            top (int) : number of mutual funds to return
        Returns:
            dict with sellers

        Examples:
            >>> Stock("visa", exchange="nyse").mutualFundSellers(top=50)

        {'isRestricted': False, 'userType': 'Free', 'country': 'USA', 'rows': [{'secId': 'SAUSA000P6', 'name': 'Sands Select Growth Equity', 'securityType': 'SA', 'totalSharesHeld': 0.008241752161110987, 'totalAssets': 8.84981, 'currentShares': 173224, 'changeAmount': -2759728, 'changePercentage': -94.09386856655001, 'date': '2023-01-31T00:00:00.000', 'trend': '_PO_', 'starRating': '1'}, {'secId': 'FEUSA0000G', 'name': 'iShares S&P 500 
        Value ETF', 'securityType': 'FE', 'totalSharesHeld': 0.0, 'totalAssets': None, 'currentShares': 0, 'changeAmount': -1533314, 'changePercentage': -100.0, 'date': '2022-12-16T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'FOUSA05H9Y', 'name': 'Edgewood Growth Instl', 'securityType': 'FO', 'totalSharesHeld': 0.287273383621851, 'totalAssets': 6.7566, 'currentShares': 6037872, 'changeAmount': -1226873, 'changePercentage': -16.888039428775546, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': '3'}, {'secId': 'FEUSA0002A', 'name': 'SPDR® Portfolio S&P 500 Value ETF', 'securityType': 'FE', 'totalSharesHeld': 0.0, 'totalAssets': None, 'currentShares': 0, 'changeAmount': -1021317, 'changePercentage': -100.0, 'date': '2022-12-19T00:00:00.000', 'trend': '_PO_', 'starRating': '4'}, {'secId': 'FOUSA09F5N', 'name': 'Sands Global Growth Eqty', 'securityType': 'SA', 'totalSharesHeld': 0.1667574557982467, 'totalAssets': 5.2166, 'currentShares': 3504885, 'changeAmount': -984896, 'changePercentage': -21.936392888650918, 'date': '2022-09-30T00:00:00.000', 'trend': '_PO_', 'starRating': '1'}, {'secId': None, 'name': None, 'securityType': None, 'totalSharesHeld': 0.4622725915812087, 'totalAssets': None, 'currentShares': 9715981, 'changeAmount': -7526128, 'changePercentage': None, 'date': None, 'trend': '_PO_', 'starRating': None}], 'columnDefs': [{'columnId': 'name', 'dataType': 'string'}, {'columnId': 'starRating', 'dataType': 'string'}, {'columnId': 'totalSharesHeld', 'dataType': 'number'}, {'columnId': 'totalAssets', 'dataType': 'number'}, {'columnId': 'trend', 'dataType': 'number'}, {'columnId': 'currentShares', 'dataType': 'number'}, {'columnId': 'changeAmount', 'dataType': 'number'}, 
        {'columnId': 'changePercentage', 'dataType': 'number'}, {'columnId': 'date', 'dataType': 'string'}]}       
                
        """
        
        if not isinstance(top, int):
            raise TypeError('top parameter should be an integer')
        
        return self.GetData("ownership/v1", url_suffixe= f"Sellers/mutualfund/{top}/data")
    
    def operatingGrowth(self):
        """
        This function retrieves the operating growth of the stock.

        Returns:
            dict with operating growth

        Examples:
            >>> Stock("visa", exchange="nyse").operatingGrowth()

        {'currency': 'USD', 'asOfDate': '2022-12-31T06:00:00.000', 'dataList': [{'fiscalPeriodDate': '2013-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2013-09', 'revenuePer': {'yearOverYear': 13.020000000000001, 'threeYearAvg': 13.450000000000001, 'fiveYearAvg': 13.459999999999999, 'tenYearAvg': None}, 
        'operatingIncome': {'yearOverYear': 16.08, 'threeYearAvg': 16.81, 'fiveYearAvg': 21.8, 'tenYearAvg': None}, 'netIncomePer': {'yearOverYear': 132.28, 'threeYearAvg': 18.86, 'fiveYearAvg': 44.01, 'tenYearAvg': None}, 'epsPer': {'yearOverYear': 140.19, 'threeYearAvg': 23.7, 'fiveYearAvg': 51.21, 'tenYearAvg': None}}, {'fiscalPeriodDate': '2014-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2014-09', 'revenuePer': {'yearOverYear': 7.85, 'threeYearAvg': 11.4, 'fiveYearAvg': 12.94, 'tenYearAvg': None}, 'operatingIncome': {'yearOverYear': 12.540000000000001, 'threeYearAvg': 14.26, 'fiveYearAvg': 18.15, 'tenYearAvg': None}, 'netIncomePer': {'yearOverYear': 9.2, 'threeYearAvg': 14.21, 'fiveYearAvg': 18.240000000000002, 'tenYearAvg': None}, 'epsPer': {'yearOverYear': 13.569999999999999, 'threeYearAvg': 18.65, 'fiveYearAvg': 22.7, 'tenYearAvg': None}}, {'fiscalPeriodDate': '2015-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2015-09', 'revenuePer': {'yearOverYear': 9.27, 'threeYearAvg': 10.03, 'fiveYearAvg': 11.469999999999999, 'tenYearAvg': 17.94}, 'operatingIncome': {'yearOverYear': 11.39, 'threeYearAvg': 13.320000000000002, 'fiveYearAvg': 14.84, 'tenYearAvg': 31.540000000000003}, 'netIncomePer': {'yearOverYear': 16.37, 'threeYearAvg': 43.44, 'fiveYearAvg': 16.36, 'tenYearAvg': 37.36}, 'epsPer': {'yearOverYear': 19.72, 'threeYearAvg': 48.36, 'fiveYearAvg': 20.810000000000002, 'tenYearAvg': 35.19}}, {'fiscalPeriodDate': '2016-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2016-09', 'revenuePer': {'yearOverYear': 8.66, 'threeYearAvg': 8.59, 'fiveYearAvg': 10.42, 'tenYearAvg': 17.73}, 'operatingIncome': {'yearOverYear': 7.53, 'threeYearAvg': 10.47, 'fiveYearAvg': 12.31, 'tenYearAvg': 29.609999999999996}, 'netIncomePer': {'yearOverYear': -5.33, 'threeYearAvg': 6.35, 'fiveYearAvg': 10.42, 'tenYearAvg': 29.42}, 'epsPer': {'yearOverYear': -3.88, 'threeYearAvg': 9.33, 'fiveYearAvg': 13.969999999999999, 'tenYearAvg': 31.56}}, {'fiscalPeriodDate': '2017-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2017-09', 'revenuePer': {'yearOverYear': 21.72, 'threeYearAvg': 13.059999999999999, 'fiveYearAvg': 11.99, 'tenYearAvg': 17.73}, 'operatingIncome': {'yearOverYear': 24.6, 'threeYearAvg': 14.280000000000001, 'fiveYearAvg': 14.280000000000001, 'tenYearAvg': 26.029999999999998}, 'netIncomePer': {'yearOverYear': 11.82, 'threeYearAvg': 7.199999999999999, 'fiveYearAvg': 25.590000000000003, 'tenYearAvg': None}, 'epsPer': {'yearOverYear': 12.9, 'threeYearAvg': 9.120000000000001, 'fiveYearAvg': 28.799999999999997, 'tenYearAvg': None}}, {'fiscalPeriodDate': '2018-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2018-09', 'revenuePer': {'yearOverYear': 12.26, 'threeYearAvg': 14.08, 'fiveYearAvg': 11.84, 'tenYearAvg': 12.65}, 'operatingIncome': {'yearOverYear': 11.49, 'threeYearAvg': 14.31, 'fiveYearAvg': 13.370000000000001, 'tenYearAvg': 17.51}, 'netIncomePer': {'yearOverYear': 53.769999999999996, 'threeYearAvg': 17.64, 'fiveYearAvg': 15.65, 'tenYearAvg': 29.049999999999997}, 'epsPer': {'yearOverYear': 57.86, 'threeYearAvg': 19.66, 'fiveYearAvg': 18.43, 'tenYearAvg': 33.82}}, {'fiscalPeriodDate': '2019-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2019-09', 'revenuePer': {'yearOverYear': 11.49, 'threeYearAvg': 15.07, 'fiveYearAvg': 12.590000000000002, 'tenYearAvg': 12.770000000000001}, 'operatingIncome': {'yearOverYear': 13.569999999999999, 'threeYearAvg': 16.41, 'fiveYearAvg': 13.569999999999999, 'tenYearAvg': 15.840000000000002}, 'netIncomePer': {'yearOverYear': 17.27, 'threeYearAvg': 26.33, 'fiveYearAvg': 17.31, 'tenYearAvg': 17.77}, 'epsPer': {'yearOverYear': 20.36, 'threeYearAvg': 28.970000000000002, 'fiveYearAvg': 19.81, 'tenYearAvg': 21.240000000000002}}, {'fiscalPeriodDate': '2020-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2020-09', 'revenuePer': {'yearOverYear': -4.92, 'threeYearAvg': 5.970000000000001, 'fiveYearAvg': 9.5, 'tenYearAvg': 10.48}, 'operatingIncome': {'yearOverYear': -8.5, 'threeYearAvg': 5.029999999999999, 'fiveYearAvg': 9.19, 'tenYearAvg': 11.98}, 'netIncomePer': {'yearOverYear': -10.05, 'threeYearAvg': 17.5, 'fiveYearAvg': 11.42, 'tenYearAvg': 13.86}, 'epsPer': {'yearOverYear': -8.08, 'threeYearAvg': 20.43, 'fiveYearAvg': 13.639999999999999, 'tenYearAvg': 17.169999999999998}}, {'fiscalPeriodDate': '2021-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2021-09', 'revenuePer': {'yearOverYear': 10.34, 'threeYearAvg': 5.36, 'fiveYearAvg': 9.83, 'tenYearAvg': 10.13}, 'operatingIncome': {'yearOverYear': 12.17, 'threeYearAvg': 5.24, 'fiveYearAvg': 10.12, 'tenYearAvg': 11.21}, 'netIncomePer': {'yearOverYear': 13.3, 'threeYearAvg': 6.12, 'fiveYearAvg': 15.49, 'tenYearAvg': 12.93}, 'epsPer': {'yearOverYear': 15.129999999999999, 'threeYearAvg': 8.4, 'fiveYearAvg': 17.82, 'tenYearAvg': 15.879999999999999}}, {'fiscalPeriodDate': '2022-09-30T05:00:00.000', 'fiscalPeriodYearMonth': '2022-09', 'revenuePer': {'yearOverYear': 21.59, 'threeYearAvg': 8.450000000000001, 'fiveYearAvg': 9.81, 'tenYearAvg': 10.89}, 'operatingIncome': {'yearOverYear': 24.51, 'threeYearAvg': 8.52, 'fiveYearAvg': 10.100000000000001, 'tenYearAvg': 12.17}, 'netIncomePer': {'yearOverYear': 21.490000000000002, 'threeYearAvg': 7.380000000000001, 'fiveYearAvg': 17.43, 'tenYearAvg': 21.44}, 'epsPer': {'yearOverYear': 24.33, 'threeYearAvg': 9.58, 'fiveYearAvg': 20.11, 'tenYearAvg': 24.38}}, {'fiscalPeriodDate': '2022-12-31T06:00:00.000', 'fiscalPeriodYearMonth': 'Latest Qtr', 'revenuePer': {'yearOverYear': 12.42, 'threeYearAvg': None, 'fiveYearAvg': None, 'tenYearAvg': None}, 'operatingIncome': {'yearOverYear': None, 'threeYearAvg': None, 'fiveYearAvg': None, 'tenYearAvg': None}, 'netIncomePer': {'yearOverYear': 5.56, 'threeYearAvg': None, 'fiveYearAvg': None, 'tenYearAvg': None}, 'epsPer': {'yearOverYear': 8.74, 'threeYearAvg': None, 'fiveYearAvg': None, 'tenYearAvg': None}}]}  
        
        """
        return self.GetData("keyStats/growthTable", url_suffixe='')
    
    def operatingMargin(self):
        """
        This function retrieves the operating margin of the stock.

        Returns:
            dict with operating margin

        Examples:
            >>> Stock("visa", exchange="nyse").operatingMargin()

        {'currency': None, 'asOfDate': '2022-12-31T06:00:00.000', 'dataList': [{'fiscalPeriodYear': '2013', 'morningstarEndingDate': '2013-09-30T05:00:00.000', 'grossMargin': 79.62, 'operatingMargin': 61.49, 'netMargin': 42.120000000000005, 'ebitdaMargin': 64.86, 'taxRate': 31.380000000000003, 'roa': 13.059999999999999, 'roe': 18.21, 'roic': 18.21, 'interestCoverage': None, 'daysInSales': 22.9946, 'daysInInventory': None, 'daysInPayment': 25.55, 'cashConversionCycle': None, 'receivableTurnover': 15.8733, 'inventoryTurnover': None, 'fixedAssetsTurnover': 6.9982, 'assetsTurnover': 0.3101}, {'fiscalPeriodYear': '2014', 'morningstarEndingDate': '2014-09-30T05:00:00.000', 'grossMargin': 81.25, 'operatingMargin': 64.16, 'netMargin': 42.68, 'ebitdaMargin': 67.58999999999999, 'taxRate': 29.599999999999998, 'roa': 14.549999999999999, 'roe': 19.97, 'roic': 19.97, 'interestCoverage': None, 'daysInSales': 22.7443, 'daysInInventory': None, 'daysInPayment': 25.36, 'cashConversionCycle': None, 'receivableTurnover': 
        16.048, 'inventoryTurnover': None, 'fixedAssetsTurnover': 7.0099, 'assetsTurnover': 0.3409}, {'fiscalPeriodYear': '2015', 'morningstarEndingDate': '2015-09-30T05:00:00.000', 'grossMargin': 81.61, 'operatingMargin': 65.4, 'netMargin': 45.48, 'ebitdaMargin': 68.96, 'taxRate': 29.65, 'roa': 16.02, 'roe': 22.05, 'roic': 22.05, 'interestCoverage': None, 'daysInSales': 21.9447, 'daysInInventory': None, 'daysInPayment': 19.5868, 'cashConversionCycle': None, 'receivableTurnover': 16.6327, 'inventoryTurnover': None, 'fixedAssetsTurnover': 7.3439, 'assetsTurnover': 0.3523}, {'fiscalPeriodYear': '2016', 'morningstarEndingDate': '2016-09-30T05:00:00.000', 'grossMargin': 81.67, 'operatingMargin': 64.73, 'netMargin': 39.31, 
        'ebitdaMargin': 59.28, 'taxRate': 25.22, 'roa': 11.37, 'roe': 20.79, 'roic': 14.069999999999999, 'interestCoverage': 19.7635, 'daysInSales': 22.8458, 'daysInInventory': None, 'daysInPayment': 21.7891, 'cashConversionCycle': None, 'receivableTurnover': 15.9767, 'inventoryTurnover': None, 
        'fixedAssetsTurnover': 7.47, 'assetsTurnover': 0.2893}, {'fiscalPeriodYear': '2017', 'morningstarEndingDate': '2017-09-30T05:00:00.000', 'grossMargin': 82.31, 'operatingMargin': 66.25, 'netMargin': 35.23, 'ebitdaMargin': 69.8, 'taxRate': 42.71, 'roa': 9.8, 'roe': 23.76, 'roic': 15.049999999999999, 'interestCoverage': 21.7709, 'daysInSales': 21.6022, 'daysInInventory': None, 'daysInPayment': 21.464, 'cashConversionCycle': None, 'receivableTurnover': 16.8965, 'inventoryTurnover': None, 'fixedAssetsTurnover': 8.3389, 'assetsTurnover': 0.2781}, {'fiscalPeriodYear': '2018', 
        'morningstarEndingDate': '2018-09-30T05:00:00.000', 'grossMargin': 81.28999999999999, 'operatingMargin': 65.8, 'netMargin': 48.24, 'ebitdaMargin': 68.08, 'taxRate': 19.56, 'roa': 14.49, 'roe': 35.65, 'roic': 23.44, 'interestCoverage': 21.9248, 'daysInSales': 20.7215, 'daysInInventory': None, 'daysInPayment': 17.133, 'cashConversionCycle': None, 'receivableTurnover': 17.6145, 'inventoryTurnover': None, 'fixedAssetsTurnover': 8.7234, 'assetsTurnover': 0.3004}, {'fiscalPeriodYear': '2019', 'morningstarEndingDate': '2019-09-30T05:00:00.000', 'grossMargin': 81.87, 'operatingMargin': 67.03, 'netMargin': 52.56999999999999, 'ebitdaMargin': 69.88, 'taxRate': 18.84, 'roa': 17.04, 'roe': 41.83, 'roic': 26.52, 'interestCoverage': None, 'daysInSales': 21.8425, 'daysInInventory': None, 'daysInPayment': 14.8541, 'cashConversionCycle': None, 'receivableTurnover': 16.7105, 'inventoryTurnover': None, 'fixedAssetsTurnover': 8.8937, 'assetsTurnover': 0.3241}, {'fiscalPeriodYear': '2020', 'morningstarEndingDate': 
        '2020-09-30T05:00:00.000', 'grossMargin': 79.35, 'operatingMargin': 64.51, 'netMargin': 49.74, 'ebitdaMargin': 69.0, 'taxRate': 21.2, 'roa': 14.16, 'roe': 36.01, 'roic': 21.65, 'interestCoverage': 27.7248, 'daysInSales': 26.3984, 'daysInInventory': None, 'daysInPayment': 13.3477, 'cashConversionCycle': None, 'receivableTurnover': 13.8266, 'inventoryTurnover': None, 'fixedAssetsTurnover': 8.0434, 'assetsTurnover': 0.2847}, {'fiscalPeriodYear': '2021', 'morningstarEndingDate': '2021-09-30T05:00:00.000', 'grossMargin': 79.38, 'operatingMargin': 65.58, 'netMargin': 51.07000000000001, 'ebitdaMargin': 72.1, 'taxRate': 23.36, 'roa': 15.03, 'roe': 37.51, 'roic': 22.96, 'interestCoverage': 32.3119, 'daysInSales': 27.1498, 'daysInInventory': None, 'daysInPayment': 16.1569, 'cashConversionCycle': None, 'receivableTurnover': 13.4439, 'inventoryTurnover': None, 'fixedAssetsTurnover': 8.8426, 'assetsTurnover': 0.2943}, {'fiscalPeriodYear': '2022', 'morningstarEndingDate': '2022-09-30T05:00:00.000', 'grossMargin': 80.44, 'operatingMargin': 67.15, 'netMargin': 51.03, 'ebitdaMargin': 66.64999999999999, 'taxRate': 17.53, 'roa': 17.76, 'roe': 42.68, 'roic': 27.13, 'interestCoverage': 34.71, 'daysInSales': 24.8315, 'daysInInventory': None, 'daysInPayment': 19.2909, 'cashConversionCycle': None, 'receivableTurnover': 14.6991, 'inventoryTurnover': None, 'fixedAssetsTurnover': 9.872, 'assetsTurnover': 0.3481}, {'fiscalPeriodYear': 'Current', 'morningstarEndingDate': '2022-12-31T06:00:00.000', 'grossMargin': 80.35, 'operatingMargin': 66.88, 'netMargin': 50.28, 'ebitdaMargin': 65.08, 'taxRate': 16.68, 'roa': 18.14, 'roe': 43.29, 'roic': 28.860000000000003, 'interestCoverage': 34.671, 'daysInSales': 25.8814, 'daysInInventory': None, 'daysInPayment': 15.6877, 'cashConversionCycle': None, 'receivableTurnover': 14.1028, 'inventoryTurnover': None, 'fixedAssetsTurnover': 
        9.8265, 'assetsTurnover': 0.3608}, {'fiscalPeriodYear': '5-Yr', 'morningstarEndingDate': None, 'grossMargin': 80.466, 'operatingMargin': 66.014, 'netMargin': 50.53, 'ebitdaMargin': 69.142, 'taxRate': 20.098, 'roa': 15.696, 'roe': 38.736, 'roic': 24.34, 'interestCoverage': 23.3343, 'daysInSales': 24.1887, 'daysInInventory': 0.0, 'daysInPayment': 16.1565, 'cashConversionCycle': 0.0, 'receivableTurnover': 15.2589, 'inventoryTurnover': 0.0, 'fixedAssetsTurnover': 8.875, 'assetsTurnover': 0.3103}, {'fiscalPeriodYear': 'Index', 'morningstarEndingDate': None, 'grossMargin': None, 'operatingMargin': None, 'netMargin': None, 'ebitdaMargin': None, 'taxRate': None, 'roa': None, 'roe': None, 'roic': None, 'interestCoverage': None, 'daysInSales': None, 'daysInInventory': None, 'daysInPayment': None, 'cashConversionCycle': None, 'receivableTurnover': None, 'inventoryTurnover': None, 'fixedAssetsTurnover': None, 'assetsTurnover': None}]}
                
        """
        return self.GetData("keyStats/OperatingAndEfficiency", url_suffixe='')
    
    def operatingPerformance(self):
        """
        This function retrieves the operating performance the stock.

        Returns:
            dict with voperating performance

        Examples:
            >>> Stock("visa", exchange="nyse").operatingPerformance()

        {'_meta': {'responseStatus': '200700', 'hint': 'Securities successfully returned', 'performanceId': '0P0000CPCP'}, 'reported': {'reportType': 'Fiscal', 'reportType_label': 'operating.performance.report.type', 'columnDefs': ['2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', 
        '2021', '2022', 'TTM', '5-Yr', 'Index'], 'columnDefs_labels': ['2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', 'operating.performance.headers.oneyearttm', 'operating.performance.headers.fiveyear', 'operating.performance.headers.index'], 'Collapsed': {'rows': [{'label': 'Return on Assets %', 'salDataId': 'return.on.assets.reported.label', 'datum': ['13.06', '14.55', '16.02', '11.37', '9.80', '14.49', '17.04', '14.16', '15.03', '17.76', '18.14', '15.67', '10.16618'], 'percentage': True}, {'label': 'Return on Equity %', 'salDataId': 'return.on.equity.reported.label', 'datum': ['18.21', '19.97', '22.05', '20.79', '23.76', '35.65', '41.83', '36.01', '37.51', '42.68', '43.29', '38.39', '28.89043'], 'percentage': True}, {'label': 'Return on Invested Capital %', 'salDataId': 'return.on.invested.capital.reported.label', 'datum': 
        ['18.21', '19.97', '22.05', '14.07', '15.05', '23.44', '26.52', '21.65', '22.96', '27.13', '28.86', '24.38', '18.40613'], 'percentage': True}]}, 'Expanded': {'rows': [{'label': 'Financial Leverage', 'salDataId': 'financial.leverage.reported.label', 'datum': ['1.338147', '1.40696', '1.348301', '2.354661', '2.496034', '2.425883', '2.48354', '2.599891', '2.402156', '2.402996', None, None, None]}, {'label': 'Gross Margin %', 'salDataId': 'gross.margin.reported.label', 'datum': ['79.62', '81.25', '81.61', '81.67', '82.31', '81.29', '81.87', '79.35', '79.38', '80.44', '80.35', '80.53', None], 'percentage': True}, {'label': 'Operating Margin %', 'salDataId': 'operating.margin.reported.label', 'datum': ['61.49', '64.16', '65.40', '64.73', '66.25', '65.80', '67.03', '64.51', '65.58', '67.15', '66.88', '66.05', None], 'percentage': True}, {'label': 'Net Margin %', 'salDataId': 'net.margin.reported.label', 'datum': ['42.12', '42.68', '45.48', '39.31', '35.23', '48.24', '52.57', '49.74', '51.07', '51.03', '50.28', '50.24', '16.9328'], 'percentage': True}, {'label': 'Days Sales Outstanding', 'salDataId': 'days.sales.outstanding.reported.label', 'datum': ['22.994566', '22.744253', '21.944705', '22.845776', '21.602163', '20.721529', '21.842495', '26.398425', '27.149761', '24.831457', '25.881422', '24.58235175', None]}, {'label': 'Days Inventory', 'salDataId': 'days.inventory.reported.label', 'datum': []}, {'label': 'Days Payables', 'salDataId': 'days.payables.reported.label', 'datum': ['25.55', '25.359992', '19.586761', '21.789074', '21.463978', '17.133039', '14.854142', '13.347739', '16.156942', '19.290947', '15.687679', '13.263802649999999', None]}, {'label': 'Receivables Turnover', 'salDataId': 'receivables.turnover.reported.label', 'datum': ['15.873315', '16.04801', '16.632714', '15.976695', '16.896457', '17.61453', '16.710545', '13.826582', '13.443949', '14.699097', '14.10278', '14.998921450000003', None]}, {'label': 'Inventory Turnover', 'salDataId': 'inventory.turnover.reported.label', 'datum': []}, {'label': 'Fixed Asset Turnover', 'salDataId': 'fixed.asset.turnover.reported.label', 'datum': ['6.998217', '7.009934', '7.343915', '7.470035', '8.33886', '8.723386', '8.893749', '8.043446', '8.842627', '9.872011', '9.826497', '8.87280395', None]}, {'label': 'Total Asset Turnover', 'salDataId': 'total.asset.turnover.reported.label', 'datum': ['0.310074', '0.340879', '0.352262', '0.289285', '0.278126', '0.300418', '0.324078', '0.284651', '0.294295', '0.348106', '0.360838', '0.3115904', None]}, {'label': 'EBITDA Margin %', 'salDataId': 'ebitda.margin.reported.label', 'datum': ['64.86', '67.59', '68.96', '59.28', '69.80', '68.08', '69.88', '69.00', '72.10', '66.65', '65.08', '69.28', None], 'percentage': True}, {'label': 'EBITDA (Bil)', 'salDataId': 'ebitda.reported.label', 'datum': ['7.64', '8.59', '9.57', '8.94', '12.81', '14.03', '16.06', '15.07', '17.38', '19.54', '19.65', '16.26', None], 'orderOfMagnitude': 'Bil'}, {'label': 'Free Cash Flow/Net Income', 'salDataId': 'free.cash.flow.net.income.reported.label', 'datum': ['0.512249', '1.223244', '0.975032', '0.843098', '1.268995', '1.16445', '0.995695', '0.893061', '1.179595', '1.19536', '1.169006', '1.0773241499999997', None]}]}}, 'restated': {'reportType': 'Fiscal', 'reportType_label': 'operating.performance.report.type', 'columnDefs': ['2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', 'TTM', '5-Yr', 'Index'], 'columnDefs_labels': ['2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', 'operating.performance.headers.oneyearttm', 'operating.performance.headers.fiveyear', 'operating.performance.headers.index'], 'Collapsed': {'rows': [{'label': 'Return on Assets %', 'salDataId': 'return.on.assets.restated.label', 'datum': ['13.06', '14.55', '16.20', '11.47', '10.15', '15.02', '17.04', '14.16', '15.03', '17.76', '18.14', '15.67', '10.16618'], 'percentage': True}, {'label': 'Return on Equity %', 'salDataId': 'return.on.equity.restated.label', 'datum': ['18.21', '19.97', 
        '22.05', '20.79', '24.62', '36.94', '41.83', '36.01', '35.83', '40.88', '43.29', '38.39', '28.89043'], 'percentage': True}, {'label': 'Return on Invested Capital %', 'salDataId': 'return.on.invested.capital.restated.label', 'datum': ['18.21', '19.99', '22.06', '14.07', '14.82', '24.25', 
        '27.47', '21.65', '22.33', '26.42', '28.86', '24.38', '18.40613'], 'percentage': True}]}, 'Expanded': {'rows': [{'label': 'Financial Leverage', 
        'salDataId': 'financial.leverage.restated.label', 'datum': ['1.338147', '1.40696', '1.319181', '2.354661', '2.496034', '2.425883', '2.48354', '2.599891', '2.205326', '2.402996', None, None, None]}, {'label': 'Gross Margin %', 'salDataId': 'gross.margin.restated.label', 'datum': ['79.62', '81.25', '81.61', '81.67', '82.31', '81.29', '81.87', '79.35', '79.38', '80.44', '80.35', '80.53', None], 'percentage': True}, {'label': 'Operating Margin %', 'salDataId': 'operating.margin.restated.label', 'datum': ['61.49', '64.16', '65.40', '64.73', '66.25', '65.80', '67.03', '64.51', '65.58', '67.15', '66.88', '66.05', None], 'percentage': True}, {'label': 'Net Margin %', 'salDataId': 'net.margin.restated.label', 'datum': ['42.12', '42.68', '45.48', '39.31', '36.49', '49.98', '52.57', '49.74', '51.07', '51.03', '50.28', '50.24', '16.9328'], 'percentage': True}, {'label': 'Days Sales Outstanding', 'salDataId': 'days.sales.outstanding.restated.label', 'datum': ['22.994566', '22.744253', '21.944705', '22.845776', '21.602163', '20.721529', '21.842495', '26.398425', '27.149761', '24.831457', '25.881422', '24.58235175', None]}, {'label': 'Days Inventory', 'salDataId': 'days.inventory.restated.label', 'datum': []}, {'label': 'Days Payables', 'salDataId': 'days.payables.restated.label', 'datum': 
        ['25.55', '25.359992', '19.586761', '21.789074', '21.463978', '17.133039', '14.854142', '13.347739', '16.156942', '19.290947', '15.687679', '13.263802649999999', None]}, {'label': 'Receivables Turnover', 'salDataId': 'receivables.turnover.restated.label', 'datum': ['15.873315', '16.04801', '16.632714', '15.976695', '16.896457', '17.61453', '16.710545', '13.826582', '13.443949', '14.699097', '14.10278', '14.998921450000003', None]}, {'label': 'Inventory Turnover', 'salDataId': 'inventory.turnover.restated.label', 'datum': []}, {'label': 'Fixed Asset Turnover', 'salDataId': 'fixed.asset.turnover.restated.label', 'datum': ['6.998217', '7.009934', '7.343915', '7.470035', '8.33886', '8.723386', '8.893749', '8.043446', '8.842627', '9.872011', '9.826497', '8.87280395', None]}, {'label': 'Total Asset Turnover', 'salDataId': 'total.asset.turnover.restated.label', 'datum': ['0.310074', '0.340879', '0.35619', '0.291716', '0.278126', '0.300418', '0.324078', '0.284651', '0.294295', '0.348106', '0.360838', 
        '0.3115904', None]}, {'label': 'EBITDA Margin %', 'salDataId': 'ebitda.margin.restated.label', 'datum': ['64.86', '64.30', '68.39', '59.28', '69.28', '68.08', '69.95', '69.00', '72.10', '66.65', '65.08', '69.28', None], 'percentage': True}, {'label': 'EBITDA (Bil)', 'salDataId': 'ebitda.restated.label', 'datum': ['7.64', '8.17', '9.49', '8.94', '12.72', '14.03', '16.07', '15.07', '17.38', '19.54', '16.26', '19.65', None], 'orderOfMagnitude': 'Bil'}, {'label': 'Free Cash Flow/Net Income', 'salDataId': 'free.cash.flow.net.income.restated.label', 'datum': ['0.512249', '1.223244', '0.975032', '0.843098', '1.285266', '1.186584', '0.995695', '0.893061', '1.179595', '1.19536', '1.169006', '1.0773241499999997', None]}]}}, 'footer': {'indexLabel': 'Index:', 'indexName': 'Morningstar US Market TR USD', 'ebitdaCurrency': 'USD'}} 
                
        """
        return self.GetData("operatingPerformance/v2", url_suffixe='')
    
    def split(self):
        """
        This function retrieves the split history of the stock.

        Returns:
            dict with split history

        Examples:
            >>> Stock("visa", exchange="nyse").split()

        {'splitHistory': [{'date': '2015-03-19T05:00:00.000', 'ratio': '4:1'}], 'splitOffHistory': []}
        
        """
        return self.GetData("split")
    
    def trailingTotalReturn(self):
        """
        This function retrieves the performance of the stock and its index.

        Returns:
            dict with performance

        Examples:
            >>> Stock("visa", exchange="nyse").trailingTotalReturn()

            {'returnDate': '2023-02-17T06:00:00.000', 'trailingTotalReturnsList': [{'name': 'V', 'trailing1DayReturn': '-1.00518', 'trailing1WeekReturn': '-1.60211', 'trailing1MonthReturn': '0.45291', 'trailing3MonthReturn': '6.17091', 'trailing6MonthReturn': '4.63360', 'trailingYearToDateReturn': '7.82152', 'trailing1YearReturn': '0.26713', 'trailing3YearReturn': '2.70789', 'trailing5YearReturn': '13.53443', 'trailing10YearReturn': '19.39632', 'trailing15YearReturn': None}, {'name': 'Credit Services', 'trailing1DayReturn': '-1.22834', 'trailing1WeekReturn': '-2.01049', 'trailing1MonthReturn': '0.87680', 'trailing3MonthReturn': '6.21149', 'trailing6MonthReturn': '0.56680', 'trailingYearToDateReturn': '8.78095', 'trailing1YearReturn': '-8.56893', 'trailing3YearReturn': '0.72212', 'trailing5YearReturn': '9.64393', 'trailing10YearReturn': '14.79228', 'trailing15YearReturn': '14.14500'}, 
            {'name': 'Morningstar US Market TR USD', 'trailing1DayReturn': '-0.31137', 'trailing1WeekReturn': '0.02286', 'trailing1MonthReturn': '2.67092', 'trailing3MonthReturn': '4.31986', 'trailing6MonthReturn': '-3.40218', 'trailingYearToDateReturn': '7.15929', 'trailing1YearReturn': '-5.73343', 'trailing3YearReturn': '7.73707', 'trailing5YearReturn': '9.84826', 'trailing10YearReturn': '12.16770', 'trailing15YearReturn': '9.84099'}, {'name': '+/-Credit Services', 'trailing1DayReturn': '0.22316', 'trailing1WeekReturn': '0.40838', 'trailing1MonthReturn': '-0.42389', 'trailing3MonthReturn': '-0.04058', 'trailing6MonthReturn': '4.06680', 'trailingYearToDateReturn': '-0.95943', 'trailing1YearReturn': '8.83606', 'trailing3YearReturn': '1.98577', 'trailing5YearReturn': '3.89050', 'trailing10YearReturn': '4.60404', 'trailing15YearReturn': None}, {'name': '+/-Morningstar US Market TR USD', 'trailing1DayReturn': '-0.69381', 'trailing1WeekReturn': '-1.62497', 'trailing1MonthReturn': '-2.21801', 'trailing3MonthReturn': '1.85105', 'trailing6MonthReturn': '8.03578', 'trailingYearToDateReturn': '0.66223', 'trailing1YearReturn': '6.00056', 'trailing3YearReturn': '-5.02918', 'trailing5YearReturn': '3.68617', 'trailing10YearReturn': '7.22862', 'trailing15YearReturn': None}]} 
                    
        """
        return self.GetData("trailingTotalReturns")
    
    def transactionHistory(self):
        """
        This function retrieves the transaction of key people.

        Returns:
            list o dict of transaction of key people

        Examples:
            >>> Stock("visa", exchange="nyse").transactionHistory()

        [{'date': '2023-02-03T06:00:00.000', 'tc': 'S', 'share': 61721.0, 'value': 14190997.2457, 'price': 229.9217, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2023-02-03T06:00:00.000', 'tc': 'M', 'share': 61721.0, 'value': 3310405.835, 'price': 53.635, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2023-01-24T06:00:00.000', 'tc': 'A', 'share': 1050.0, 'value': 0, 'price': 0, 'name': 'Maynard G. Webb', 'title': 'Independent Director', 'personId': 'PS000025A9', 'currency': 'USD'}, {'date': '2023-01-24T06:00:00.000', 'tc': 'A', 'share': 1050.0, 'value': 0.0, 'price': 0.0, 'name': 'Kermit R. Crawford', 'title': 'Independent Director', 'personId': 'PS0000211V', 'currency': 'USD'}, {'date': '2023-01-24T06:00:00.000', 'tc': 'A', 'share': 1050.0, 'value': 0.0, 'price': 0.0, 'name': 'Denise M. Morrison', 'title': 'Independent Director', 'personId': 'PS00001XZV', 'currency': 'USD'}, {'date': '2023-01-24T06:00:00.000', 'tc': 'A', 'share': 1050.0, 'value': 0.0, 'price': 0.0, 'name': 'John F. Lundgren', 'title': 'Lead 
        Independent Director', 'personId': 'PS00000QQ3', 'currency': 'USD'}, {'date': '2023-01-24T06:00:00.000', 'tc': 'A', 'share': 1050.0, 'value': 0.0, 'price': 0.0, 'name': 'Lloyd A. Carney', 'title': 'Independent Director', 'personId': 'PS00000X5N', 'currency': 'USD'}, {'date': '2023-01-24T06:00:00.000', 'tc': 'A', 'share': 1050.0, 'value': 0.0, 'price': 0.0, 'name': 'Teri L. List-Stoll', 'title': 'Independent Director', 'personId': 'PS00007T2K', 'currency': 'USD'}, {'date': '2023-01-24T06:00:00.000', 'tc': 'A', 'share': 1050.0, 'value': 0.0, 'price': 0.0, 'name': 'Ramon L. Laguarta', 'title': 'Independent Director', 'personId': 'PS0000A238', 'currency': 'USD'}, {'date': '2023-01-24T06:00:00.000', 'tc': 'A', 'share': 1050.0, 'value': 0, 'price': 0, 'name': 'Linda J. Rendle', 'title': 'Independent Director', 'personId': 'PS0000AINZ', 'currency': 'USD'}, {'date': '2023-01-15T06:00:00.000', 'tc': 'M', 'share': 449.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of 
        the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2023-01-15T06:00:00.000', 'tc': 'M', 'share': 576.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2023-01-09T06:00:00.000', 'tc': 'M', 'share': 58197.0, 'value': 12803340.0, 'price': 220.0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2023-01-09T06:00:00.000', 'tc': 'S', 'share': 58197.0, 'value': 3635275.605, 'price': 62.465, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-12-02T06:00:00.000', 'tc': 'M', 'share': 10000.0, 'value': 536400.0, 'price': 53.64, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-12-02T06:00:00.000', 'tc': 'S', 'share': 15627.0, 'value': 3391059.0, 'price': 
        217.0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-12-02T06:00:00.000', 'tc': 'S', 'share': 10000.0, 'value': 2161500.0, 'price': 216.15, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-12-01T06:00:00.000', 'tc': 'S', 'share': 40457.0, 'value': 8777409.1205, 'price': 216.9565, 'name': 
        'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'M', 'share': 56566.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'F', 'share': 28819.0, 'value': 6253723.0, 'price': 217.0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'S', 'share': 
        1060.0, 'value': 221695.926, 'price': 209.1471, 'name': 'Lloyd A. Carney', 'title': 'Independent Director', 'personId': 'PS00000X5N', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'M', 'share': 26346.0, 'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'F', 'share': 13063.0, 'value': 2834671.0, 'price': 217.0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'A', 'share': 26346.0, 'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'A', 
        'share': 56566.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'F', 'share': 15368.0, 'value': 3334856.0, 'price': 217.0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'M', 'share': 30995.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'A', 'share': 30995.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'F', 'share': 6322.0, 'value': 1371874.0, 'price': 217.0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'M', 'share': 12398.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'A', 'share': 12398.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'F', 'share': 16136.0, 'value': 3501512.0, 'price': 217.0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'M', 'share': 32545.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'A', 'share': 32545.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'F', 'share': 4946.0, 'value': 1073282.0, 'price': 217.0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'M', 'share': 9686.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-11-30T06:00:00.000', 'tc': 'A', 'share': 9686.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-11-28T06:00:00.000', 'tc': 'G', 'share': 23146.0, 'value': 0.0, 'price': 0.0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 8321.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 8230.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 8837.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'F', 'share': 12547.0, 'value': 2644907.6, 'price': 210.8, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of 
        the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 25498.0, 'value': 0, 'price': 
        0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 93819.0, 'value': 0.0, 'price': 0.0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 4215.0, 'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 4356.0, 'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'F', 'share': 5982.0, 'value': 1261005.6, 'price': 210.8, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 3876.0, 'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 10674.0, 'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 39273.0, 'value': 0.0, 'price': 0.0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 1824.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'F', 'share': 2753.0, 'value': 580332.4, 'price': 210.8, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 2925.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 2027.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 8539.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 31419.0, 'value': 0.0, 'price': 0.0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 5538.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 5159.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 4788.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief 
        Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'F', 'share': 7678.0, 'value': 1618522.4, 'price': 210.8, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 16010.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 58910.0, 'value': 0.0, 'price': 0.0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 4898.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'F', 'share': 7040.0, 'value': 1484032.0, 'price': 210.8, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 5186.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 4560.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 14824.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 54546.0, 'value': 0.0, 'price': 0.0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 493.0, 'value': 0, 'price': 0, 'name': 'Peter Andreski', 'title': 
        'Senior Vice President, Global Corporate Controller and Chief Accounting Officer', 'personId': 'PS0000DTTA', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'F', 'share': 621.0, 'value': 130906.8, 'price': 210.8, 'name': 'Peter Andreski', 'title': 'Senior Vice President, Global Corporate Controller and Chief Accounting Officer', 'personId': 'PS0000DTTA', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 498.0, 'value': 0, 'price': 0, 'name': 'Peter Andreski', 'title': 'Senior Vice President, Global Corporate Controller and Chief Accounting Officer', 'personId': 'PS0000DTTA', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 482.0, 'value': 0, 'price': 0, 'name': 'Peter Andreski', 'title': 'Senior Vice President, Global Corporate Controller and Chief Accounting Officer', 'personId': 'PS0000DTTA', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 2182.0, 'value': 0.0, 'price': 0.0, 'name': 'Peter Andreski', 'title': 'Senior Vice President, Global Corporate Controller and Chief Accounting Officer', 'personId': 'PS0000DTTA', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 1779.0, 'value': 0, 'price': 0, 'name': 'Peter Andreski', 'title': 'Senior Vice President, 
        Global Corporate Controller and Chief Accounting Officer', 'personId': 'PS0000DTTA', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'F', 'share': 2318.0, 'value': 488634.4, 'price': 210.8, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 1576.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 1536.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 1425.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 4744.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 17455.0, 'value': 0.0, 'price': 0.0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 794.0, 'value': 0, 'price': 0, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 783.0, 'value': 0, 'price': 0, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'M', 'share': 829.0, 'value': 0, 'price': 0, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'F', 'share': 833.0, 'value': 175596.4, 'price': 210.8, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 2965.0, 'value': 0, 'price': 0, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2022-11-19T06:00:00.000', 'tc': 'A', 'share': 10909.0, 
        'value': 0.0, 'price': 0.0, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2022-10-15T05:00:00.000', 'tc': 'A', 'share': 411.0, 'value': 0.0, 'price': 0.0, 'name': 'Kermit R. Crawford', 'title': 'Independent Director', 'personId': 'PS0000211V', 'currency': 'USD'}, {'date': '2022-09-29T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 
        'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-09-29T05:00:00.000', 'tc': 'S', 'share': 6000.0, 'value': 1080000.0, 'price': 180.0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 
        'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-09-29T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-08-31T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-08-31T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1809000.0, 'price': 201.0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-08-31T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-07-29T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-07-29T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1908990.0, 'price': 212.11, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-07-29T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 
        80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-07-15T05:00:00.000', 'tc': 'F', 'share': 607.0, 'value': 127494.28, 'price': 210.04, 'name': 'Peter Andreski', 'title': 'Senior Vice President, Global Corporate Controller and Chief Accounting Officer', 'personId': 'PS0000DTTA', 'currency': 'USD'}, {'date': '2022-07-15T05:00:00.000', 'tc': 'M', 'share': 1754.0, 'value': 0, 'price': 0, 'name': 'Peter Andreski', 'title': 'Senior Vice President, Global Corporate Controller and Chief Accounting Officer', 'personId': 'PS0000DTTA', 'currency': 'USD'}, {'date': '2022-06-30T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-06-30T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1758330.0, 'price': 195.37, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-06-30T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-06-14T05:00:00.000', 'tc': 'G', 'share': 159696.0, 'value': 0.0, 'price': 0.0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-06-14T05:00:00.000', 'tc': 'G', 'share': 159696.0, 'value': 0.0, 'price': 0.0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-05-31T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-05-31T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1893420.0, 'price': 210.38, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-05-31T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-04-29T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-04-29T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1967580.0, 'price': 218.62, 'name': 'Alfred F. Kelly', 'title': 'Executive 
        Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-04-29T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-04-15T05:00:00.000', 'tc': 'A', 'share': 881.0, 'value': 0.0, 'price': 0.0, 'name': 'Teri L. List-Stoll', 'title': 'Independent Director', 'personId': 'PS00007T2K', 'currency': 'USD'}, {'date': '2022-04-01T05:00:00.000', 'tc': 'S', 'share': 6467.0, 'value': 1446344.55, 'price': 223.65, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-04-01T05:00:00.000', 'tc': 'M', 'share': 6467.0, 'value': 403961.155, 'price': 62.465, 'name': 'Ryan McInerney', 'title': 'Chief Executive 
        Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-03-31T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-03-31T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 2015190.0, 'price': 223.91, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-03-31T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 
        329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, 
        {'date': '2022-03-28T05:00:00.000', 'tc': 'S', 'share': 6466.0, 'value': 1422520.0, 'price': 220.0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-03-28T05:00:00.000', 'tc': 'M', 'share': 6466.0, 'value': 403898.69, 'price': 62.465, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-02-28T06:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-02-28T06:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1944000.0, 'price': 216.0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-02-28T06:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-02-08T06:00:00.000', 'tc': 'S', 'share': 6466.0, 'value': 1462027.26, 'price': 226.11, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 
        'currency': 'USD'}, {'date': '2022-02-08T06:00:00.000', 'tc': 'M', 'share': 6466.0, 'value': 403898.69, 'price': 62.465, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-01-31T06:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 326460.0, 'price': 108.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-01-31T06:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 2024100.0, 'price': 224.9, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-01-31T06:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-01-31T06:00:00.000', 'tc': 'F', 'share': 5872.0, 'value': 1328070.24, 'price': 226.17, 'name': 'Paul 
        D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-01-31T06:00:00.000', 'tc': 'M', 'share': 12739.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2022-01-25T06:00:00.000', 'tc': 'A', 'share': 1114.0, 'value': 0.0, 'price': 0.0, 'name': 'Denise M. Morrison', 'title': 'Independent Director', 'personId': 'PS00001XZV', 'currency': 'USD'}, {'date': '2022-01-25T06:00:00.000', 'tc': 'A', 'share': 1114.0, 'value': 0.0, 'price': 0.0, 'name': 'John F. Lundgren', 'title': 'Lead Independent Director', 'personId': 'PS00000QQ3', 'currency': 'USD'}, {'date': '2022-01-25T06:00:00.000', 'tc': 'A', 'share': 1114.0, 'value': 0, 'price': 0, 'name': 'Maynard G. Webb', 'title': 'Independent Director', 'personId': 'PS000025A9', 'currency': 'USD'}, {'date': '2022-01-25T06:00:00.000', 'tc': 'A', 'share': 1114.0, 'value': 0.0, 'price': 0.0, 'name': 'Lloyd A. Carney', 'title': 
        'Independent Director', 'personId': 'PS00000X5N', 'currency': 'USD'}, {'date': '2022-01-25T06:00:00.000', 'tc': 'A', 'share': 1114.0, 'value': 0.0, 'price': 0.0, 'name': 'Ramon L. Laguarta', 'title': 'Independent Director', 'personId': 'PS0000A238', 'currency': 'USD'}, {'date': '2022-01-25T06:00:00.000', 'tc': 'A', 'share': 1114.0, 'value': 0, 'price': 0, 'name': 'Linda J. Rendle', 'title': 'Independent Director', 'personId': 'PS0000AINZ', 'currency': 'USD'}, {'date': '2022-01-15T06:00:00.000', 'tc': 'M', 'share': 449.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2022-01-15T06:00:00.000', 'tc': 'M', 'share': 576.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 
        'USD'}, {'date': '2022-01-03T06:00:00.000', 'tc': 'S', 'share': 7109.0, 'value': 1563980.0, 'price': 220.0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2022-01-03T06:00:00.000', 'tc': 'S', 'share': 11194.0, 'value': 2434918.88, 'price': 217.52, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2022-01-03T06:00:00.000', 'tc': 'M', 'share': 11194.0, 'value': 552395.915, 'price': 49.3475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-12-09T06:00:00.000', 'tc': 'M', 'share': 11193.0, 'value': 552346.5675, 'price': 49.3475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-12-09T06:00:00.000', 'tc': 'S', 'share': 11193.0, 'value': 2350530.0, 'price': 210.0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-12-01T06:00:00.000', 'tc': 'J', 'share': 7290.0, 'value': 0.0, 'price': 0.0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-12-01T06:00:00.000', 'tc': 'J', 'share': 7290.0, 'value': 0.0, 'price': 0.0, 'name': 'Ryan McInerney', 
        'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'F', 'share': 37064.0, 'value': 7181891.28, 'price': 193.77, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'M', 'share': 70467.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1757430.0, 'price': 195.27, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 
        'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'F', 'share': 13165.0, 'value': 2550982.05, 'price': 193.77, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'M', 'share': 26553.0, 'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'A', 'share': 26553.0, 'value': 0, 'price': 0, 'name': 'Vasant M. 
        Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'A', 'share': 70467.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'F', 'share': 13874.0, 'value': 2688364.98, 'price': 193.77, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'M', 'share': 27983.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'A', 'share': 27983.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'F', 'share': 14786.0, 'value': 2865083.22, 'price': 193.77, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'M', 'share': 29821.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 
        'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'A', 'share': 29821.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'M', 'share': 15523.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'F', 'share': 8165.0, 'value': 1582132.05, 'price': 193.77, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-11-30T06:00:00.000', 'tc': 'A', 'share': 15523.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 10667.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 8321.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 8230.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'F', 'share': 13871.0, 'value': 2786129.06, 'price': 200.86, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 26511.0, 'value': 0, 'price': 0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 123387.0, 'value': 0.0, 'price': 0.0, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 4020.0, 'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'F', 'share': 5820.0, 'value': 1169005.2, 'price': 200.86, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 4215.0, 'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 3876.0, 'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 60824.0, 'value': 0.0, 'price': 0.0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 13069.0, 
        'value': 0, 'price': 0, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 
        'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'F', 'share': 6584.0, 'value': 1322462.24, 'price': 200.86, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 4898.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 4559.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 4236.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'F', 'share': 7171.0, 'value': 1440367.06, 'price': 200.86, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 5159.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 4787.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': 
        '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 4515.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 16616.0, 'value': 0, 'price': 0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 77334.0, 'value': 0.0, 'price': 0.0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'F', 'share': 2599.0, 'value': 522035.14, 'price': 200.86, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 2027.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 1824.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 2350.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 40839.0, 'value': 0.0, 'price': 0.0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 8775.0, 'value': 0, 'price': 0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 15558.0, 'value': 0, 'price': 0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 72410.0, 'value': 0.0, 'price': 0.0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 1425.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 1535.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'F', 'share': 1574.0, 'value': 316153.64, 'price': 200.86, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 1076.0, 'value': 0, 'price': 0, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'F', 'share': 928.0, 'value': 186398.08, 'price': 200.86, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 783.0, 'value': 0, 'price': 0, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, 
        {'date': '2021-11-19T06:00:00.000', 'tc': 'M', 'share': 793.0, 'value': 0, 'price': 0, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 2489.0, 'value': 0, 'price': 0, 'name': 
        'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 
        'share': 11586.0, 'value': 0.0, 'price': 0.0, 'name': 'Julie B. Rottenberg', 'title': 'General Counsel', 'personId': 'PS0000CUNJ', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 22013.0, 'value': 0.0, 'price': 0.0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2021-11-19T06:00:00.000', 'tc': 'A', 'share': 4730.0, 'value': 0, 'price': 
        0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2021-11-01T05:00:00.000', 'tc': 'S', 'share': 11193.0, 'value': 2377840.92, 'price': 212.44, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-11-01T05:00:00.000', 'tc': 'M', 'share': 11193.0, 'value': 552346.5675, 'price': 49.3475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-10-29T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-10-29T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1877130.0, 'price': 208.57, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-10-29T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-10-15T05:00:00.000', 'tc': 'F', 'share': 1338.0, 'value': 309064.62, 'price': 230.99, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2021-10-15T05:00:00.000', 'tc': 'M', 'share': 2518.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2021-10-01T05:00:00.000', 'tc': 'M', 'share': 8778.0, 'value': 395426.955, 'price': 45.0475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-10-01T05:00:00.000', 'tc': 'M', 'share': 2416.0, 'value': 119223.56, 'price': 49.3475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-10-01T05:00:00.000', 'tc': 'S', 'share': 11194.0, 'value': 2509358.98, 'price': 224.17, 'name': 
        'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-09-30T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 2048220.0, 'price': 227.58, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-09-30T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-09-30T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-09-01T05:00:00.000', 'tc': 'S', 'share': 11193.0, 'value': 2564316.3, 'price': 229.1, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-09-01T05:00:00.000', 'tc': 'M', 'share': 11193.0, 'value': 504216.6675, 'price': 45.0475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-08-31T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-08-31T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 2078190.0, 'price': 230.91, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-08-31T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': 
        '2021-08-02T05:00:00.000', 'tc': 'S', 'share': 11193.0, 'value': 2756679.198, 'price': 246.286, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-08-02T05:00:00.000', 'tc': 'M', 'share': 11193.0, 'value': 504216.6675, 'price': 45.0475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-07-30T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 2224890.0, 'price': 247.21, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-07-30T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-07-30T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-07-28T05:00:00.000', 'tc': 'S', 'share': 2091.0, 'value': 520052.61, 'price': 248.71, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-07-06T05:00:00.000', 'tc': 'S', 'share': 42336.0, 'value': 10160640.0, 'price': 240.0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-07-06T05:00:00.000', 'tc': 'M', 'share': 42336.0, 'value': 3393230.4, 'price': 80.15, 'name': 'Rajat Taneja', 'title': 'President, Technology', 
        'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-07-01T05:00:00.000', 'tc': 'S', 'share': 11194.0, 'value': 2621634.8, 'price': 234.2, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-07-01T05:00:00.000', 'tc': 'M', 'share': 11194.0, 'value': 504261.715, 'price': 45.0475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer 
        and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-06-30T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 
        'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-06-30T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 2115990.0, 'price': 235.11, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-06-30T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-06-29T05:00:00.000', 'tc': 'S', 'share': 6275.0, 'value': 1476695.75, 'price': 235.33, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-06-02T05:00:00.000', 'tc': 'G', 'share': 703.0, 'value': 0.0, 'price': 0.0, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-06-02T05:00:00.000', 'tc': 'S', 'share': 12735.0, 'value': 2948291.3115, 'price': 231.5109, 'name': 'Kelly Mahon Tullier', 'title': 'Vice Chairman, Chief People and Administrative Officer and Corporate Secretary', 'personId': 'PS00008OTS', 'currency': 'USD'}, {'date': '2021-06-01T05:00:00.000', 'tc': 'S', 'share': 11193.0, 'value': 2568780.0684, 'price': 229.4988, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-06-01T05:00:00.000', 'tc': 'M', 'share': 11193.0, 'value': 504216.6675, 'price': 45.0475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-05-28T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-05-28T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 2048310.0, 'price': 227.59, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-05-28T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-05-17T05:00:00.000', 'tc': 'S', 'share': 1300.0, 'value': 293956.13, 'price': 226.1201, 'name': 'Lloyd A. Carney', 'title': 'Independent Director', 'personId': 'PS00000X5N', 'currency': 'USD'}, {'date': '2021-05-03T05:00:00.000', 'tc': 'S', 'share': 10000.0, 'value': 2322296.0, 'price': 232.2296, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-05-03T05:00:00.000', 'tc': 'S', 'share': 11193.0, 'value': 2619721.65, 'price': 234.05, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-05-03T05:00:00.000', 'tc': 'M', 'share': 11193.0, 'value': 504216.6675, 'price': 45.0475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-04-30T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-04-30T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 2117880.0, 'price': 235.32, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of 
        the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-04-30T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-04-30T05:00:00.000', 'tc': 'S', 'share': 25000.0, 'value': 5856567.5, 'price': 234.2627, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-04-30T05:00:00.000', 'tc': 'M', 'share': 15000.0, 'value': 1202250.0, 'price': 80.15, 'name': 'Vasant M. Prabhu', 'title': 'Vice Chairman and Chief Financial Officer', 'personId': 'PS00001RQY', 'currency': 'USD'}, {'date': '2021-04-22T05:00:00.000', 'tc': 'S', 'share': 31750.0, 'value': 7302500.0, 'price': 230.0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-04-22T05:00:00.000', 'tc': 'M', 'share': 31750.0, 'value': 2544762.5, 'price': 80.15, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-04-01T05:00:00.000', 'tc': 'S', 'share': 11194.0, 'value': 2394060.78, 'price': 213.87, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-04-01T05:00:00.000', 'tc': 'M', 'share': 11194.0, 'value': 504261.715, 'price': 45.0475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-03-31T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-03-31T05:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-03-31T05:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1912950.0, 'price': 212.55, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-03-08T06:00:00.000', 'tc': 'S', 'share': 31750.0, 'value': 7080250.0, 'price': 223.0, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-03-08T06:00:00.000', 'tc': 'M', 'share': 31750.0, 'value': 2544762.5, 'price': 80.15, 'name': 'Rajat Taneja', 'title': 'President, Technology', 'personId': 'PS00007VT2', 'currency': 'USD'}, {'date': '2021-03-01T06:00:00.000', 'tc': 'S', 'share': 11193.0, 'value': 2407278.51, 'price': 215.07, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, 
        {'date': '2021-03-01T06:00:00.000', 'tc': 'M', 'share': 11193.0, 'value': 504216.6675, 'price': 45.0475, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-02-26T06:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-02-26T06:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1926450.0, 'price': 214.05, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-02-26T06:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-02-08T06:00:00.000', 'tc': 'M', 'share': 11193.0, 'value': 504216.6675, 'price': 45.0475, 'name': 'Ryan McInerney', 
        'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-02-08T06:00:00.000', 'tc': 'S', 'share': 11193.0, 'value': 2350530.0, 'price': 210.0, 'name': 'Ryan McInerney', 'title': 'Chief Executive Officer and Director', 'personId': 'PS00008TV4', 'currency': 'USD'}, {'date': '2021-01-31T06:00:00.000', 'tc': 'F', 'share': 5761.0, 'value': 1113313.25, 'price': 193.25, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2021-01-31T06:00:00.000', 'tc': 'M', 'share': 12738.0, 'value': 0, 'price': 0, 'name': 'Paul D. Fabara', 'title': 'Chief Risk Officer', 'personId': 'PS0000A8VE', 'currency': 'USD'}, {'date': '2021-01-29T06:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 242460.0, 'price': 80.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-01-29T06:00:00.000', 'tc': 'S', 'share': 9000.0, 'value': 1779570.0, 'price': 197.73, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-01-29T06:00:00.000', 'tc': 'M', 'share': 3000.0, 'value': 329460.0, 'price': 109.82, 'name': 'Alfred F. Kelly', 'title': 'Executive Chairman of the Board', 'personId': 'PS0000018X', 'currency': 'USD'}, {'date': '2021-01-26T06:00:00.000', 'tc': 'A', 'share': 1064.0, 'value': 0.0, 'price': 0.0, 'name': 'Lloyd A. Carney', 'title': 'Independent Director', 'personId': 'PS00000X5N', 'currency': 'USD'}, {'date': '2021-01-26T06:00:00.000', 'tc': 'A', 'share': 1064.0, 'value': 0, 'price': 0, 'name': 'Maynard G. Webb', 'title': 'Independent Director', 'personId': 'PS000025A9', 'currency': 'USD'}, {'date': '2021-01-26T06:00:00.000', 'tc': 'A', 'share': 1064.0, 'value': 0.0, 'price': 0.0, 'name': 'John F. Lundgren', 'title': 'Lead Independent Director', 'personId': 'PS00000QQ3', 'currency': 'USD'}, {'date': '2021-01-26T06:00:00.000', 'tc': 'A', 'share': 1064.0, 'value': 0.0, 'price': 0.0, 'name': 'Denise M. Morrison', 'title': 'Independent Director', 'personId': 'PS00001XZV', 'currency': 'USD'}, {'date': '2021-01-26T06:00:00.000', 'tc': 'A', 'share': 1064.0, 'value': 0.0, 'price': 0.0, 'name': 'Ramon L. Laguarta', 'title': 'Independent Director', 'personId': 'PS0000A238', 'currency': 'USD'}, {'date': '2021-01-26T06:00:00.000', 'tc': 'A', 'share': 1064.0, 'value': 0, 'price': 0, 'name': 'Linda J. Rendle', 'title': 'Independent Director', 'personId': 'PS0000AINZ', 'currency': 'USD'}]
                
        """
        return self.GetData("insiders/transactionHistory")
    
    
    def transactionSummary(self):
        """
        This function retrieves the summuary of transactions of key people.

        Returns:
            list of dict with transactions

        Examples:
            >>> Stock("visa", exchange="nyse").transactionSummary()

        [{'buy': 0, 'acquisition': 0, 'optionExercise': 12.466, 'sell': 15.466, 'disposition': 0, 'date': '2022-03'}, {'buy': 0, 'acquisition': 0.881, 'optionExercise': 12.467, 'sell': 15.467, 'disposition': 0, 'date': '2022-04'}, {'buy': 0, 'acquisition': 0, 'optionExercise': 6, 'sell': 9, 'disposition': 0, 'date': '2022-05'}, {'buy': 0, 'acquisition': 0, 'optionExercise': 6, 'sell': 9, 'disposition': 159.696, 'date': '2022-06'}, {'buy': 0, 'acquisition': 0, 'optionExercise': 7.754, 'sell': 9, 'disposition': 0.607, 'date': '2022-07'}, {'buy': 0, 'acquisition': 0, 'optionExercise': 6, 'sell': 9, 'disposition': 0, 'date': '2022-08'}, {'buy': 0, 'acquisition': 0, 'optionExercise': 6, 'sell': 6, 'disposition': 0, 'date': 
        '2022-09'}, {'buy': 0, 'acquisition': 0.411, 'optionExercise': 0, 'sell': 0, 'disposition': 0, 'date': '2022-10'}, {'buy': 0, 'acquisition': 562.082, 'optionExercise': 251.692, 'sell': 1.06, 'disposition': 147.572, 'date': '2022-11'}, {'buy': 0, 'acquisition': 0, 'optionExercise': 10, 'sell': 66.084, 'disposition': 0, 'date': '2022-12'}, {'buy': 0, 'acquisition': 8.4, 'optionExercise': 59.222, 'sell': 58.197, 'disposition': 0, 'date': '2023-01'}, {'buy': 0, 'acquisition': 0, 'optionExercise': 61.721, 'sell': 61.721, 'disposition': 0, 'date': '2023-02'}]
                
        """
        return self.GetData("insiders/transactionChart")
    
    def valuation(self):
        """
        This function retrieves the valution of the stock.

        Returns:
            dict with valuation

        Examples:
            >>> Stock("visa", exchange="nyse").valuation()

        {'Collapsed': {'rows': [{'label': 'Price/Sales', 'salDataId': 'price.sales.label', 'isQuantitative': False, 'datum': ['12.4026', '13.0254', '13.7277', '12.4878', '14.8751', '14.9104', '18.5798', '22.2575', '19.6707', '15.1407', '15.7133', '18.094', '2.2514'], 'subLevel': ''}, {'label': 'Price/Earnings', 'salDataId': 'price.earnings.label', 'isQuantitative': False, 'datum': ['29.3386', '30.4176', '30.0581', '31.4597', '40.7214', 
        '29.8507', '35.3195', '44.7301', '38.492', '29.68', '31.2235', '36.0285', '19.5046'], 'subLevel': ''}, {'label': 'Price/Cash Flow', 'salDataId': 'price.cash.flow.label', 'isQuantitative': False, 'datum': ['48.3382', '22.963', '28.9399', '33.7891', '29.6566', '24.1712', '33.394', '46.5744', '31.1395', '23.5437', '25.2469', '31.8568', '14.7124'], 'subLevel': ''}, {'label': 'Price/Book', 'salDataId': 'price.book.label', 'isQuantitative': False, 'datum': ['5.2157', '5.8667', '6.225', '6.7248', '9.5416', '10.3016', '14.0113', '15.2222', '13.3573', '12.2797', '12.7255', '12.8212', '3.5905'], 'subLevel': ''}], 'columnDefs': ['Calendar', '2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', 'Current', '5-Yr', 'Index'], 'columnDefs_labels': ['tabular.data.label.column.year', '2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', 'valuation.headers.current', 'valuation.headers.fiveyear', 'valuation.headers.index'], 'userType': None, 'footer': {'asOfLabel': 'As of', 'asOfDate': '2023-02-20T00:00:00.000', 'indexLabel': 'Index:', 'indexName': 'Morningstar US Market TR USD', 'enterpriseValueCurrency': 'USD'}}, 'Expanded': {'rows': [{'label': 'Price/Forward Earnings', 'salDataId': 'price.forward.earnings.label', 'isQuantitative': False, 'datum': ['24.0385', '24.5098', '26.8097', '23.6407', '28.169', '24.6914', '30.1205', '39.8406', '30.3951', '24.9377', '26.3852', '30.1163', None], 'subLevel': ''}, {'label': 'PEG Ratio', 'salDataId': 'peg.ratio.label', 'isQuantitative': False, 'datum': ['1.4517', '1.4198', '76.6975', '1.6673', '1.558', '1.3047', '2.0559', '3.6943', '1.6947', '1.585', '1.6774', '1.8345', None], 'subLevel': ''}, {'label': 'Earnings Yield %', 'salDataId': 'earnings.yield.label', 'isQuantitative': False, 'datum': ['3.41', '3.29', '3.33', '3.18', '2.46', '3.35', '2.83', '2.24', '2.6', '3.37', 
        '3.2', '2.83', None], 'subLevel': ''}, {'label': 'Enterprise Value (Bil)', 'salDataId': 'enterprise.value.label', 'isQuantitative': False, 'datum': ['135.89', '156.87', '179.75', '195.54', '270.31', '304.36', '419.56', '482.89', '466.49', '443.88', '474.24', '410.99', None], 'subLevel': 
        '', 'orderOfMagnitude': 'Bil'}, {'label': 'Enterprise Value/EBIT', 'salDataId': 'enterprise.value.ebit.label', 'isQuantitative': False, 'datum': ['18.7643', '19.2483', '19.8008', '23.1712', '22.0535', '22.6827', '27.2421', '33.7544', '28.1427', '23.7702', '25.2835', '26.9025', None], 'subLevel': ''}, {'label': 'Enterprise Value/EBITDA', 'salDataId': 'enterprise.value.ebitda.label', 'isQuantitative': False, 'datum': ['17.7891', '18.273', '18.7789', '21.8703', '21.0965', '21.6918', '26.1292', '32.0368', '26.8408', '22.7225', '24.1382', '25.682', None], 'subLevel': ''}], 'columnDefs': ['Calendar', '2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', 'Current', '5-Yr', 'Index'], 'columnDefs_labels': ['tabular.data.label.column.year', '2013', '2014', '2015', '2016', '2017', '2018', '2019', '2020', '2021', '2022', 'valuation.headers.current', 'valuation.headers.fiveyear', 'valuation.headers.index']}}   
                
        """
        return self.GetData("valuation", url_suffixe='')
